@@ -74,9 +74,9 @@ void field_overlay_source::build_summary_lines(const listview_curses &lv)
                 last_line = lss.find_line(lss.at(lv.get_bottom()));
                 last_time = "Last message: " ANSI_BOLD_START + precise_time_ago(
                     last_line->get_timeval(), true) + ANSI_NORM;
-                str2reltime(last_line->get_time_in_millis() -
-                            first_line->get_time_in_millis(),
-                            time_span);
+                duration2str(last_line->get_time_in_millis() -
+                             first_line->get_time_in_millis(),
+                             time_span);
 
                 time_t local_now = convert_log_time_to_local(now);
                 time_t five_minutes_ago = local_now - (5 * 60 * 60);
@@ -89,7 +89,7 @@ void field_overlay_source::build_summary_lines(const listview_curses &lv)
                     bm[&logfile_sub_source::BM_ERRORS];
 
                 if (now > last_line->get_time() && from_five_min_ago != -1) {
-                    bookmark_vector<vis_line_t>::iterator five_min_lower =
+                    auto five_min_lower =
                         lower_bound(error_bookmarks.begin(),
                                     error_bookmarks.end(),
                                     from_five_min_ago);
@@ -105,7 +105,7 @@ void field_overlay_source::build_summary_lines(const listview_curses &lv)
                         error_rate = error_count / time_diff;
 
                         if (from_ten_secs_ago != -1) {
-                            bookmark_vector<vis_line_t>::iterator ten_sec_lower =
+                            auto ten_sec_lower =
                                 lower_bound(error_bookmarks.begin(),
                                             error_bookmarks.end(),
                                             from_ten_secs_ago);
@@ -129,7 +129,7 @@ void field_overlay_source::build_summary_lines(const listview_curses &lv)
                 }
             }
 
-            this->fos_summary_lines.push_back(attr_line_t());
+            this->fos_summary_lines.emplace_back();
             attr_line_t &sum_line = this->fos_summary_lines.back();
             if (tss.empty()) {
                 sum_line.with_ansi_string(
@@ -207,7 +207,7 @@ void field_overlay_source::build_field_lines(const listview_curses &lv)
 
     content_line_t cl = lss.at(lv.get_top());
     std::shared_ptr<logfile> file = lss.find(cl);
-    logfile::iterator ll = file->begin() + cl;
+    auto ll = file->begin() + cl;
     log_format *format = file->get_format();
     bool display = false;
 
@@ -332,21 +332,17 @@ void field_overlay_source::build_field_lines(const listview_curses &lv)
     this->fos_known_key_size = 0;
     this->fos_unknown_key_size = 0;
 
-    for (std::vector<logline_value>::iterator iter =
-            this->fos_log_helper.ldh_line_values.begin();
-         iter != this->fos_log_helper.ldh_line_values.end();
-         ++iter) {
-        int this_key_size = iter->lv_name.size();
+    for (auto & ldh_line_value : this->fos_log_helper.ldh_line_values) {
+        int this_key_size = ldh_line_value.lv_name.size();
 
-        if (iter->lv_kind == logline_value::VALUE_STRUCT) {
+        if (ldh_line_value.lv_kind == logline_value::VALUE_STRUCT) {
             this_key_size += 9;
         }
         this->fos_known_key_size = max(
             this->fos_known_key_size, this_key_size);
     }
 
-    for (data_parser::element_list_t::iterator iter =
-            this->fos_log_helper.ldh_parser->dp_pairs.begin();
+    for (auto iter = this->fos_log_helper.ldh_parser->dp_pairs.begin();
          iter != this->fos_log_helper.ldh_parser->dp_pairs.end();
          ++iter) {
         std::string colname = this->fos_log_helper.ldh_parser->get_element_string(
@@ -375,8 +371,7 @@ void field_overlay_source::build_field_lines(const listview_curses &lv)
 
     const log_format *last_format = NULL;
 
-    for (size_t lpc = 0; lpc < this->fos_log_helper.ldh_line_values.size(); lpc++) {
-        logline_value &lv = this->fos_log_helper.ldh_line_values[lpc];
+    for (auto & lv : this->fos_log_helper.ldh_line_values) {
         string format_name = lv.lv_format->get_name().to_string();
         attr_line_t al;
         string str, value_str = lv.to_string();
@@ -527,7 +522,7 @@ void field_overlay_source::build_meta_line(const listview_curses &lv,
             const auto *tc = dynamic_cast<const textview_curses *>(&lv);
             if (tc) {
                 const textview_curses::highlight_map_t &hm = tc->get_highlights();
-                auto hl_iter = hm.find("$search");
+                auto hl_iter = hm.find({highlight_source_t::PREVIEW, "search"});
 
                 if (hl_iter != hm.end()) {
                     hl_iter->second.annotate(al, 2);
