@@ -136,7 +136,7 @@ string sql_readlink(const char *path)
     struct stat st;
 
     if (lstat(path, &st) == -1) {
-        throw sqlite_func_error("unable to stat path: %s -- %s", path, strerror(errno));
+        throw sqlite_func_error("unable to stat path: {} -- {}", path, strerror(errno));
     }
 
     char buf[st.st_size];
@@ -147,7 +147,7 @@ string sql_readlink(const char *path)
         if (errno == EINVAL) {
             return path;
         }
-        throw sqlite_func_error("unable to read link: %s -- %s", path, strerror(errno));
+        throw sqlite_func_error("unable to read link: {} -- {}", path, strerror(errno));
     }
 
     return string(buf, rc);
@@ -159,7 +159,7 @@ string sql_realpath(const char *path)
     char resolved_path[PATH_MAX];
 
     if (realpath(path, resolved_path) == NULL) {
-        throw sqlite_func_error("Could not get real path for %s -- %s",
+        throw sqlite_func_error("Could not get real path for {} -- {}",
                                 path, strerror(errno));
     }
 
@@ -178,13 +178,30 @@ int fs_extension_functions(struct FuncDef **basic_funcs,
                 .sql_function()
                 .with_parameter({"path", "The path"})
                 .with_tags({"filename"})
-                .with_example({"SELECT basename('foobar')"})
-                .with_example({"SELECT basename('foo/bar')"})
-                .with_example({"SELECT basename('foo/bar/')"})
-                .with_example({"SELECT basename('')"})
-                .with_example({"SELECT basename('foo\\bar')"})
-                .with_example({"SELECT basename('foobar')"})
-                .with_example({"SELECT basename('/')"})
+                .with_example({
+                    "To get the base of a plain file name",
+                    "SELECT basename('foobar')"
+                })
+                .with_example({
+                    "To get the base of a path",
+                    "SELECT basename('foo/bar')"
+                })
+                .with_example({
+                    "To get the base of a directory",
+                    "SELECT basename('foo/bar/')"
+                })
+                .with_example({
+                    "To get the base of an empty string",
+                    "SELECT basename('')"
+                })
+                .with_example({
+                    "To get the base of a Windows path",
+                    "SELECT basename('foo\\bar')"
+                })
+                .with_example({
+                    "To get the base of the root directory",
+                    "SELECT basename('/')"
+                })
         ),
 
         sqlite_func_adapter<decltype(&sql_dirname), sql_dirname>::builder(
@@ -193,11 +210,26 @@ int fs_extension_functions(struct FuncDef **basic_funcs,
                 .sql_function()
                 .with_parameter({"path", "The path"})
                 .with_tags({"filename"})
-                .with_example({"SELECT dirname('foo/bar')"})
-                .with_example({"SELECT dirname('/foo/bar')"})
-                .with_example({"SELECT dirname('/bar')"})
-                .with_example({"SELECT dirname('foo\\bar')"})
-                .with_example({"SELECT dirname('')"})
+                .with_example({
+                    "To get the directory of a relative file path",
+                    "SELECT dirname('foo/bar')"
+                })
+                .with_example({
+                    "To get the directory of an absolute file path",
+                    "SELECT dirname('/foo/bar')"
+                })
+                .with_example({
+                    "To get the directory of a file in the root directory",
+                    "SELECT dirname('/bar')"
+                })
+                .with_example({
+                    "To get the directory of a Windows path",
+                    "SELECT dirname('foo\\bar')"
+                })
+                .with_example({
+                    "To get the directory of an empty path",
+                    "SELECT dirname('')"
+                })
         ),
 
         sqlite_func_adapter<decltype(&sql_joinpath), sql_joinpath>::builder(
@@ -209,10 +241,22 @@ int fs_extension_functions(struct FuncDef **basic_funcs,
                     "an absolute path and any preceding elements will be ignored.")
                                     .one_or_more())
                 .with_tags({"filename"})
-                .with_example({"SELECT joinpath('foo', 'bar')"})
-                .with_example({"SELECT joinpath('', 'foo', 'bar')"})
-                .with_example({"SELECT joinpath('/', 'foo', 'bar')"})
-                .with_example({"SELECT joinpath('/', 'foo', '/bar')"})
+                .with_example({
+                    "To join a directory and file name into a relative path",
+                    "SELECT joinpath('foo', 'bar')"
+                })
+                .with_example({
+                    "To join an empty component with other names into a relative path",
+                    "SELECT joinpath('', 'foo', 'bar')"
+                })
+                .with_example({
+                    "To create an absolute path with two path components",
+                    "SELECT joinpath('/', 'foo', 'bar')"
+                })
+                .with_example({
+                    "To create an absolute path from a path component that starts with a forward slash",
+                    "SELECT joinpath('/', 'foo', '/bar')"
+                })
         ),
 
         sqlite_func_adapter<decltype(&sql_readlink), sql_readlink>::builder(
