@@ -32,6 +32,10 @@
 #include <numeric>
 #include <algorithm>
 
+#include <pcrecpp.h>
+
+#include "fmt/format.h"
+#include "lnav_util.hh"
 #include "ansi_scrubber.hh"
 #include "help_text_formatter.hh"
 #include "readline_highlighters.hh"
@@ -41,9 +45,42 @@ using namespace std;
 
 std::multimap<std::string, help_text *> help_text::TAGGED;
 
-void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out, bool synopsis_only)
+static vector<help_text *> get_related(const help_text &ht)
 {
-    static size_t body_indent = 2;
+    vector<help_text *> retval;
+
+    for (const auto &tag : ht.ht_tags) {
+        auto tagged = help_text::TAGGED.equal_range(tag);
+
+        for (auto tag_iter = tagged.first;
+             tag_iter != tagged.second;
+             ++tag_iter) {
+            if (tag_iter->second == &ht) {
+                continue;
+            }
+
+            help_text &related = *tag_iter->second;
+
+            if (!related.ht_opposites.empty() &&
+                find_if(related.ht_opposites.begin(),
+                        related.ht_opposites.end(),
+                        [&ht](const char *x) {
+                            return strcmp(x, ht.ht_name) == 0;
+                        }) == related.ht_opposites.end()) {
+                continue;
+            }
+
+            retval.push_back(&related);
+
+        }
+    }
+
+    return retval;
+}
+
+void format_help_text_for_term(const help_text &ht, size_t width, attr_line_t &out, bool synopsis_only)
+{
+    static const size_t body_indent = 2;
 
     view_colors &vc = view_colors::singleton();
     text_wrap_settings tws;
@@ -52,7 +89,7 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
     tws.with_width(width);
 
     switch (ht.ht_context) {
-        case HC_COMMAND: {
+        case help_context_t::HC_COMMAND: {
             out.append("Synopsis", &view_curses::VC_STYLE, A_UNDERLINE)
                 .append("\n")
                 .append(body_indent, ' ')
@@ -60,14 +97,14 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
                 .append(ht.ht_name, &view_curses::VC_STYLE, A_BOLD);
             for (auto &param : ht.ht_parameters) {
                 out.append(" ");
-                if (param.ht_nargs == HN_OPTIONAL) {
+                if (param.ht_nargs == help_nargs_t::HN_OPTIONAL) {
                     out.append("[");
                 }
                 out.append(param.ht_name, &view_curses::VC_STYLE, A_UNDERLINE);
-                if (param.ht_nargs == HN_OPTIONAL) {
+                if (param.ht_nargs == help_nargs_t::HN_OPTIONAL) {
                     out.append("]");
                 }
-                if (param.ht_nargs == HN_ONE_OR_MORE) {
+                if (param.ht_nargs == help_nargs_t::HN_ONE_OR_MORE) {
                     out.append("1", &view_curses::VC_STYLE, A_UNDERLINE);
                     out.append(" [");
                     out.append("...", &view_curses::VC_STYLE, A_UNDERLINE);
@@ -84,37 +121,62 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
                 .append("\n");
             break;
         }
-        case HC_SQL_FUNCTION:
-        case HC_SQL_TABLE_VALUED_FUNCTION: {
+        case help_context_t::HC_SQL_FUNCTION:
+        case help_context_t::HC_SQL_TABLE_VALUED_FUNCTION: {
+            size_t line_start = body_indent;
+            bool break_all = false;
             bool needs_comma = false;
 
-            out.append("Synopsis", &view_curses::VC_STYLE, A_UNDERLINE)
-                .append("\n")
-                .append(body_indent, ' ')
+            if (!synopsis_only) {
+                out.append("Synopsis", &view_curses::VC_STYLE, A_UNDERLINE)
+                    .append("\n");
+            }
+
+            line_start = out.length();
+            out.append(body_indent, ' ')
                 .append(ht.ht_name, &view_curses::VC_STYLE, A_BOLD)
                 .append("(");
             for (auto &param : ht.ht_parameters) {
+                if (!param.ht_flag_name && needs_comma) {
+                    out.append(", ");
+                }
+                if (break_all ||
+                    (int)(out.get_string().length() - line_start + 10) >=
+                    tws.tws_width) {
+                    out.append("\n");
+                    line_start = out.get_string().length();
+                    out.append(body_indent + strlen(ht.ht_name) + 1, ' ');
+                    break_all = true;
+                }
                 if (param.ht_flag_name) {
                     out.append(" ")
                         .append(param.ht_flag_name, &view_curses::VC_STYLE, A_BOLD)
                         .append(" ");
-                } else if (needs_comma) {
-                    out.append(", ");
+                }
+                if (param.ht_nargs == help_nargs_t::HN_OPTIONAL) {
+                    out.append("[");
                 }
                 out.append(param.ht_name, &view_curses::VC_STYLE, A_UNDERLINE);
-                if (param.ht_nargs == HN_ZERO_OR_MORE ||
-                    param.ht_nargs == HN_ONE_OR_MORE) {
+                if (param.ht_nargs == help_nargs_t::HN_OPTIONAL) {
+                    out.append("]");
+                }
+                if (param.ht_nargs == help_nargs_t::HN_ZERO_OR_MORE ||
+                    param.ht_nargs == help_nargs_t::HN_ONE_OR_MORE) {
                     out.append(", ...");
                 }
                 needs_comma = true;
             }
-            out.append(") -- ")
-                .append(attr_line_t::from_ansi_str(ht.ht_summary),
-                        &tws.with_indent(body_indent + 2))
+            out.append(") -- ");
+            if (break_all) {
+                out.append("\n")
+                    .append(body_indent + strlen(ht.ht_name) + 1, ' ');
+            }
+            out.append(attr_line_t::from_ansi_str(ht.ht_summary),
+                       &tws.with_indent(body_indent + 2))
                 .append("\n");
             break;
         }
-        case HC_SQL_KEYWORD: {
+        case help_context_t::HC_SQL_KEYWORD: {
             size_t line_start = body_indent;
             bool break_all = false;
 
@@ -133,8 +195,8 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
                     out.append(body_indent + strlen(ht.ht_name) + 1, ' ');
                     break_all = true;
                 }
-                if (param.ht_nargs == HN_ZERO_OR_MORE ||
-                    param.ht_nargs == HN_OPTIONAL) {
+                if (param.ht_nargs == help_nargs_t::HN_ZERO_OR_MORE ||
+                    param.ht_nargs == help_nargs_t::HN_OPTIONAL) {
                     if (!break_all) {
                         out.append(" ");
                     }
@@ -155,8 +217,8 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
                         .append(param.ht_name, &view_curses::VC_STYLE,
                                 A_UNDERLINE);
                     if (!param.ht_parameters.empty()) {
-                        if (param.ht_nargs == HN_ZERO_OR_MORE ||
-                            param.ht_nargs == HN_ONE_OR_MORE) {
+                        if (param.ht_nargs == help_nargs_t::HN_ZERO_OR_MORE ||
+                            param.ht_nargs == help_nargs_t::HN_ONE_OR_MORE) {
                             out.append("1", &view_curses::VC_STYLE, A_UNDERLINE);
                         }
                         if (param.ht_parameters[0].ht_flag_name) {
@@ -171,8 +233,8 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
                                    A_UNDERLINE);
                     }
                 }
-                if (param.ht_nargs == HN_ZERO_OR_MORE ||
-                    param.ht_nargs == HN_ONE_OR_MORE) {
+                if (param.ht_nargs == help_nargs_t::HN_ZERO_OR_MORE ||
+                    param.ht_nargs == help_nargs_t::HN_ONE_OR_MORE) {
                     bool needs_comma = param.ht_parameters.empty() ||
                                        !param.ht_flag_name;
 
@@ -209,8 +271,8 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
                        .append(param.ht_group_end, &view_curses::VC_STYLE,
                                A_BOLD);
                 }
-                if (param.ht_nargs == HN_ZERO_OR_MORE ||
-                    param.ht_nargs == HN_OPTIONAL) {
+                if (param.ht_nargs == help_nargs_t::HN_ZERO_OR_MORE ||
+                    param.ht_nargs == help_nargs_t::HN_OPTIONAL) {
                     out.append("]");
                 }
             }
@@ -287,46 +349,25 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
         }
     }
     if (!synopsis_only && !ht.ht_tags.empty()) {
-        vector<string> tags;
+        auto related_help = get_related(ht);
+        auto related_refs = vector<string>();
 
-        for (const auto &tag : ht.ht_tags) {
-            auto tagged = help_text::TAGGED.equal_range(tag);
-
-            for (auto tag_iter = tagged.first;
-                 tag_iter != tagged.second;
-                 ++tag_iter) {
-                if (tag_iter->second == &ht) {
-                    continue;
-                }
-
-                help_text &related = *tag_iter->second;
-
-                if (!related.ht_opposites.empty() &&
-                    find_if(related.ht_opposites.begin(),
-                            related.ht_opposites.end(),
-                            [&ht](const char *x) {
-                                return strcmp(x, ht.ht_name) == 0;
-                            }) == related.ht_opposites.end()) {
-                    continue;
-                }
-
-                string name = related.ht_name;
-                switch (related.ht_context) {
-                    case HC_COMMAND:
-                        name = ":" + name;
-                        break;
-                    case HC_SQL_FUNCTION:
-                    case HC_SQL_TABLE_VALUED_FUNCTION:
-                        name = name + "()";
-                        break;
-                    default:
-                        break;
-                }
-                tags.push_back(name);
+        for (auto related : related_help) {
+            string name = related->ht_name;
+            switch (related->ht_context) {
+                case help_context_t::HC_COMMAND:
+                    name = ":" + name;
+                    break;
+                case help_context_t::HC_SQL_FUNCTION:
+                case help_context_t::HC_SQL_TABLE_VALUED_FUNCTION:
+                    name = name + "()";
+                    break;
+                default:
+                    break;
             }
+            related_refs.push_back(name);
         }
-
-        stable_sort(tags.begin(), tags.end());
+        stable_sort(related_refs.begin(), related_refs.end());
 
         out.append("See Also", &view_curses::VC_STYLE, A_UNDERLINE)
            .append("\n")
@@ -334,34 +375,32 @@ void format_help_text_for_term(const help_text &ht, int width, attr_line_t &out,
 
         bool first = true;
         size_t line_start = out.length();
-        for (const auto &tag : tags) {
+        for (const auto &ref : related_refs) {
             if (!first) {
                 out.append(", ");
             }
-            if ((out.length() - line_start + tag.length()) > width) {
+            if ((out.length() - line_start + ref.length()) > width) {
                 out.append("\n")
                    .append(body_indent, ' ');
                 line_start = out.length();
             }
-            out.append(tag, &view_curses::VC_STYLE, A_BOLD);
+            out.append(ref, &view_curses::VC_STYLE, A_BOLD);
             first = false;
         }
     }
 }
 
-void format_example_text_for_term(const help_text &ht, int width, attr_line_t &out)
+void format_example_text_for_term(const help_text &ht,
+                                  const help_example_to_attr_line_fun_t eval,
+                                  size_t width, attr_line_t &out)
 {
     text_wrap_settings tws;
 
     tws.with_width(width);
 
     if (!ht.ht_example.empty()) {
-        map<string, string> vars;
         int count = 1;
 
-        vars["name"] = ht.ht_name;
-        add_ansi_vars(vars);
-
         out.append(ht.ht_example.size() == 1 ? "Example" : "Examples",
                    &view_curses::VC_STYLE,
                    A_UNDERLINE)
@@ -379,15 +418,15 @@ void format_example_text_for_term(const help_text &ht, int width, attr_line_t &o
                 out.append("\n");
             }
             switch (ht.ht_context) {
-                case HC_COMMAND:
+                case help_context_t::HC_COMMAND:
                     ex_line.insert(0, 1, ' ');
                     ex_line.insert(0, 1, ':');
                     ex_line.insert(1, ht.ht_name);
                     readline_command_highlighter(ex_line, 0);
                     break;
-                case HC_SQL_KEYWORD:
-                case HC_SQL_FUNCTION:
-                case HC_SQL_TABLE_VALUED_FUNCTION:
+                case help_context_t::HC_SQL_KEYWORD:
+                case help_context_t::HC_SQL_FUNCTION:
+                case help_context_t::HC_SQL_TABLE_VALUED_FUNCTION:
                     readline_sqlite_highlighter(ex_line, 0);
                     prompt = ";";
                     break;
@@ -398,14 +437,165 @@ void format_example_text_for_term(const help_text &ht, int width, attr_line_t &o
             out.append("#")
                .append(to_string(count))
                .append(" ")
+               .append(ex.he_description)
+               .append(":\n   ")
                .append(prompt)
                .append(ex_line, &tws.with_indent(3 + keyword_offset + 1))
                .append("\n")
                .append(3, ' ')
-               .append(ex.he_result, &tws.with_indent(3))
+               .append(eval(ht, ex), &tws.with_indent(3))
                .append("\n");
 
             count += 1;
         }
     }
 }
+
+static std::string link_name(const help_text &ht)
+{
+    const static pcrecpp::RE SCRUBBER("[^\\w_]");
+
+    auto scrubbed_name = string(ht.ht_name);
+    for (auto &param : ht.ht_parameters) {
+        if (param.ht_name[0]) {
+            continue;
+        }
+
+        scrubbed_name += "_";
+        scrubbed_name += param.ht_flag_name;
+    }
+    SCRUBBER.GlobalReplace("_", &scrubbed_name);
+
+    return tolower(scrubbed_name);
+}
+
+void format_help_text_for_rst(const help_text &ht,
+                              const help_example_to_attr_line_fun_t eval,
+                              FILE *rst_file)
+{
+    const char *prefix;
+    int out_count = 0;
+
+    if (!ht.ht_name || !ht.ht_name[0]) {
+        return;
+    }
+
+    bool is_sql_func = false, is_sql = false;
+    switch (ht.ht_context) {
+        case help_context_t::HC_COMMAND:
+            prefix = ":";
+            break;
+        case help_context_t::HC_SQL_FUNCTION:
+        case help_context_t::HC_SQL_TABLE_VALUED_FUNCTION:
+            is_sql = is_sql_func = true;
+            prefix = "";
+            break;
+        case help_context_t::HC_SQL_KEYWORD:
+            is_sql = true;
+            prefix = "";
+            break;
+        default:
+            prefix = "";
+            break;
+    }
+
+    fprintf(rst_file, "\n.. _%s:\n\n", link_name(ht).c_str());
+    out_count += fprintf(rst_file, "%s%s", prefix, ht.ht_name);
+    if (is_sql_func) {
+        out_count += fprintf(rst_file, "(");
+    }
+    bool needs_comma = false;
+    for (auto &param: ht.ht_parameters) {
+        if (needs_comma) {
+            if (param.ht_flag_name) {
+                out_count += fprintf(rst_file, " ");
+            } else {
+                out_count += fprintf(rst_file, ", ");
+            }
+        }
+        if (!is_sql_func) {
+            out_count += fprintf(rst_file, " ");
+        }
+
+        if (param.ht_flag_name) {
+            out_count += fprintf(rst_file, "%s ", param.ht_flag_name);
+        }
+        if (param.ht_name[0]) {
+            out_count += fprintf(rst_file, "*%s*", param.ht_name);
+        }
+        if (is_sql_func) {
+            needs_comma = true;
+        }
+    }
+    if (is_sql_func) {
+        out_count += fprintf(rst_file, ")");
+    }
+    fprintf(rst_file, "\n");
+    fprintf(rst_file, "%s\n\n", string(out_count, '^').c_str());
+
+    fprintf(rst_file, "  %s\n", ht.ht_summary);
+    fprintf(rst_file, "\n");
+    if (ht.ht_description) {
+        fprintf(rst_file, "  %s\n", ht.ht_description);
+    }
+
+    int param_count = 0;
+    for (auto &param: ht.ht_parameters) {
+        if (param.ht_summary && param.ht_summary[0]) {
+            param_count += 1;
+        }
+    }
+
+    if (param_count > 0) {
+        fprintf(rst_file, "  **Parameters:**\n\n");
+        for (auto &param: ht.ht_parameters) {
+            if (param.ht_summary && param.ht_summary[0]) {
+                fprintf(rst_file, "    * **%s** --- %s\n",
+                        param.ht_name,
+                        param.ht_summary);
+            }
+        }
+        fprintf(rst_file, "\n");
+    }
+    if (is_sql) {
+        prefix = ";";
+    }
+    if (!ht.ht_example.empty()) {
+        fprintf(rst_file, "  **Examples:**\n\n");
+        for (auto &example: ht.ht_example) {
+            fprintf(rst_file, "    %s:\n\n", example.he_description);
+            fprintf(rst_file, "    .. code-block::  %s\n\n",
+                is_sql ? "custsqlite" : "lnav");
+            if (ht.ht_context == help_context_t::HC_COMMAND) {
+                fprintf(rst_file, "      %s%s %s\n", prefix, ht.ht_name,
+                        example.he_cmd);
+            } else {
+                fprintf(rst_file, "      %s%s\n", prefix, example.he_cmd);
+            }
+            auto result = eval(ht, example);
+            if (!result.empty()) {
+                vector<attr_line_t> lines;
+
+                result.split_lines(lines);
+                for (auto &line : lines) {
+                    fprintf(rst_file, "      %s\n", line.get_string().c_str());
+                }
+            }
+            fprintf(rst_file, "\n");
+        }
+    }
+
+    if (!ht.ht_tags.empty()) {
+        auto related_refs = vector<string>();
+
+        for (auto related : get_related(ht)) {
+            related_refs.emplace_back(fmt::format(":ref:`{}`", link_name(*related)));
+        }
+        stable_sort(related_refs.begin(), related_refs.end());
+
+        fprintf(rst_file, "  **See Also:**\n\n    %s\n",
+            join(related_refs.begin(), related_refs.end(), ", ").c_str());
+    }
+
+    fprintf(rst_file, "\n----\n\n");
+}
