@@ -46,6 +46,8 @@
 #include "field_overlay_source.hh"
 #include "hotkeys.hh"
 #include "log_format_loader.hh"
+#include "base/opt_util.hh"
+#include "shlex.hh"
 
 using namespace std;
 
@@ -131,58 +133,65 @@ static int key_sql_callback(exec_context &ec, sqlite3_stmt *stmt)
 
 bool handle_keyseq(const char *keyseq)
 {
-    key_map &km = lnav_config.lc_ui_keymaps[lnav_config.lc_ui_keymap];
+    key_map &km = lnav_config.lc_active_keymap;
 
     const auto &iter = km.km_seq_to_cmd.find(keyseq);
-    if (iter != km.km_seq_to_cmd.end()) {
-        vector<logline_value> values;
-        exec_context ec(&values, key_sql_callback, pipe_callback);
-        auto &var_stack = ec.ec_local_vars;
-        string result;
-
-        ec.ec_global_vars = lnav_data.ld_exec_context.ec_global_vars;
-        var_stack.push(map<string, string>());
-        auto &vars = var_stack.top();
-        vars["keyseq"] = keyseq;
-        for (const string &cmd : iter->second) {
-            log_debug("executing key sequence x%02x: %s",
-                      keyseq, cmd.c_str());
-            result = execute_any(ec, cmd);
-        }
-
-        lnav_data.ld_rl_view->set_value(result);
-        return true;
+    if (iter == km.km_seq_to_cmd.end()) {
+        return false;
     }
 
-    return false;
+    vector<logline_value> values;
+    exec_context ec(&values, key_sql_callback, pipe_callback);
+    auto &var_stack = ec.ec_local_vars;
+
+    ec.ec_global_vars = lnav_data.ld_exec_context.ec_global_vars;
+    var_stack.push(map<string, string>());
+    auto &vars = var_stack.top();
+    vars["keyseq"] = keyseq;
+    const auto &kc = iter->second;
+
+    log_debug("executing key sequence %s: %s", keyseq, kc.kc_cmd.c_str());
+    auto result = execute_any(ec, kc.kc_cmd);
+    lnav_data.ld_rl_view->set_value(
+        result.map(ok_prefix).orElse(err_to_ok).unwrap());
+
+    if (!kc.kc_alt_msg.empty()) {
+        shlex lexer(kc.kc_alt_msg);
+        string expanded_msg;
+
+        if (lexer.eval(expanded_msg, {
+            &ec.ec_local_vars.top(),
+            &ec.ec_global_vars,
+        })) {
+            lnav_data.ld_rl_view->set_alt_value(expanded_msg);
+        }
+    }
+
+    return true;
 }
 
-void handle_paging_key(int ch)
+bool handle_paging_key(int ch)
 {
     if (lnav_data.ld_view_stack.vs_views.empty()) {
-        return;
+        return false;
     }
 
     textview_curses *tc = *lnav_data.ld_view_stack.top();
     exec_context &ec = lnav_data.ld_exec_context;
-    logfile_sub_source *lss = NULL;
+    logfile_sub_source *lss = nullptr;
     text_sub_source *tc_tss = tc->get_sub_source();
     bookmarks<vis_line_t>::type &     bm  = tc->get_bookmarks();
 
-    if (tc->handle_key(ch)) {
-        return;
-    }
-
     char keyseq[16];
 
     snprintf(keyseq, sizeof(keyseq), "x%02x", ch);
 
     if (handle_keyseq(keyseq)) {
-        return;
+        return true;
     }
 
     if (tc->handle_key(ch)) {
-        return;
+        return true;
     }
 
     lss = dynamic_cast<logfile_sub_source *>(tc->get_sub_source());
@@ -194,19 +203,19 @@ void handle_paging_key(int ch)
             break;
 
         case 'a':
-            if (lnav_data.ld_last_view == NULL) {
+            if (lnav_data.ld_last_view == nullptr) {
                 alerter::singleton().chime();
             }
             else {
                 textview_curses *tc = lnav_data.ld_last_view;
 
-                lnav_data.ld_last_view = NULL;
+                lnav_data.ld_last_view = nullptr;
                 ensure_view(tc);
             }
             break;
 
         case 'A':
-            if (lnav_data.ld_last_view == NULL) {
+            if (lnav_data.ld_last_view == nullptr) {
                 alerter::singleton().chime();
             }
             else {
@@ -215,8 +224,8 @@ void handle_paging_key(int ch)
                 auto *dst_view = dynamic_cast<text_time_translator *>(tc->get_sub_source());
                 auto *src_view = dynamic_cast<text_time_translator *>(top_tc->get_sub_source());
 
-                lnav_data.ld_last_view = NULL;
-                if (src_view != NULL && dst_view != NULL) {
+                lnav_data.ld_last_view = nullptr;
+                if (src_view != nullptr && dst_view != nullptr) {
                     struct timeval top_time = src_view->time_for_row(top_tc->get_top());
 
                     tc->set_top(vis_line_t(dst_view->row_for_time(top_time)));
@@ -229,7 +238,7 @@ void handle_paging_key(int ch)
             if (xterm_mouse::is_available()) {
                 lnav_data.ld_mouse.set_enabled(!lnav_data.ld_mouse.is_enabled());
                 lnav_data.ld_rl_view->set_value(
-                    string("info: mouse mode -- ") +
+                    ok_prefix("info: mouse mode -- ") +
                     (lnav_data.ld_mouse.is_enabled() ?
                      ANSI_BOLD("enabled") : ANSI_BOLD("disabled")));
             }
@@ -250,7 +259,7 @@ void handle_paging_key(int ch)
             tc->get_bookmarks()[&textview_curses::BM_USER].clear();
             tc->reload_data();
 
-            lnav_data.ld_rl_view->set_value("Cleared bookmarks");
+            lnav_data.ld_rl_view->set_value(ok_prefix("Cleared bookmarks"));
             break;
 
         case '>':
@@ -260,7 +269,6 @@ void handle_paging_key(int ch)
             tc->horiz_shift(tc->get_top(),
                             tc->get_bottom(),
                             tc->get_left(),
-                            "$search",
                             range);
             if (range.second != INT_MAX) {
                 tc->set_left(range.second);
@@ -283,7 +291,6 @@ void handle_paging_key(int ch)
                 tc->horiz_shift(tc->get_top(),
                                 tc->get_bottom(),
                                 tc->get_left(),
-                                "$search",
                                 range);
                 if (range.first != -1) {
                     tc->set_left(range.first);
@@ -618,7 +625,8 @@ void handle_paging_key(int ch)
                         start_helper.lh_string_attrs, &logline::L_OPID);
                 if (!opid_range.is_valid()) {
                     alerter::singleton().chime();
-                    lnav_data.ld_rl_view->set_value("Log message does not contain an opid");
+                    lnav_data.ld_rl_view->set_value(
+                        err_prefix("Log message does not contain an opid"));
                 } else {
                     unsigned int opid_hash = start_line.get_opid();
                     logline_helper next_helper(*lss);
@@ -664,7 +672,7 @@ void handle_paging_key(int ch)
                         string opid_str = start_helper.to_string(opid_range);
 
                         lnav_data.ld_rl_view->set_value(
-                                "No more messages found with opid: " + opid_str);
+                            err_prefix("No more messages found with opid: " + opid_str));
                         alerter::singleton().chime();
                     }
                 }
@@ -678,7 +686,7 @@ void handle_paging_key(int ch)
                         "now",
                         "today",
                         "yesterday",
-                        NULL
+                        nullptr
                 };
 
                 logfile_sub_source &lss      = lnav_data.ld_log_source;
@@ -713,7 +721,7 @@ void handle_paging_key(int ch)
                         const logline_value_stats *stats = ldh_line_value.lv_format->stats_for_value(
                             ldh_line_value.lv_name);
 
-                        if (stats == NULL) {
+                        if (stats == nullptr) {
                             continue;
                         }
 
@@ -766,6 +774,7 @@ void handle_paging_key(int ch)
                 }
             }
 
+            rollback_lnav_config = lnav_config;
             lnav_data.ld_doc_status_source.set_title("Command Help");
             add_view_text_possibilities(lnav_data.ld_rl_view, LNM_COMMAND, "filter", tc);
             lnav_data.ld_rl_view->add_possibility(LNM_COMMAND, "filter", tc->get_last_search());
@@ -860,7 +869,8 @@ void handle_paging_key(int ch)
         case 't':
             if (lnav_data.ld_text_source.current_file() == nullptr) {
                 alerter::singleton().chime();
-                lnav_data.ld_rl_view->set_value("No text files loaded");
+                lnav_data.ld_rl_view->set_value(
+                    err_prefix("No text files loaded"));
             }
             else if (toggle_view(&lnav_data.ld_views[LNV_TEXT])) {
                 lnav_data.ld_rl_view->set_alt_value(HELP_MSG_2(
@@ -880,25 +890,32 @@ void handle_paging_key(int ch)
 
         case 'I':
         {
-            hist_source2 &hs = lnav_data.ld_hist_source2;
+            auto &hs = lnav_data.ld_hist_source2;
+            auto &hist_tc = lnav_data.ld_views[LNV_HISTOGRAM];
 
-            if (toggle_view(&lnav_data.ld_views[LNV_HISTOGRAM])) {
-                struct timeval log_top = lss->time_for_row(lnav_data.ld_views[LNV_LOG].get_top());
+            if (toggle_view(&hist_tc)) {
+                auto *src_view = dynamic_cast<text_time_translator *>(tc->get_sub_source());
 
-                tc = *lnav_data.ld_view_stack.top();
-                tc->set_top(vis_line_t(hs.row_for_time(log_top)));
+                if (src_view != nullptr) {
+                    struct timeval log_top = src_view->time_for_row(tc->get_top());
+
+                    hist_tc.set_top(vis_line_t(hs.row_for_time(log_top)));
+                }
             }
             else {
-                textview_curses &hist_tc = lnav_data.ld_views[LNV_HISTOGRAM];
-                textview_curses &log_tc = lnav_data.ld_views[LNV_LOG];
-                lss = &lnav_data.ld_log_source;
-                struct timeval hist_top_time = hs.time_for_row(hist_tc.get_top());
-                struct timeval curr_top_time = lss->time_for_row(log_tc.get_top());
-                if (hs.row_for_time(hist_top_time) != hs.row_for_time(curr_top_time)) {
-                    vis_line_t new_top = lss->find_from_time(hist_top_time);
-                    log_tc.set_top(new_top);
-                    log_tc.set_needs_update();
-                }
+                lnav_data.ld_view_stack.top() | [&](auto top_tc) {
+                    auto *dst_view = dynamic_cast<text_time_translator *>(top_tc->get_sub_source());
+
+                    if (dst_view != nullptr) {
+                        struct timeval hist_top_time = hs.time_for_row(hist_tc.get_top());
+                        struct timeval curr_top_time = dst_view->time_for_row(top_tc->get_top());
+                        if (hs.row_for_time(hist_top_time) != hs.row_for_time(curr_top_time)) {
+                            vis_line_t new_top = vis_line_t(dst_view->row_for_time(hist_top_time));
+                            top_tc->set_top(new_top);
+                            top_tc->set_needs_update();
+                        }
+                    }
+                };
             }
         }
             break;
@@ -961,7 +978,7 @@ void handle_paging_key(int ch)
                         const char *col_value = dls.dls_rows[db_row][lpc];
                         size_t col_len = strlen(col_value);
 
-                        if (dts.scan(col_value, col_len, NULL, &tm, tv) != NULL) {
+                        if (dts.scan(col_value, col_len, nullptr, &tm, tv) != nullptr) {
                             vis_line_t vl;
 
                             vl = lnav_data.ld_log_source.find_from_time(tv);
@@ -1072,11 +1089,7 @@ void handle_paging_key(int ch)
             break;
 
         default:
-            log_warning("unhandled %d", ch);
-            lnav_data.ld_rl_view->set_value("Unrecognized keystroke, press "
-            ANSI_BOLD("?")
-            " to view help");
-            alerter::singleton().chime();
-            break;
+            return false;
     }
+    return true;
 }
