@@ -40,8 +40,8 @@
 
 #include "sqlite3.h"
 
-#include "yajlpp.hh"
-#include "json_op.hh"
+#include "yajlpp/yajlpp.hh"
+#include "yajlpp/json_op.hh"
 #include "mapbox/variant.hpp"
 #include "vtab_module.hh"
 
@@ -197,18 +197,17 @@ static void sql_jget(sqlite3_context *context,
     const char *ptr_in = (const char *)sqlite3_value_text(argv[1]);
     json_ptr jp(ptr_in);
     sql_json_op jo(jp);
-    auto_mem<yajl_gen_t> gen(yajl_gen_free);
     auto_mem<yajl_handle_t> handle(yajl_free);
     const unsigned char *err;
+    yajlpp_gen gen;
 
-    gen = yajl_gen_alloc(nullptr);
-    yajl_gen_config(gen.in(), yajl_gen_beautify, false);
+    yajl_gen_config(gen, yajl_gen_beautify, false);
 
     jo.jo_ptr_callbacks = json_op::gen_callbacks;
     jo.jo_ptr_callbacks.yajl_null = gen_handle_null;
     jo.jo_ptr_callbacks.yajl_boolean = gen_handle_boolean;
     jo.jo_ptr_callbacks.yajl_string = gen_handle_string;
-    jo.jo_ptr_data = gen.in();
+    jo.jo_ptr_data = gen.get_handle();
 
     handle.reset(yajl_alloc(&json_op::ptr_callbacks, nullptr, &jo));
     switch (yajl_parse(handle.in(), (const unsigned char *)json_in, strlen(json_in))) {
@@ -257,17 +256,172 @@ static void sql_jget(sqlite3_context *context,
         return;
     }
 
-    const unsigned char *buf;
-    size_t len;
+    string_fragment result = gen.to_string_fragment();
 
-    yajl_gen_get_buf(gen, &buf, &len);
-
-    if (len == 0) {
+    if (result.empty()) {
         null_or_default(context, argc, argv);
         return;
     }
 
-    sqlite3_result_text(context, (const char *) buf, len, SQLITE_TRANSIENT);
+    sqlite3_result_text(context, result.data(), result.length(), SQLITE_TRANSIENT);
+}
+
+struct concat_context {
+    concat_context(yajl_gen gen_handle)
+        : cc_gen_handle(gen_handle) {
+    }
+
+    yajl_gen cc_gen_handle;
+    int cc_depth{0};
+};
+
+static int concat_gen_null(void *ctx)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    if (cc->cc_depth > 0) {
+        return yajl_gen_null(cc->cc_gen_handle) == yajl_gen_status_ok;
+    }
+
+    return 1;
+}
+
+static int concat_gen_boolean(void *ctx, int val)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    return yajl_gen_bool(cc->cc_gen_handle, val) == yajl_gen_status_ok;
+}
+
+static int concat_gen_number(void *ctx, const char *val, size_t len)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    return yajl_gen_number(cc->cc_gen_handle, val, len) == yajl_gen_status_ok;
+}
+
+static int concat_gen_string(void *ctx, const unsigned char *val, size_t len)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    return yajl_gen_string(cc->cc_gen_handle, val, len) == yajl_gen_status_ok;
+}
+
+static int concat_gen_start_map(void *ctx)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    cc->cc_depth += 1;
+    return yajl_gen_map_open(cc->cc_gen_handle) == yajl_gen_status_ok;
+}
+
+static int concat_gen_end_map(void *ctx)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    cc->cc_depth -= 1;
+    return yajl_gen_map_close(cc->cc_gen_handle) == yajl_gen_status_ok;
+}
+
+static int concat_gen_map_key(void *ctx, const unsigned char *key, size_t len)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    return yajl_gen_string(cc->cc_gen_handle, key, len) == yajl_gen_status_ok;
+}
+
+static int concat_gen_start_array(void *ctx)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    cc->cc_depth += 1;
+    if (cc->cc_depth == 1) {
+        return 1;
+    }
+    return yajl_gen_array_open(cc->cc_gen_handle) == yajl_gen_status_ok;
+}
+
+static int concat_gen_end_array(void *ctx)
+{
+    auto cc = static_cast<concat_context *>(ctx);
+
+    cc->cc_depth -= 1;
+    if (cc->cc_depth == 0) {
+        return 1;
+    }
+    return yajl_gen_array_close(cc->cc_gen_handle) == yajl_gen_status_ok;
+}
+
+static void concat_gen_elements(yajl_gen gen, const unsigned char *text, size_t len)
+{
+    auto_mem<yajl_handle_t> handle(yajl_free);
+    yajl_callbacks callbacks = {nullptr};
+    concat_context cc{gen};
+
+    callbacks.yajl_null = concat_gen_null;
+    callbacks.yajl_boolean = concat_gen_boolean;
+    callbacks.yajl_number = concat_gen_number;
+    callbacks.yajl_string = concat_gen_string;
+    callbacks.yajl_start_map = concat_gen_start_map;
+    callbacks.yajl_end_map = concat_gen_end_map;
+    callbacks.yajl_map_key = concat_gen_map_key;
+    callbacks.yajl_start_array = concat_gen_start_array;
+    callbacks.yajl_end_array = concat_gen_end_array;
+
+    handle = yajl_alloc(&callbacks, nullptr, &cc);
+    yajl_config(handle, yajl_allow_comments, 1);
+    if (yajl_parse(handle, (const unsigned char *) text, len) != yajl_status_ok ||
+        yajl_complete_parse(handle) != yajl_status_ok) {
+        unique_ptr<unsigned char, decltype(&free)> err_msg(
+            yajl_get_error(handle, 1, (const unsigned char *) text, len), free);
+
+        throw sqlite_func_error("Invalid JSON: {}", (const char *) err_msg.get());
+    }
+}
+
+static json_string json_concat(nonstd::optional<const char *> json_in, const vector<sqlite3_value *> &values)
+{
+    yajlpp_gen gen;
+
+    yajl_gen_config(gen, yajl_gen_beautify, false);
+
+    {
+        yajlpp_array array(gen);
+
+        if (json_in) {
+            concat_gen_elements(gen,
+                                (const unsigned char *) json_in.value(),
+                                strlen(json_in.value()));
+        }
+
+        for (const auto val: values) {
+            switch (sqlite3_value_type(val)) {
+                case SQLITE_NULL:
+                    array.gen();
+                    break;
+                case SQLITE_INTEGER:
+                    array.gen(sqlite3_value_int64(val));
+                    break;
+                case SQLITE_FLOAT:
+                    array.gen(sqlite3_value_double(val));
+                    break;
+                case SQLITE3_TEXT: {
+                    auto text_val = sqlite3_value_text(val);
+
+                    if (sqlite3_value_subtype(val) == JSON_SUBTYPE) {
+                        concat_gen_elements(gen,
+                                            text_val,
+                                            strlen((const char *) text_val));
+                    } else {
+                        array.gen((const char *) text_val);
+                    }
+                    break;
+                }
+            }
+        }
+    }
+
+    return json_string(gen);
 }
 
 struct json_agg_context {
@@ -456,14 +610,47 @@ int json_extension_functions(struct FuncDef **basic_funcs,
                              struct FuncDefAgg **agg_funcs)
 {
     static struct FuncDef json_funcs[] = {
+        sqlite_func_adapter<decltype(&json_concat), json_concat>::builder(
+            help_text("json_concat",
+                      "Returns an array with the given values concatenated onto the end.  "
+                      "If the initial value is null, the result will be an array with "
+                      "the given elements.  If the initial value is an array, the result "
+                      "will be an array with the given values at the end.  If the initial "
+                      "value is not null or an array, the result will be an array with "
+                      "two elements: the initial value and the given value.")
+                .sql_function()
+                .with_parameter({"json", "The initial JSON value."})
+                .with_parameter(help_text("value", "The value(s) to add to the end of the array.")
+                                    .one_or_more())
+                .with_tags({"json"})
+                .with_example({
+                    "To append the number 4 to null",
+                    "SELECT json_concat(NULL, 4)"
+                })
+                .with_example({
+                    "To append 4 and 5 to the array [1, 2, 3]",
+                    "SELECT json_concat('[1, 2, 3]', 4, 5)"
+                })
+                .with_example({
+                    "To concatenate two arrays together",
+                    "SELECT json_concat('[1, 2, 3]', json('[4, 5]'))"
+                })
+        ),
+
         sqlite_func_adapter<decltype(&json_contains), json_contains>::builder(
-            help_text("json_contains", "")
+            help_text("json_contains", "Check if a JSON value contains the given element.")
                 .sql_function()
                 .with_parameter({"json", "The JSON value to query."})
                 .with_parameter({"value", "The value to look for in the first argument"})
                 .with_tags({"json"})
-                .with_example({"SELECT json_contains('[1, 2, 3]', 4)"})
-                .with_example({"SELECT json_contains('[\"abc\", \"def\"]', 'def')"})
+                .with_example({
+                    "To test if a JSON array contains the number 4",
+                    "SELECT json_contains('[1, 2, 3]', 4)"
+                })
+                .with_example({
+                    "To test if a JSON array contains the string 'def'",
+                    "SELECT json_contains('[\"abc\", \"def\"]', 'def')"
+                })
         ),
 
         {
@@ -476,21 +663,59 @@ int json_extension_functions(struct FuncDef **basic_funcs,
                 .with_parameter(help_text("default", "The default value if the value was not found")
                                     .optional())
                 .with_tags({"json"})
-                .with_example({"SELECT jget('1', '')"})
-                .with_example({"SELECT jget('{ \"a\": 1, \"b\": 2 }', '/b')"})
-                .with_example({"SELECT jget(null, '/msg', 'Hello')"})
+                .with_example({
+                    "To get the root of a JSON value",
+                    "SELECT jget('1', '')"
+                })
+                .with_example({
+                    "To get the property named 'b' in a JSON object",
+                    "SELECT jget('{ \"a\": 1, \"b\": 2 }', '/b')"
+                })
+                .with_example({
+                    "To get the 'msg' property and return a default if it does not exist",
+                    "SELECT jget(null, '/msg', 'Hello')"
+                })
         },
 
-        { NULL }
+        { nullptr }
     };
 
     static struct FuncDefAgg json_agg_funcs[] = {
             { "json_group_object", -1, 0,
-                    sql_json_group_object_step, sql_json_group_object_final, },
+                    sql_json_group_object_step, sql_json_group_object_final,
+                    help_text("json_group_object")
+                        .sql_function()
+                        .with_summary("Collect the given values from a query into a JSON object")
+                        .with_parameter(help_text("name", "The property name for the value"))
+                        .with_parameter(help_text("value", "The value to add to the object")
+                                            .one_or_more())
+                        .with_tags({"json"})
+                        .with_example({
+                            "To create an object from arguments",
+                            "SELECT json_group_object('a', 1, 'b', 2)"
+                        })
+                        .with_example({
+                            "To create an object from a pair of columns",
+                            "SELECT json_group_object(column1, column2) FROM (VALUES ('a', 1), ('b', 2))"
+                        })},
             { "json_group_array", -1, 0,
-                    sql_json_group_array_step, sql_json_group_array_final, },
+                    sql_json_group_array_step, sql_json_group_array_final,
+                    help_text("json_group_array")
+                        .sql_function()
+                        .with_summary("Collect the given values from a query into a JSON array")
+                        .with_parameter(help_text("value", "The values to append to the array")
+                                            .one_or_more())
+                        .with_tags({"json"})
+                        .with_example({
+                            "To create an array from arguments",
+                            "SELECT json_group_array('one', 2, 3.4)"
+                        })
+                        .with_example({
+                            "To create an array from a column of values",
+                            "SELECT json_group_array(column1) FROM (VALUES (1), (2), (3))"
+                        })},
 
-            { NULL }
+            { nullptr }
     };
 
     *basic_funcs = json_funcs;
