@@ -36,7 +36,7 @@
 
 #include <cmath>
 
-#include "lnav_log.hh"
+#include "base/lnav_log.hh"
 #include "listview_curses.hh"
 
 using namespace std;
@@ -260,7 +260,7 @@ void listview_curses::do_update()
                  gutter_y++) {
                 int range_start = 0, range_end;
                 view_colors::role_t role = this->vc_default_role;
-                view_colors::role_t bar_role = view_colors::VCR_STATUS;
+                view_colors::role_t bar_role = view_colors::VCR_SCROLLBAR;
                 int attrs;
                 chtype ch = ACS_VLINE;
 
@@ -287,7 +287,7 @@ void listview_curses::do_update()
             int y = this->lv_y + height - 1;
 
             mvwin_wchnstr(this->lv_window, y, this->lv_x, row_ch, width - 1);
-            for (int lpc = 0; lpc < width - 1; lpc++) {
+            for (unsigned long lpc = 0; lpc < width - 1; lpc++) {
                 row_ch[lpc].attr |= A_UNDERLINE;
             }
             mvwadd_wchnstr(this->lv_window, y, this->lv_x, row_ch, width - 1);
