@@ -80,10 +80,10 @@
 #endif
 
 #include "lnav.hh"
-#include "help.hh"
-#include "init-sql.hh"
+#include "help-txt.h"
+#include "init-sql.h"
 #include "logfile.hh"
-#include "lnav_log.hh"
+#include "base/lnav_log.hh"
 #include "log_accel.hh"
 #include "lnav_util.hh"
 #include "ansi_scrubber.hh"
@@ -132,8 +132,7 @@
 #include <curl/curl.h>
 #endif
 
-#include "papertrail_proc.hh"
-#include "yajlpp.hh"
+#include "yajlpp/yajlpp.hh"
 #include "readline_callbacks.hh"
 #include "command_executor.hh"
 #include "plain_text_source.hh"
@@ -145,6 +144,10 @@
 #include "shlex.hh"
 #include "log_actions.hh"
 
+#ifndef SYSCONFDIR
+#define SYSCONFDIR "/usr/etc"
+#endif
+
 using namespace std;
 
 static multimap<lnav_flags_t, string> DEFAULT_FILES;
@@ -223,6 +226,8 @@ static std::vector<std::string> DEFAULT_DB_KEY_NAMES = {
     "st_gid",
 };
 
+const static size_t MAX_STDIN_CAPTURE_SIZE = 10 * 1024 * 1024;
+
 static void regenerate_unique_file_names()
 {
     unique_path_generator upg;
@@ -334,8 +339,8 @@ bool setup_logline_table(exec_context &ec)
 
         for (const auto &pair : sqlite_function_help) {
             switch (pair.second->ht_context) {
-                case HC_SQL_FUNCTION:
-                case HC_SQL_TABLE_VALUED_FUNCTION: {
+                case help_context_t::HC_SQL_FUNCTION:
+                case help_context_t::HC_SQL_TABLE_VALUED_FUNCTION: {
                     string poss = pair.first +
                         (pair.second->ht_parameters.empty() ? "()" : ("("));
 
@@ -535,7 +540,9 @@ void rebuild_indexes()
         textfile_sub_source *tss = &lnav_data.ld_text_source;
         textfile_callback cb;
 
-        tss->rescan_files(cb);
+        if (tss->rescan_files(cb)) {
+            text_view.reload_data();
+        }
 
         if (cb.front_file != nullptr) {
             ensure_view(&text_view);
@@ -559,7 +566,7 @@ void rebuild_indexes()
          file_iter != lnav_data.ld_files.end(); ) {
         auto lf = *file_iter;
 
-        if (!lf->exists() || lf->is_closed()) {
+        if ((!lf->exists() || lf->is_closed())) {
             log_info("closed log file: %s", lf->get_filename().c_str());
             if (!lf->is_valid_filename()) {
                 lnav_data.ld_file_names.erase(lf->get_filename());
@@ -578,9 +585,12 @@ void rebuild_indexes()
     logfile_sub_source::rebuild_result result = lss.rebuild_index();
     if (result != logfile_sub_source::rebuild_result::rr_no_change) {
         size_t new_count = lss.text_line_count();
-        bool force = result == logfile_sub_source::rebuild_result::rr_full_rebuild;
+        bool force =
+            result == logfile_sub_source::rebuild_result::rr_full_rebuild;
 
-        if ((!scroll_downs[LNV_LOG] || log_view.get_top() > new_count) && force) {
+        if ((!scroll_downs[LNV_LOG] ||
+             log_view.get_top() > vis_line_t(new_count)) &&
+            force) {
             scroll_downs[LNV_LOG] = false;
         }
 
@@ -821,7 +831,7 @@ static void usage()
         "             to install the default set of third-party formats.\n"
         "  -u         Update formats installed from git repositories.\n"
         "  -C         Check configuration and then exit.\n"
-        "  -d file    Write debug messages to the given file.\n"
+        "  -d path    Write debug messages to the given file.\n"
         "  -V         Print version information.\n"
         "\n"
         "  -a         Load all of the most recent log file types.\n"
@@ -835,7 +845,7 @@ static void usage()
         "  -f path    Execute the commands in the given file.\n"
         "  -n         Run without the curses UI. (headless mode)\n"
         "  -q         Do not print the log messages after executing all\n"
-        "             of the commands or when lnav is reading from stdin.\n"
+        "             of the commands.\n"
         "\n"
         "Optional arguments:\n"
         "  logfile1          The log files or directories to view.  If a\n"
@@ -1052,9 +1062,9 @@ bool rescan_files(bool required)
 
                 expand_filename(path, false);
             }
-        }
-        else {
-            retval = retval || watch_logfile(iter->first, iter->second, required);
+        } else {
+            retval = retval ||
+                     watch_logfile(iter->first, iter->second, required);
         }
     }
 
@@ -1062,7 +1072,7 @@ bool rescan_files(bool required)
          file_iter != lnav_data.ld_files.end(); ) {
         auto lf = *file_iter;
 
-        if (!lf->exists() || lf->is_closed()) {
+        if ((!lf->exists() || lf->is_closed())) {
             log_info("Log file no longer exists or is closed: %s",
                      lf->get_filename().c_str());
             return true;
@@ -1079,11 +1089,6 @@ class lnav_behavior : public mouse_behavior {
 public:
     lnav_behavior() {};
 
-    int scroll_polarity(int button)
-    {
-        return button == xterm_mouse::XT_SCROLL_UP ? -1 : 1;
-    };
-
     void mouse_event(int button, bool release, int x, int y)
     {
         textview_curses *tc = *(lnav_data.ld_view_stack.top());
@@ -1127,14 +1132,9 @@ public:
 private:
 };
 
-static void handle_key(int ch) {
+static bool handle_key(int ch) {
     lnav_data.ld_input_state.push_back(ch);
 
-    if (lnav_data.ld_mode == LNM_PAGING) {
-        auto top_tc = lnav_data.ld_view_stack.top();
-
-    }
-
     switch (ch) {
         case CTRL('d'):
         case KEY_RESIZE:
@@ -1142,12 +1142,11 @@ static void handle_key(int ch) {
         default: {
             switch (lnav_data.ld_mode) {
                 case LNM_PAGING:
-                    handle_paging_key(ch);
-                    break;
+                    return handle_paging_key(ch);
 
                 case LNM_FILTER:
                     if (!lnav_data.ld_filter_view.handle_key(ch)) {
-                        handle_paging_key(ch);
+                        return handle_paging_key(ch);
                     }
                     break;
 
@@ -1165,30 +1164,23 @@ static void handle_key(int ch) {
             }
         }
     }
+
+    return true;
 }
 
-static input_dispatcher::escape_match_t match_escape_seq(const char *escape_buffer)
+static input_dispatcher::escape_match_t match_escape_seq(const char *keyseq)
 {
     if (lnav_data.ld_mode != LNM_PAGING) {
         return input_dispatcher::escape_match_t::NONE;
     }
 
-    char keyseq[32] = "";
-
-    for (size_t lpc = 0; escape_buffer[lpc]; lpc++) {
-        snprintf(keyseq + strlen(keyseq), sizeof(keyseq) - strlen(keyseq),
-                 "x%02x",
-                 escape_buffer[lpc]);
-    }
-
-    key_map &km = lnav_config.lc_ui_keymaps[lnav_config.lc_ui_keymap];
+    auto &km = lnav_config.lc_active_keymap;
     auto iter = km.km_seq_to_cmd.find(keyseq);
     if (iter != km.km_seq_to_cmd.end()) {
         return input_dispatcher::escape_match_t::FULL;
     }
 
     auto lb = km.km_seq_to_cmd.lower_bound(keyseq);
-
     if (lb == km.km_seq_to_cmd.end()) {
         return input_dispatcher::escape_match_t::NONE;
     }
@@ -1198,26 +1190,13 @@ static input_dispatcher::escape_match_t match_escape_seq(const char *escape_buff
         return l.first.size() < r.first.size();
     });
 
-    if (strlen(escape_buffer) < longest->first.size()) {
+    if (strlen(keyseq) < longest->first.size()) {
         return input_dispatcher::escape_match_t::PARTIAL;
     }
 
     return input_dispatcher::escape_match_t::NONE;
 }
 
-static void handle_escape_seq(const char *escape_buffer)
-{
-    char keyseq[32] = "";
-
-    for (size_t lpc = 0; escape_buffer[lpc]; lpc++) {
-        snprintf(keyseq + strlen(keyseq), sizeof(keyseq) - strlen(keyseq),
-                 "x%02x",
-                 escape_buffer[lpc]);
-    }
-
-    handle_keyseq(keyseq);
-}
-
 void update_hits(void *dummy, textview_curses *tc)
 {
     auto top_tc = lnav_data.ld_view_stack.top();
@@ -1272,7 +1251,7 @@ void update_hits(void *dummy, textview_curses *tc)
                          .set_value("Matching lines for search");
                 lnav_data.ld_preview_source
                          .replace_with(all_matches)
-                         .set_text_format(TF_UNKNOWN);
+                         .set_text_format(text_format_t::TF_UNKNOWN);
             }
         }
     }
@@ -1368,6 +1347,13 @@ static void looper()
         screen_curses sc;
         lnav_behavior lb;
 
+        auto_fd errpipe[2];
+        auto_fd::pipe(errpipe);
+
+        dup2(errpipe[1], STDERR_FILENO);
+        errpipe[1].reset();
+        log_pipe_err(errpipe[0]);
+
         ui_periodic_timer::singleton();
 
         lnav_data.ld_mouse.set_behavior(&lb);
@@ -1407,12 +1393,12 @@ static void looper()
                 for (auto &hl : format->lf_highlighters) {
                     if (hl.h_fg.empty()) {
                         hl.with_attrs(hl.h_attrs | vc.attrs_for_ident(hl.h_pattern));
-                    } else {
-                        hl.with_attrs(hl.h_attrs | vc.ensure_color_pair(hl.h_fg, hl.h_bg));
                     }
 
-                    lnav_data.ld_views[LNV_LOG].get_highlights()[
-                        "$" + format->get_name().to_string() + "-" + hl.h_pattern] = hl;
+                    lnav_data.ld_views[LNV_LOG].get_highlights()[{
+                        highlight_source_t::CONFIGURATION,
+                        format->get_name().to_string() + "-" + hl.h_pattern
+                    }] = hl;
                 }
             }
         }
@@ -1447,7 +1433,7 @@ static void looper()
             lnav_data.ld_views[lpc].set_search_action(
                 textview_curses::action(update_hits));
             using std::placeholders::_1;
-            lnav_data.ld_views[lpc].tc_search_event_handler =
+            lnav_data.ld_views[lpc].tc_state_event_handler =
                 std::bind(&bottom_status_source::update_search_term,
                     &lnav_data.ld_bottom_source, _1);
         }
@@ -1501,7 +1487,8 @@ static void looper()
             sc.window_change();
         }
 
-        execute_file(ec, dotlnav_path("session"));
+        auto session_path = dotlnav_path() / "session";
+        execute_file(ec, session_path.str());
 
         sb.invoke(*lnav_data.ld_view_stack.top());
         vsb.invoke(*lnav_data.ld_view_stack.top());
@@ -1510,9 +1497,27 @@ static void looper()
             input_dispatcher &id = lnav_data.ld_input_dispatcher;
 
             id.id_escape_matcher = match_escape_seq;
-            id.id_escape_handler = handle_escape_seq;
+            id.id_escape_handler = handle_keyseq;
             id.id_key_handler = handle_key;
             id.id_mouse_handler = bind(&xterm_mouse::handle_mouse, &lnav_data.ld_mouse);
+            id.id_unhandled_handler = [](const char *keyseq) {
+                auto enc_len = lnav_config.lc_ui_keymap.size() * 2;
+                auto encoded_name = (char *) alloca(enc_len);
+
+                log_info("unbound keyseq: %s", keyseq);
+                json_ptr::encode(encoded_name, enc_len, lnav_config.lc_ui_keymap.c_str());
+                // XXX we should have a hotkey for opening a prompt that is
+                // pre-filled with a suggestion that the user can complete.
+                // This quick-fix key could be used for other stuff as well
+                lnav_data.ld_rl_view->set_value(fmt::format(
+                    ANSI_CSI ANSI_COLOR_PARAM(COLOR_YELLOW) ";" ANSI_BOLD_PARAM ANSI_CHAR_ATTR
+                    "Unrecognized key"
+                    ANSI_NORM
+                    ", bind to a command using \u2014 "
+                    ANSI_BOLD(":config") " /ui/keymap-defs/{}/{}/command <cmd>",
+                    encoded_name, keyseq));
+                alerter::singleton().chime();
+            };
         }
 
         bool session_loaded = false;
@@ -1640,8 +1645,8 @@ static void looper()
                 }
                 rebuild_indexes();
                 if (!initial_build &&
-                        lnav_data.ld_log_source.text_line_count() == 0 &&
-                        lnav_data.ld_text_source.text_line_count() > 0) {
+                    lnav_data.ld_log_source.text_line_count() == 0 &&
+                    lnav_data.ld_text_source.text_line_count() > 0) {
                     ensure_view(&lnav_data.ld_views[LNV_TEXT]);
                     lnav_data.ld_views[LNV_TEXT].set_top(vis_line_t(0));
                     lnav_data.ld_rl_view->set_alt_value(
@@ -1657,8 +1662,8 @@ static void looper()
                     lnav_data.ld_views[LNV_LOG].set_top(tc_log->get_top_for_last_row());
                 }
                 if (!initial_build &&
-                        lnav_data.ld_log_source.text_line_count() == 0 &&
-                        !lnav_data.ld_other_files.empty()) {
+                    lnav_data.ld_log_source.text_line_count() == 0 &&
+                    !lnav_data.ld_other_files.empty()) {
                     ensure_view(&lnav_data.ld_views[LNV_SCHEMA]);
                 }
 
@@ -1674,7 +1679,7 @@ static void looper()
 
                 if (!session_loaded) {
                     load_session();
-                    if (!lnav_data.ld_session_file_names.empty()) {
+                    if (lnav_data.ld_session_save_time) {
                         std::string ago;
 
                         ago = time_ago(lnav_data.ld_session_save_time);
@@ -1684,19 +1689,23 @@ static void looper()
                                         (ANSI_NORM "; press Ctrl-R to reset session"));
                     }
 
-                    vector<pair<string, string> > msgs;
-
-                    execute_init_commands(ec, msgs);
-
-                    if (!msgs.empty()) {
-                        pair<string, string> last_msg = msgs.back();
-
-                        lnav_data.ld_rl_view->set_value(last_msg.first);
-                        lnav_data.ld_rl_view->set_alt_value(last_msg.second);
-                    }
-
                     session_loaded = true;
                 }
+
+                if (initial_build) {
+                    vector<pair<Result<string, string>, string>> cmd_results;
+
+                    execute_init_commands(ec, cmd_results);
+
+                    if (!cmd_results.empty()) {
+                        auto last_cmd_result = cmd_results.back();
+
+                        lnav_data.ld_rl_view->set_value(
+                            last_cmd_result.first.orElse(err_to_ok).unwrap());
+                        lnav_data.ld_rl_view->set_alt_value(
+                            last_cmd_result.second);
+                    }
+                }
             }
 
             if (lnav_data.ld_winched) {
@@ -1813,10 +1822,11 @@ int main(int argc, char *argv[])
     int lpc, c, retval = EXIT_SUCCESS;
 
     shared_ptr<piper_proc> stdin_reader;
-    const char *         stdin_out = NULL;
+    const char *         stdin_out = nullptr;
     int                  stdin_out_fd = -1;
     bool exec_stdin = false;
     const char *LANG = getenv("LANG");
+    filesystem::path stdin_tmp_path;
 
     if (LANG == nullptr || strcmp(LANG, "C") == 0) {
         setenv("LANG", "en_US.utf-8", 1);
@@ -1855,6 +1865,9 @@ int main(int argc, char *argv[])
 #endif
 
     lnav_data.ld_debug_log_name = "/dev/null";
+    lnav_data.ld_config_paths.emplace_back("/etc/lnav");
+    lnav_data.ld_config_paths.emplace_back(SYSCONFDIR "/lnav");
+    lnav_data.ld_config_paths.emplace_back(dotlnav_path());
     while ((c = getopt(argc, argv, "hHarRCc:I:iuf:d:nqtw:vVW")) != -1) {
         switch (c) {
         case 'h':
@@ -1921,7 +1934,7 @@ int main(int argc, char *argv[])
 
         case 'd':
             lnav_data.ld_debug_log_name = optarg;
-            lnav_log_level = LOG_LEVEL_TRACE;
+            lnav_log_level = lnav_log_level_t::TRACE;
             break;
 
         case 'a':
@@ -1990,14 +2003,15 @@ int main(int argc, char *argv[])
     add_global_vars(ec);
 
     if (lnav_data.ld_flags & LNF_UPDATE_FORMATS) {
-        if (!update_git_formats()) {
+        if (!update_installs_from_git()) {
             return EXIT_FAILURE;
         }
         return EXIT_SUCCESS;
     }
 
     if (lnav_data.ld_flags & LNF_INSTALL) {
-        string installed_path = dotlnav_path("formats/installed/");
+        auto formats_installed_path = dotlnav_path() / "formats/installed";
+        auto configs_installed_path = dotlnav_path() / "configs/installed";
 
         if (argc == 0) {
             fprintf(stderr, "error: expecting file format paths\n");
@@ -2006,7 +2020,9 @@ int main(int argc, char *argv[])
 
         for (lpc = 0; lpc < argc; lpc++) {
             if (endswith(argv[lpc], ".git")) {
-                install_git_format(argv[lpc]);
+                if (!install_from_git(argv[lpc])) {
+                    return EXIT_FAILURE;
+                }
                 continue;
             }
 
@@ -2015,28 +2031,46 @@ int main(int argc, char *argv[])
                 continue;
             }
 
-            vector<intern_string_t> format_list = load_format_file(argv[lpc], loader_errors);
-
-            if (!loader_errors.empty()) {
-                print_errors(loader_errors);
-                return EXIT_FAILURE;
-            }
-            if (format_list.empty()) {
-                fprintf(stderr, "error: format file is empty: %s\n", argv[lpc]);
+            auto file_type_result = detect_config_file_type(argv[lpc]);
+            if (file_type_result.isErr()) {
+                fprintf(stderr, "error: %s\n", file_type_result.unwrapErr().c_str());
                 return EXIT_FAILURE;
             }
+            auto file_type = file_type_result.unwrap();
 
-            string dst_name = format_list[0].to_string() + ".json";
-            string dst_path = installed_path + dst_name;
+            string dst_name;
+            if (file_type == config_file_type::CONFIG) {
+                dst_name = basename(argv[lpc]);
+            } else {
+                vector<intern_string_t> format_list = load_format_file(
+                    argv[lpc], loader_errors);
+
+                if (!loader_errors.empty()) {
+                    print_errors(loader_errors);
+                    return EXIT_FAILURE;
+                }
+                if (format_list.empty()) {
+                    fprintf(stderr, "error: format file is empty: %s\n",
+                            argv[lpc]);
+                    return EXIT_FAILURE;
+                }
+
+                dst_name = format_list[0].to_string() + ".json";
+            }
+            auto dst_path = (file_type == config_file_type::CONFIG ?
+                             configs_installed_path :
+                             formats_installed_path) /
+                            dst_name;
             auto_fd in_fd, out_fd;
 
             if ((in_fd = open(argv[lpc], O_RDONLY)) == -1) {
                 perror("unable to open file to install");
             }
-            else if ((out_fd = open(dst_path.c_str(),
-                    O_WRONLY | O_CREAT | O_TRUNC, 0644)) == -1) {
+            else if ((out_fd = openp(dst_path,
+                                     O_WRONLY | O_CREAT | O_TRUNC,
+                                     0644)) == -1) {
                 fprintf(stderr, "error: unable to open destination: %s -- %s\n",
-                        dst_path.c_str(), strerror(errno));
+                        dst_path.str().c_str(), strerror(errno));
             }
             else {
                 char buffer[2048];
@@ -2049,7 +2083,7 @@ int main(int argc, char *argv[])
                         written = write(out_fd, buffer, rc);
                         if (written == -1) {
                             fprintf(stderr,
-                                    "error: unable to install format file -- %s\n",
+                                    "error: unable to install file -- %s\n",
                                     strerror(errno));
                             exit(EXIT_FAILURE);
                         }
@@ -2058,7 +2092,7 @@ int main(int argc, char *argv[])
                     }
                 }
 
-                fprintf(stderr, "info: installed: %s\n", dst_path.c_str());
+                fprintf(stderr, "info: installed: %s\n", dst_path.str().c_str());
             }
         }
         return EXIT_SUCCESS;
@@ -2109,7 +2143,7 @@ int main(int argc, char *argv[])
         auto_mem<char, sqlite3_free> errmsg;
 
         if (sqlite3_exec(lnav_data.ld_db.in(),
-                         init_sql,
+                         (const char *) init_sql.bsf_data,
                          nullptr,
                          nullptr,
                          errmsg.out()) != SQLITE_OK) {
@@ -2205,11 +2239,10 @@ int main(int argc, char *argv[])
     lnav_data.ld_looping        = true;
     lnav_data.ld_mode           = LNM_PAGING;
 
-    if (isatty(STDIN_FILENO) && argc == 0 &&
+    if ((isatty(STDIN_FILENO) || is_dev_null(STDIN_FILENO)) && argc == 0 &&
         !(lnav_data.ld_flags & LNF__ALL)) {
         lnav_data.ld_flags |= LNF_SYSLOG;
     }
-
     if (lnav_data.ld_flags != 0) {
         char start_dir[FILENAME_MAX];
 
@@ -2232,6 +2265,51 @@ int main(int argc, char *argv[])
         }
     }
 
+    {
+        const auto internals_dir = getenv("DUMP_INTERNALS_DIR");
+
+        if (internals_dir) {
+            dump_schema_to(lnav_config_handlers, internals_dir, "config-v1.schema.json");
+            dump_schema_to(root_format_handler, internals_dir, "format-v1.schema.json");
+
+            execute_examples();
+
+            auto cmd_ref_path = filesystem::path(internals_dir) / "cmd-ref.rst";
+            auto cmd_file = unique_ptr<FILE, decltype(&fclose)>(fopen(cmd_ref_path.str().c_str(), "w+"), fclose);
+
+            if (cmd_file.get()) {
+                set<readline_context::command_t *> unique_cmds;
+
+                for (auto &cmd : lnav_commands) {
+                    if (unique_cmds.find(cmd.second) != unique_cmds.end()) {
+                        continue;
+                    }
+                    unique_cmds.insert(cmd.second);
+                    format_help_text_for_rst(cmd.second->c_help, eval_example,
+                                             cmd_file.get());
+                }
+            }
+
+            auto sql_ref_path = filesystem::path(internals_dir) / "sql-ref.rst";
+            auto sql_file = unique_ptr<FILE, decltype(&fclose)>(fopen(sql_ref_path.str().c_str(), "w+"), fclose);
+            set<help_text *> unique_sql_help;
+
+            if (sql_file.get()) {
+                for (auto &sql : sqlite_function_help) {
+                    if (unique_sql_help.find(sql.second) !=
+                        unique_sql_help.end()) {
+                        continue;
+                    }
+                    unique_sql_help.insert(sql.second);
+                    format_help_text_for_rst(*sql.second, eval_example,
+                                             sql_file.get());
+                }
+            }
+
+            return EXIT_SUCCESS;
+        }
+    }
+
     for (lpc = 0; lpc < argc; lpc++) {
         logfile_open_options default_loo;
         auto_mem<char> abspath;
@@ -2272,9 +2350,15 @@ int main(int argc, char *argv[])
                         "Cannot open fifo: %s -- %s\n",
                         argv[lpc],
                         strerror(errno));
+                retval = EXIT_FAILURE;
             } else {
                 auto fifo_piper = make_shared<piper_proc>(
-                    fifo_fd.release(), false);
+                    fifo_fd.release(),
+                    false,
+                    open_temp_file(system_tmpdir() / "lnav.fifo.XXXXXX")
+                        .then([](auto pair) { pair.first.remove_file(); })
+                        .expect("Cannot create temporary file for FIFO")
+                        .second);
                 int fifo_out_fd = fifo_piper->get_fd();
                 char desc[128];
 
@@ -2286,7 +2370,7 @@ int main(int argc, char *argv[])
                 lnav_data.ld_pipers.push_back(fifo_piper);
             }
         }
-        else if ((abspath = realpath(argv[lpc], NULL)) == NULL) {
+        else if ((abspath = realpath(argv[lpc], nullptr)) == nullptr) {
             perror("Cannot find file");
             retval = EXIT_FAILURE;
         }
@@ -2307,10 +2391,12 @@ int main(int argc, char *argv[])
         rescan_files(true);
         for (auto &ld_file : lnav_data.ld_files) {
             auto lf = ld_file;
+            logfile::rebuild_result_t rebuild_result;
 
-            lf->rebuild_index();
-
-            lf->rebuild_index();
+            do {
+                rebuild_result = lf->rebuild_index();
+            } while (rebuild_result == logfile::RR_NEW_LINES ||
+                     rebuild_result == logfile::RR_NEW_ORDER);
             log_format *fmt = lf->get_format();
             if (fmt == NULL) {
                 fprintf(stderr, "error:%s:no format found for file\n",
@@ -2318,17 +2404,18 @@ int main(int argc, char *argv[])
                 retval = EXIT_FAILURE;
                 continue;
             }
-            for (auto line_iter = lf->begin();
-                    line_iter != lf->end();
-                    ++line_iter) {
+            for (auto line_iter = lf->begin(); line_iter != lf->end(); ++line_iter) {
                 if (!line_iter->is_continued()) {
                     continue;
                 }
 
-                shared_buffer_ref sbr;
                 size_t partial_len;
 
-                lf->read_line(line_iter, sbr);
+                auto read_result = lf->read_line(line_iter);
+                if (read_result.isErr()) {
+                    continue;
+                }
+                shared_buffer_ref sbr = read_result.unwrap();
                 if (fmt->scan_for_partial(sbr, partial_len)) {
                     long line_number = distance(lf->begin(), line_iter);
                     string full_line(sbr.get_data(), sbr.length());
@@ -2365,16 +2452,40 @@ int main(int argc, char *argv[])
         retval = EXIT_FAILURE;
     }
 
-    if (!isatty(STDIN_FILENO) && !exec_stdin) {
+    if (!isatty(STDIN_FILENO) && !is_dev_null(STDIN_FILENO) && !exec_stdin) {
+        if (stdin_out == nullptr) {
+            auto pattern = dotlnav_path() / "stdin-captures/stdin.XXXXXX";
+
+            auto open_result = open_temp_file(pattern);
+            if (open_result.isErr()) {
+                fprintf(stderr,
+                        "Unable to open temporary file for stdin: %s",
+                        open_result.unwrapErr().c_str());
+                return EXIT_FAILURE;
+            }
+
+            auto temp_pair = open_result.unwrap();
+            stdin_tmp_path = temp_pair.first;
+            stdin_out_fd = temp_pair.second;
+        } else {
+            if ((stdin_out_fd = open(stdin_out, O_RDWR | O_CREAT | O_TRUNC, 0600)) == -1) {
+                perror("Unable to open output file for stdin");
+                return EXIT_FAILURE;
+            }
+        }
+
         stdin_reader = make_shared<piper_proc>(
-            STDIN_FILENO, lnav_data.ld_flags & LNF_TIMESTAMP, stdin_out);
-        stdin_out_fd = stdin_reader->get_fd();
+            STDIN_FILENO, lnav_data.ld_flags & LNF_TIMESTAMP, stdin_out_fd);
         lnav_data.ld_file_names["stdin"]
-            .with_fd(stdin_out_fd);
+            .with_fd(stdin_out_fd)
+            .with_include_in_session(false);
+        lnav_data.ld_pipers.push_back(stdin_reader);
+    }
+
+    if (!isatty(STDIN_FILENO) && isatty(STDOUT_FILENO)) {
         if (dup2(STDOUT_FILENO, STDIN_FILENO) == -1) {
             perror("cannot dup stdout to stdin");
         }
-        lnav_data.ld_pipers.push_back(stdin_reader);
     }
 
     if (lnav_data.ld_file_names.empty() &&
@@ -2424,8 +2535,7 @@ int main(int argc, char *argv[])
             }
 
             if (lnav_data.ld_flags & LNF_HEADLESS) {
-                std::vector<pair<string, string> > msgs;
-                std::vector<pair<string, string> >::iterator msg_iter;
+                std::vector<pair<Result<string, string>, string>> cmd_results;
                 textview_curses *log_tc, *text_tc, *tc;
                 bool found_error = false;
 
@@ -2450,21 +2560,19 @@ int main(int argc, char *argv[])
                 }
 
                 log_info("Executing initial commands");
-                execute_init_commands(lnav_data.ld_exec_context, msgs);
+                execute_init_commands(lnav_data.ld_exec_context, cmd_results);
                 wait_for_pipers();
                 lnav_data.ld_curl_looper.process_all();
                 rebuild_indexes();
 
-                for (msg_iter = msgs.begin();
-                     msg_iter != msgs.end();
-                     ++msg_iter) {
-                    if (strncmp("error:", msg_iter->first.c_str(), 6) == 0) {
-                        fprintf(stderr, "%s\n", msg_iter->first.c_str());
+                for (auto &pair : cmd_results) {
+                    if (pair.first.isErr()) {
+                        fprintf(stderr, "%s\n", pair.first.unwrapErr().c_str());
                         found_error = true;
                     }
-                    else if (startswith(msg_iter->first, "info:") &&
+                    else if (startswith(pair.first.unwrap(), "info:") &&
                              lnav_data.ld_flags & LNF_VERBOSE) {
-                        printf("%s\n", msg_iter->first.c_str());
+                        printf("%s\n", pair.first.unwrap().c_str());
                     }
                 }
 
@@ -2539,23 +2647,12 @@ int main(int argc, char *argv[])
                 }
             }
             else {
-                auto_fd errpipe[2];
 
                 lnav_data.ld_curl_looper.start();
 
                 init_session();
 
-                log_info("  session_id=%s", lnav_data.ld_session_id.c_str());
-
-                scan_sessions();
-
                 guard_termios gt(STDIN_FILENO);
-                auto_fd::pipe(errpipe);
-
-                dup2(errpipe[1], STDERR_FILENO);
-                errpipe[1].reset();
-                log_pipe_err(errpipe[0]);
-
                 lnav_log_orig_termios = gt.get_termios();
 
                 looper();
@@ -2579,32 +2676,32 @@ int main(int argc, char *argv[])
             }
         }
 
-        // When reading from stdin, dump out the last couple hundred lines so
-        // the user can have the text in their terminal history.
+        // When reading from stdin, tell the user where the capture file is
+        // stored so they can look at it later.
         if (stdin_out_fd != -1 &&
-                !(lnav_data.ld_flags & LNF_QUIET) &&
-                !(lnav_data.ld_flags & LNF_HEADLESS)) {
-            struct stat st;
+            stdin_out == nullptr &&
+            !(lnav_data.ld_flags & LNF_QUIET) &&
+            !(lnav_data.ld_flags & LNF_HEADLESS)) {
+            if (stdin_tmp_path.file_size() > MAX_STDIN_CAPTURE_SIZE) {
+                log_info("not saving large stdin capture -- %s",
+                         stdin_tmp_path.str().c_str());
+                stdin_tmp_path.remove_file();
+            } else {
+                auto home = getenv("HOME");
+                auto path_str = stdin_tmp_path.str();
 
-            fstat(stdin_out_fd, &st);
-            auto file_iter = find_if(lnav_data.ld_files.begin(),
-                                     lnav_data.ld_files.end(),
-                                     same_file(st));
-            if (file_iter != lnav_data.ld_files.end()) {
-                logfile::iterator line_iter;
-                auto lf = *file_iter;
-                string str;
-
-                for (line_iter = lf->begin();
-                     line_iter != lf->end();
-                     ++line_iter) {
-                    lf->read_line(line_iter, str);
-
-                    if (write(STDOUT_FILENO, str.c_str(), str.size()) == -1 ||
-                        write(STDOUT_FILENO, "\n", 1) == -1) {
-                        perror("3 write to STDOUT");
+                if (home != nullptr && startswith(path_str, home)) {
+                    path_str = path_str.substr(strlen(home));
+                    if (path_str[0] != '/') {
+                        path_str.insert(0, 1, '/');
                     }
+                    path_str.insert(0, 1, '~');
                 }
+
+                fprintf(stderr,
+                        "info: stdin was captured, you can reopen it using -- "
+                        "lnav %s\n",
+                        path_str.c_str());
             }
         }
     }
