@@ -37,13 +37,17 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <stdarg.h>
+#include <paths.h>
 
 #include <fstream>
 
 #include "auto_fd.hh"
 #include "lnav_util.hh"
-#include "pcrepp.hh"
+#include "pcrepp/pcrepp.hh"
 #include "lnav_config.hh"
+#include "base/result.h"
+#include "ansi_scrubber.hh"
+#include "view_curses.hh"
 
 using namespace std;
 
@@ -360,7 +364,6 @@ time_t tm2sec(const struct tm *t)
     }                          /* must be a valid time */
 }
 
-static const int MONSPERYEAR = 12;
 static const int SECSPERMIN = 60;
 static const int SECSPERHOUR = 60 * SECSPERMIN;
 static const int SECSPERDAY = 24 * SECSPERHOUR;
@@ -371,15 +374,17 @@ static const int EPOCH_YEAR = 1970;
 
 #define isleap(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
 
-static const int mon_lengths[2][MONSPERYEAR] = {
-        {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
-        {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
-} ;
-
 static const int year_lengths[2] = {
         365,
         366
-} ;
+};
+
+const unsigned short int mon_yday[2][13] = {
+    /* Normal years.  */
+    { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 },
+    /* Leap years.  */
+    { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 }
+};
 
 static void secs2wday(const struct timeval &tv, struct tm *res)
 {
@@ -407,7 +412,7 @@ struct tm *secs2tm(time_t *tim_p, struct tm *res)
     time_t lcltime;
     int y;
     int yleap;
-    const int *ip;
+    const unsigned short int *ip;
 
     /* base decision about std/dst time on current time */
     lcltime = *tim_p;
@@ -455,9 +460,11 @@ struct tm *secs2tm(time_t *tim_p, struct tm *res)
 
     res->tm_year = y - YEAR_BASE;
     res->tm_yday = days;
-    ip = mon_lengths[yleap];
-    for (res->tm_mon = 0; days >= ip[res->tm_mon]; ++res->tm_mon)
-        days -= ip[res->tm_mon];
+    ip = mon_yday[isleap(y)];
+    for (y = 11; days < (long int) ip[y]; --y)
+        continue;
+    days -= ip[y];
+    res->tm_mon = y;
     res->tm_mday = days + 1;
 
     res->tm_isdst = 0;
@@ -661,28 +668,26 @@ size_t strtonum<long>(long &num_out, const char *string, size_t len);
 template
 size_t strtonum<int>(int &num_out, const char *string, size_t len);
 
-string build_path(const vector<string> &paths)
+string build_path(const vector<filesystem::path> &paths)
 {
     string retval;
 
-    for (vector<string>::const_iterator path_iter = paths.begin();
-         path_iter != paths.end();
-         ++path_iter) {
-        if (path_iter->empty()) {
+    for (const auto &path : paths) {
+        if (path.empty()) {
             continue;
         }
         if (!retval.empty()) {
             retval += ":";
         }
-        retval += *path_iter;
+        retval += path.str();
     }
     retval += ":" + string(getenv("PATH"));
     return retval;
 }
 
-bool read_file(const char *filename, string &out)
+bool read_file(const filesystem::path &filename, string &out)
 {
-    std::ifstream sql_file(filename);
+    std::ifstream sql_file(filename.str());
 
     if (sql_file) {
         out.assign((std::istreambuf_iterator<char>(sql_file)),
@@ -721,3 +726,70 @@ size_t abbreviate_str(char *str, size_t len, size_t max_len)
 
     return len;
 }
+
+filesystem::path system_tmpdir()
+{
+    const char *tmpdir;
+
+    if ((tmpdir = getenv("TMPDIR")) == nullptr) {
+        tmpdir = _PATH_VARTMP;
+    }
+
+    return filesystem::path(tmpdir);
+}
+
+Result<std::pair<filesystem::path, int>, std::string>
+open_temp_file(const filesystem::path &pattern)
+{
+    auto pattern_str = pattern.str();
+    char pattern_copy[pattern_str.size() + 1];
+    int fd;
+
+    strcpy(pattern_copy, pattern_str.c_str());
+    if ((fd = mkstemp(pattern_copy)) == -1) {
+        throw Err(strerror(errno));
+    }
+
+    return Ok(make_pair(filesystem::path(pattern_copy), fd));
+}
+
+bool is_dev_null(const struct stat &st)
+{
+    struct stat null_stat;
+
+    stat("/dev/null", &null_stat);
+
+    return st.st_dev == null_stat.st_dev &&
+           st.st_ino == null_stat.st_ino;
+}
+
+bool is_dev_null(int fd)
+{
+    struct stat fd_stat;
+
+    fstat(fd, &fd_stat);
+    return is_dev_null(fd_stat);
+}
+
+std::string ok_prefix(std::string msg)
+{
+    if (msg.empty()) {
+        return msg;
+    }
+
+    return std::string(ANSI_COLOR(COLOR_GREEN) "\u2714" ANSI_NORM " ") + msg;
+}
+
+std::string err_prefix(const std::string msg)
+{
+    if (msg.empty()) {
+        return msg;
+    }
+
+    return std::string(ANSI_COLOR(COLOR_RED) "\u2718" ANSI_NORM " ") + msg;
+}
+
+Result<std::string, std::string> err_to_ok(const std::string msg)
+{
+    return Ok(err_prefix(msg));
+}
