@@ -34,8 +34,9 @@
 #include <string.h>
 #include <strings.h>
 
-#include "yajlpp.hh"
-#include "yajlpp_def.hh"
+#include "fmt/format.h"
+#include "yajlpp/yajlpp.hh"
+#include "yajlpp/yajlpp_def.hh"
 #include "sql_util.hh"
 #include "log_format.hh"
 #include "log_vtab_impl.hh"
@@ -45,21 +46,6 @@
 
 using namespace std;
 
-/*
- * Supported formats:
- *   generic
- *   syslog
- *   apache
- *   tcpdump
- *   strace
- *   vstrace
- *   csv (?)
- *   file system (?)
- *   plugins
- *   vmstat
- *   iostat
- */
-
 string_attr_type logline::L_PREFIX("prefix");
 string_attr_type logline::L_TIMESTAMP("timestamp");
 string_attr_type logline::L_FILE("file");
@@ -71,7 +57,7 @@ string_attr_type logline::L_META("meta");
 external_log_format::mod_map_t external_log_format::MODULE_FORMATS;
 std::vector<external_log_format *> external_log_format::GRAPH_ORDERED_FORMATS;
 
-struct line_range logline_value::origin_in_full_msg(const char *msg, size_t len) const
+struct line_range logline_value::origin_in_full_msg(const char *msg, ssize_t len) const
 {
     if (this->lv_sub_offset == 0) {
         return this->lv_origin;
@@ -161,7 +147,8 @@ bool log_format::next_format(pcre_format *fmt, int &index, int &locked_index)
     return retval;
 }
 
-const char *log_format::log_scanf(const char *line,
+const char *log_format::log_scanf(uint32_t line_number,
+                                  const char *line,
                                   size_t len,
                                   pcre_format *fmt,
                                   const char *time_fmt[],
@@ -199,7 +186,15 @@ const char *log_format::log_scanf(const char *line,
 
             if (retval) {
                 if (curr_fmt != pat_index) {
-                    this->lf_pattern_locks.emplace_back(0, curr_fmt);
+                    uint32_t lock_line;
+
+                    if (this->lf_pattern_locks.empty()) {
+                        lock_line = 0;
+                    } else {
+                        lock_line = line_number;
+                    }
+
+                    this->lf_pattern_locks.emplace_back(lock_line, curr_fmt);
                 }
                 this->lf_timestamp_flags = tm_out->et_flags;
                 done = true;
@@ -221,7 +216,6 @@ void log_format::check_for_new_year(std::vector<logline> &dst, exttm etm,
 
     time_t diff = dst.back().get_time() - log_tv.tv_sec;
     int off_year = 0, off_month = 0, off_day = 0, off_hour = 0;
-    std::vector<logline>::iterator iter;
     bool do_change = true;
 
     if (diff <= 0) {
@@ -244,8 +238,8 @@ void log_format::check_for_new_year(std::vector<logline> &dst, exttm etm,
     }
     log_debug("%d:detected time rollover; offsets=%d %d %d %d", dst.size(),
               off_year, off_month, off_day, off_hour);
-    for (iter = dst.begin(); iter != dst.end(); iter++) {
-        time_t     ot = iter->get_time();
+    for (auto &ll : dst) {
+        time_t     ot = ll.get_time();
         struct tm otm;
 
         gmtime_r(&ot, &otm);
@@ -253,7 +247,11 @@ void log_format::check_for_new_year(std::vector<logline> &dst, exttm etm,
         otm.tm_mon  -= off_month;
         otm.tm_mday -= off_day;
         otm.tm_hour -= off_hour;
-        iter->set_time(tm2sec(&otm));
+        auto new_time = tm2sec(&otm);
+        if (new_time == -1) {
+            continue;
+        }
+        ll.set_time(new_time);
     }
 }
 
@@ -393,23 +391,20 @@ static int json_array_end(void *ctx)
 
         sbr.subset(jlu->jlu_shared_buffer, jlu->jlu_sub_start,
             sub_end - jlu->jlu_sub_start);
-        jlu->jlu_format->jlf_line_values.push_back(
-            logline_value(field_name, sbr));
+        jlu->jlu_format->jlf_line_values.emplace_back(field_name, sbr, -1, jlu->jlu_format);
         jlu->jlu_format->jlf_line_values.back().lv_kind = logline_value::VALUE_JSON;
     }
 
     return 1;
 }
 
-static struct json_path_handler json_log_handlers[] = {
-    json_path_handler("^/\\w+$").
-    add_cb(read_json_null).
-    add_cb(read_json_bool).
-    add_cb(read_json_int).
-    add_cb(read_json_double).
-    add_cb(read_json_field),
-
-    json_path_handler()
+static struct json_path_container json_log_handlers = {
+    json_path_handler(pcrepp("\\w+"))
+        .add_cb(read_json_null)
+        .add_cb(read_json_bool)
+        .add_cb(read_json_int)
+        .add_cb(read_json_double)
+        .add_cb(read_json_field)
 };
 
 static int rewrite_json_field(yajlpp_parse_context *ypc, const unsigned char *str, size_t len);
@@ -422,7 +417,7 @@ static int rewrite_json_null(yajlpp_parse_context *ypc)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.emplace_back(field_name);
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name, jlu->jlu_format);
 
     return 1;
 }
@@ -435,7 +430,7 @@ static int rewrite_json_bool(yajlpp_parse_context *ypc, int val)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.emplace_back(field_name, (bool)val);
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name, (bool)val, jlu->jlu_format);
 
     return 1;
 }
@@ -448,7 +443,7 @@ static int rewrite_json_int(yajlpp_parse_context *ypc, long long val)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.emplace_back(field_name, (int64_t)val);
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name, (int64_t)val, jlu->jlu_format);
 
     return 1;
 }
@@ -461,20 +456,18 @@ static int rewrite_json_double(yajlpp_parse_context *ypc, double val)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.emplace_back(field_name, val);
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name, val, jlu->jlu_format);
 
     return 1;
 }
 
-static struct json_path_handler json_log_rewrite_handlers[] = {
-    json_path_handler("^/\\w+$")
+static struct json_path_container json_log_rewrite_handlers = {
+    json_path_handler(pcrepp("\\w+"))
         .add_cb(rewrite_json_null)
         .add_cb(rewrite_json_bool)
         .add_cb(rewrite_json_int)
         .add_cb(rewrite_json_double)
-        .add_cb(rewrite_json_field),
-
-    json_path_handler()
+        .add_cb(rewrite_json_field)
 };
 
 bool external_log_format::scan_for_partial(shared_buffer_ref &sbr, size_t &len_out)
@@ -500,43 +493,32 @@ bool external_log_format::scan_for_partial(shared_buffer_ref &sbr, size_t &len_o
     return (int)len_out > pat->p_timestamp_end;
 }
 
-log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *> lf,
+log_format::scan_result_t external_log_format::scan(logfile &lf,
                                                     std::vector<logline> &dst,
-                                                    off_t offset,
+                                                    const line_info &li,
                                                     shared_buffer_ref &sbr)
 {
     if (this->elf_type == ELF_TYPE_JSON) {
         yajlpp_parse_context &ypc = *(this->jlf_parse_context);
-        logline ll(offset, 0, 0, LEVEL_INFO);
+        logline ll(li.li_file_range.fr_offset, 0, 0, LEVEL_INFO);
         yajl_handle handle = this->jlf_yajl_handle.in();
         json_log_userdata jlu(sbr);
 
-        if (sbr.empty()) {
+        if (li.li_partial) {
+            log_debug("skipping partial line at offset %d", li.li_file_range.fr_offset);
             return log_format::SCAN_INCOMPLETE;
         }
 
         const auto *line_data = (const unsigned char *) sbr.get_data();
-        size_t line_end = sbr.length();
-
-        if (line_end > 0) {
-            line_end -= 1;
-        }
-        while (line_end > 0 && isspace(line_data[line_end])) {
-            line_end -= 1;
-        }
-
-        if (line_end == 0 || line_data[line_end] != '}') {
-            return log_format::SCAN_INCOMPLETE;
-        }
 
         yajl_reset(handle);
-        ypc.set_static_handler(json_log_handlers[0]);
+        ypc.set_static_handler(json_log_handlers.jpc_children[0]);
         ypc.ypc_userdata = &jlu;
         ypc.ypc_ignore_unused = true;
         ypc.ypc_alt_callbacks.yajl_start_array = json_array_start;
         ypc.ypc_alt_callbacks.yajl_start_map = json_array_start;
-        ypc.ypc_alt_callbacks.yajl_end_array = NULL;
-        ypc.ypc_alt_callbacks.yajl_end_map = NULL;
+        ypc.ypc_alt_callbacks.yajl_end_array = nullptr;
+        ypc.ypc_alt_callbacks.yajl_end_map = nullptr;
         jlu.jlu_format = this;
         jlu.jlu_base_line = &ll;
         jlu.jlu_line_value = sbr.get_data();
@@ -548,24 +530,40 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
                 return log_format::SCAN_NO_MATCH;
             }
 
+            jlu.jlu_sub_line_count += this->jlf_line_format_init_count;
             for (int lpc = 0; lpc < jlu.jlu_sub_line_count; lpc++) {
                 ll.set_sub_offset(lpc);
                 if (lpc > 0) {
                     ll.set_level((log_level_t) (ll.get_level_and_flags() |
                         LEVEL_CONTINUED));
                 }
-                dst.push_back(ll);
+                dst.emplace_back(ll);
             }
         }
         else {
             unsigned char *msg;
+            int line_count = 1;
 
             msg = yajl_get_error(handle, 1, (const unsigned char *)sbr.get_data(), sbr.length());
-            if (msg != NULL) {
-                log_debug("Unable to parse line at offset %d: %s", offset, msg);
+            if (msg != nullptr) {
+                log_debug("Unable to parse line at offset %d: %s", li.li_file_range.fr_offset, msg);
+                line_count = count(msg, msg + strlen((char *) msg), '\n');
                 yajl_free_error(handle, msg);
             }
-            return log_format::SCAN_INCOMPLETE;
+            if (!this->lf_specialized) {
+                return log_format::SCAN_NO_MATCH;
+            }
+            for (int lpc = 0; lpc < line_count; lpc++) {
+                log_level_t level = LEVEL_ERROR;
+
+                ll.set_time(dst.back().get_time());
+                if (lpc > 0) {
+                    level = (log_level_t) (level | LEVEL_CONTINUED);
+                }
+                ll.set_level(level);
+                ll.set_sub_offset(lpc);
+                dst.emplace_back(ll);
+            }
         }
 
         return log_format::SCAN_MATCH;
@@ -584,9 +582,8 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
             continue;
         }
 
-        if (!pat->match(pc, pi)) {
+        if (!pat->match(pc, pi, PCRE_NO_UTF8_CHECK)) {
             if (!this->lf_pattern_locks.empty() && pat_index != -1) {
-                log_debug("no match on pattern %d", pat_index);
                 curr_fmt = -1;
                 pat_index = -1;
             }
@@ -684,11 +681,18 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
             }
         }
 
-        dst.emplace_back(offset, log_tv, level, mod_index, opid);
+        dst.emplace_back(li.li_file_range.fr_offset, log_tv, level, mod_index, opid);
 
         if (orig_lock != curr_fmt) {
+            uint32_t lock_line;
+
             log_debug("changing pattern lock %d -> %d", orig_lock, curr_fmt);
-            this->lf_pattern_locks.emplace_back(dst.size() - 1, curr_fmt);
+            if (this->lf_pattern_locks.empty()) {
+                lock_line = 0;
+            } else {
+                lock_line = dst.size() - 1;
+            }
+            this->lf_pattern_locks.emplace_back(lock_line, curr_fmt);
         }
         return log_format::SCAN_MATCH;
     }
@@ -757,7 +761,7 @@ void external_log_format::annotate(uint64_t line_number, shared_buffer_ref &line
     int pat_index = this->pattern_index_for_line(line_number);
     pattern &pat = *this->elf_pattern_order[pat_index];
 
-    if (!pat.p_pcre->match(pc, pi)) {
+    if (!pat.p_pcre->match(pc, pi, PCRE_NO_UTF8_CHECK)) {
         // A continued line still needs a body.
         lr.lr_start = 0;
         lr.lr_end = line.length();
@@ -767,16 +771,18 @@ void external_log_format::annotate(uint64_t line_number, shared_buffer_ref &line
 
     if (!pat.p_module_format) {
         cap = pc[pat.p_timestamp_field_index];
-        lr.lr_start = cap->c_begin;
-        lr.lr_end = cap->c_end;
-        sa.push_back(string_attr(lr, &logline::L_TIMESTAMP));
+        if (cap->is_valid()) {
+            lr.lr_start = cap->c_begin;
+            lr.lr_end = cap->c_end;
+            sa.emplace_back(lr, &logline::L_TIMESTAMP);
+        }
 
         if (pat.p_module_field_index != -1) {
             module_cap = pc[pat.p_module_field_index];
             if (module_cap != NULL && module_cap->is_valid()) {
                 lr.lr_start = module_cap->c_begin;
                 lr.lr_end = module_cap->c_end;
-                sa.push_back(string_attr(lr, &logline::L_MODULE));
+                sa.emplace_back(lr, &logline::L_MODULE);
             }
         }
 
@@ -784,7 +790,7 @@ void external_log_format::annotate(uint64_t line_number, shared_buffer_ref &line
         if (cap != NULL && cap->is_valid()) {
             lr.lr_start = cap->c_begin;
             lr.lr_end = cap->c_end;
-            sa.push_back(string_attr(lr, &logline::L_OPID));
+            sa.emplace_back(lr, &logline::L_OPID);
         }
     }
 
@@ -797,14 +803,13 @@ void external_log_format::annotate(uint64_t line_number, shared_buffer_ref &line
         lr.lr_start = line.length();
         lr.lr_end = line.length();
     }
-    sa.push_back(string_attr(lr, &textview_curses::SA_BODY));
+    sa.emplace_back(lr, &textview_curses::SA_BODY);
 
     for (size_t lpc = 0; lpc < pat.p_value_by_index.size(); lpc++) {
         const indexed_value_def &ivd = pat.p_value_by_index[lpc];
         const struct scaling_factor *scaling = NULL;
         pcre_context::capture_t *cap = pc[ivd.ivd_index];
         const value_def &vd = *ivd.ivd_value_def;
-        shared_buffer_ref field;
 
         if (ivd.ivd_unit_field_index >= 0) {
             pcre_context::iterator unit_cap = pc[ivd.ivd_unit_field_index];
@@ -823,18 +828,20 @@ void external_log_format::annotate(uint64_t line_number, shared_buffer_ref &line
             }
         }
 
-        field.subset(line, cap->c_begin, cap->length());
-
-        values.emplace_back(vd.vd_name,
-                            vd.vd_kind,
-                            field,
-                            vd.vd_identifier,
-                            scaling,
-                            vd.vd_column,
-                            cap->c_begin,
-                            cap->c_end,
-                            pat.p_module_format,
-                            this);
+        if (cap->is_valid()) {
+            values.emplace_back(vd.vd_name,
+                                vd.vd_kind,
+                                line,
+                                vd.vd_identifier,
+                                scaling,
+                                vd.vd_column,
+                                cap->c_begin,
+                                cap->c_end,
+                                pat.p_module_format,
+                                this);
+        } else {
+            values.emplace_back(vd.vd_name, this);
+        }
         values.back().lv_hidden = vd.vd_hidden || vd.vd_user_hidden;
     }
 
@@ -891,11 +898,12 @@ void external_log_format::rewrite(exec_context &ec,
             continue;
         }
 
-        ec.ec_source.emplace(this->elf_name.to_string() +
-                             ":" +
-                             vd_iter->first.to_string(),
-                             1);
-        string field_value = execute_any(ec, vd.vd_rewriter);
+        auto _sg = ec.enter_source(this->elf_name.to_string() +
+                                   ":" +
+                                   vd_iter->first.to_string(),
+                                   1);
+        string field_value = execute_any(ec, vd.vd_rewriter)
+            .orElse(err_to_ok).unwrap();
         struct line_range adj_origin = iter->origin_in_full_msg(
             value_out.c_str(), value_out.length());
 
@@ -1017,7 +1025,7 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
         this->jlf_line_attrs.clear();
 
         yajl_reset(handle);
-        ypc.set_static_handler(json_log_rewrite_handlers[0]);
+        ypc.set_static_handler(json_log_rewrite_handlers.jpc_children[0]);
         ypc.ypc_userdata = &jlu;
         ypc.ypc_ignore_unused = true;
         ypc.ypc_alt_callbacks.yajl_start_array = json_array_start;
@@ -1031,8 +1039,20 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
 
         yajl_status parse_status = yajl_parse(handle,
             (const unsigned char *)sbr.get_data(), sbr.length());
-        if (parse_status == yajl_status_ok &&
-            yajl_complete_parse(handle) == yajl_status_ok) {
+        if (parse_status != yajl_status_ok ||
+            yajl_complete_parse(handle) != yajl_status_ok) {
+            unsigned char *msg;
+            string full_msg;
+
+            msg = yajl_get_error(handle, 1, (const unsigned char *)sbr.get_data(), sbr.length());
+            if (msg != nullptr) {
+                full_msg = fmt::format("lnav: unable to parse line at offset {}: {}", ll.get_offset(), msg);
+                yajl_free_error(handle, msg);
+            }
+
+            this->jlf_cached_line.resize(full_msg.size());
+            memcpy(this->jlf_cached_line.data(), full_msg.data(), full_msg.size());
+        } else {
             std::vector<logline_value>::iterator lv_iter;
             bool used_values[this->jlf_line_values.size()];
             struct line_range lr;
@@ -1054,6 +1074,7 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
                 }
             }
 
+            int sub_offset = 1 + this->jlf_line_format_init_count;
             for (const auto &jfe : this->jlf_line_format) {
                 static const intern_string_t ts_field = intern_string::lookup("__timestamp__", -1);
                 static const intern_string_t level_field = intern_string::lookup("__level__");
@@ -1107,6 +1128,7 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
                             }
                         }
                         else {
+                            sub_offset += count(str.begin(), str.end(), '\n');
                             this->json_append(jfe, str.c_str(), str.size());
                         }
 
@@ -1117,16 +1139,16 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
                         }
 
                         if (lv_iter->lv_name == this->lf_timestamp_field) {
-                            this->jlf_line_attrs.push_back(
-                                string_attr(lr, &logline::L_TIMESTAMP));
+                            this->jlf_line_attrs.emplace_back(
+                                lr, &logline::L_TIMESTAMP);
                         }
                         else if (lv_iter->lv_name == this->elf_body_field) {
-                            this->jlf_line_attrs.push_back(
-                                string_attr(lr, &textview_curses::SA_BODY));
+                            this->jlf_line_attrs.emplace_back(
+                                lr, &textview_curses::SA_BODY);
                         }
                         else if (lv_iter->lv_name == this->elf_opid_field) {
-                            this->jlf_line_attrs.push_back(
-                                    string_attr(lr, &logline::L_OPID));
+                            this->jlf_line_attrs.emplace_back(
+                                    lr, &logline::L_OPID);
                         }
                         lv_iter->lv_origin = lr;
                         used_values[distance(this->jlf_line_values.begin(),
@@ -1196,7 +1218,6 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
                 }
             }
             this->json_append_to_cache("\n", 1);
-            int sub_offset = 1;
 
             for (size_t lpc = 0; lpc < this->jlf_line_values.size(); lpc++) {
                 static const intern_string_t body_name = intern_string::lookup(
@@ -1233,33 +1254,34 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
                          nl_pos < str.size());
             }
 
-            this->jlf_line_offsets.push_back(0);
-            for (size_t lpc = 0; lpc < this->jlf_cached_line.size(); lpc++) {
-                if (this->jlf_cached_line[lpc] == '\n') {
-                    this->jlf_line_offsets.push_back(lpc);
-                }
-            }
-            this->jlf_line_offsets.push_back(this->jlf_cached_line.size());
         }
 
+        this->jlf_line_offsets.push_back(0);
+        for (size_t lpc = 0; lpc < this->jlf_cached_line.size(); lpc++) {
+            if (this->jlf_cached_line[lpc] == '\n') {
+                this->jlf_line_offsets.push_back(lpc + 1);
+            }
+        }
+        this->jlf_line_offsets.push_back(this->jlf_cached_line.size());
         this->jlf_cached_offset = ll.get_offset();
         this->jlf_cached_full = full_message;
     }
 
     off_t this_off = 0, next_off = 0;
 
-    if (!this->jlf_line_offsets.empty()) {
+    if (!this->jlf_line_offsets.empty() && ll.get_sub_offset() < this->jlf_line_offsets.size()) {
         require(ll.get_sub_offset() < this->jlf_line_offsets.size());
 
         this_off = this->jlf_line_offsets[ll.get_sub_offset()];
-        if (this->jlf_cached_line[this_off] == '\n') {
-            this_off += 1;
-        }
         if ((ll.get_sub_offset() + 1) < (int)this->jlf_line_offsets.size()) {
             next_off = this->jlf_line_offsets[ll.get_sub_offset() + 1];
         }
         else {
-            next_off = this_off;
+            next_off = this->jlf_cached_line.size();
+        }
+        if (next_off > 0 && this->jlf_cached_line[next_off - 1] == '\n' &&
+            this_off != next_off) {
+            next_off -= 1;
         }
     }
 
@@ -1270,7 +1292,7 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
     }
     else {
         sbr.share(this->jlf_share_manager,
-                  &this->jlf_cached_line[0] + this_off,
+                  this->jlf_cached_line.data() + this_off,
                   next_off - this_off);
     }
 }
@@ -1369,21 +1391,21 @@ void external_log_format::build(std::vector<std::string> &errors) {
 
             auto value_iter = this->elf_value_defs.find(name);
             if (value_iter != this->elf_value_defs.end()) {
-                auto &vd = *value_iter->second;
+                auto vd = value_iter->second;
                 indexed_value_def ivd;
 
                 ivd.ivd_index = name_iter->index();
-                if (!vd.vd_unit_field.empty()) {
+                if (!vd->vd_unit_field.empty()) {
                     ivd.ivd_unit_field_index = pat.p_pcre->name_index(
-                        vd.vd_unit_field.get());
+                        vd->vd_unit_field.get());
                 }
                 else {
                     ivd.ivd_unit_field_index = -1;
                 }
-                if (!vd.vd_internal && vd.vd_column == -1) {
-                    vd.vd_column = this->elf_column_count++;
+                if (!vd->vd_internal && vd->vd_column == -1) {
+                    vd->vd_column = this->elf_column_count++;
                 }
-                ivd.ivd_value_def = value_iter->second.get();
+                ivd.ivd_value_def = vd;
                 pat.p_value_by_index.push_back(ivd);
             }
         }
@@ -1392,10 +1414,10 @@ void external_log_format::build(std::vector<std::string> &errors) {
 
         for (int lpc = 0; lpc < (int)pat.p_value_by_index.size(); lpc++) {
             auto &ivd = pat.p_value_by_index[lpc];
-            auto &vd = *ivd.ivd_value_def;
+            auto vd = ivd.ivd_value_def;
 
-            if (!vd.vd_foreign_key && !vd.vd_identifier) {
-                switch (vd.vd_kind) {
+            if (!vd->vd_foreign_key && !vd->vd_identifier) {
+                switch (vd->vd_kind) {
                     case logline_value::VALUE_INTEGER:
                     case logline_value::VALUE_FLOAT:
                         pat.p_numeric_value_indexes.push_back(lpc);
@@ -1465,26 +1487,26 @@ void external_log_format::build(std::vector<std::string> &errors) {
 
     stable_sort(this->elf_level_pairs.begin(), this->elf_level_pairs.end());
 
-    for (auto &elf_value_def : this->elf_value_defs) {
+    for (auto &vd : this->elf_value_def_order) {
         std::vector<std::string>::iterator act_iter;
 
-        if (!elf_value_def.second->vd_internal &&
-            elf_value_def.second->vd_column == -1) {
-            elf_value_def.second->vd_column = this->elf_column_count++;
+        if (!vd->vd_internal &&
+            vd->vd_column == -1) {
+            vd->vd_column = this->elf_column_count++;
         }
 
-        if (elf_value_def.second->vd_kind == logline_value::VALUE_UNKNOWN) {
-            elf_value_def.second->vd_kind = logline_value::VALUE_TEXT;
+        if (vd->vd_kind == logline_value::VALUE_UNKNOWN) {
+            vd->vd_kind = logline_value::VALUE_TEXT;
         }
 
-        for (act_iter = elf_value_def.second->vd_action_list.begin();
-             act_iter != elf_value_def.second->vd_action_list.end();
+        for (act_iter = vd->vd_action_list.begin();
+             act_iter != vd->vd_action_list.end();
              ++act_iter) {
             if (this->lf_action_defs.find(*act_iter) ==
                 this->lf_action_defs.end()) {
                 errors.push_back("error:" +
                                  this->elf_name.to_string() + ":" +
-                                     elf_value_def.first.get() +
+                                     vd->vd_name.get() +
                                  ": cannot find action -- " + (*act_iter));
             }
         }
@@ -1618,7 +1640,9 @@ void external_log_format::build(std::vector<std::string> &errors) {
                                      this->elf_name.to_string() +
                                      ":partial sample matched -- " +
                                          elf_sample.s_line.substr(0, partial_len));
-                    errors.push_back("error:  against pattern -- " +
+                    errors.push_back("error:  against pattern " +
+                                     (*pat_iter)->p_config_path +
+                                     " -- " +
                                      (*pat_iter)->p_string);
                 }
                 else {
@@ -1656,7 +1680,13 @@ void external_log_format::build(std::vector<std::string> &errors) {
         static const intern_string_t level_field = intern_string::lookup("__level__");
         json_format_element &jfe = *iter;
 
-        if (jfe.jfe_value.empty() && !jfe.jfe_ts_format.empty()) {
+        if (!jfe.jfe_ts_format.empty()) {
+            if (!jfe.jfe_value.empty() && jfe.jfe_value != ts) {
+                log_warning("%s:line-format[%d]:ignoring field '%s' since "
+                            "timestamp-format was used",
+                            this->elf_name.get(), format_index,
+                            jfe.jfe_value.get());
+            }
             jfe.jfe_value = ts;
         }
 
@@ -1673,13 +1703,19 @@ void external_log_format::build(std::vector<std::string> &errors) {
                     snprintf(index_str, sizeof(index_str), "%d", format_index);
                     errors.push_back("error:" +
                                      this->elf_name.to_string() +
-                                     ":line_format[" +
+                                     ":line-format[" +
                                      index_str +
                                      "]:line format variable is not defined -- " +
                                      jfe.jfe_value.to_string());
                 }
                 break;
             }
+            case JLF_CONSTANT:
+                this->jlf_line_format_init_count +=
+                    std::count(jfe.jfe_default_value.begin(),
+                               jfe.jfe_default_value.end(),
+                               '\n');
+                break;
             default:
                 break;
         }
@@ -1723,7 +1759,7 @@ void external_log_format::build(std::vector<std::string> &errors) {
         }
 
         pcre *code = pcre_compile(pattern.c_str(),
-                                  0,
+                                  PCRE_CASELESS,
                                   &errptr,
                                   &eoff,
                                   nullptr);
@@ -1828,19 +1864,20 @@ public:
         const external_log_format &elf = this->elt_format;
 
         cols.resize(elf.elf_column_count);
-        for (const auto &elf_value_def : elf.elf_value_defs) {
-            const auto &vd = *elf_value_def.second;
-            pair<int, unsigned int> type_pair = log_vtab_impl::logline_value_to_sqlite_type(vd.vd_kind);
+        for (const auto &vd : elf.elf_value_def_order) {
+            pair<int, unsigned int> type_pair = log_vtab_impl::logline_value_to_sqlite_type(vd->vd_kind);
 
-            if (vd.vd_column == -1) {
+            if (vd->vd_column == -1) {
                 continue;
             }
 
-            cols[vd.vd_column].vc_name = vd.vd_name.get();
-            cols[vd.vd_column].vc_type = type_pair.first;
-            cols[vd.vd_column].vc_subtype = type_pair.second;
-            cols[vd.vd_column].vc_collator = vd.vd_collate.c_str();
-            cols[vd.vd_column].vc_comment = vd.vd_description;
+            require(0 <= vd->vd_column && vd->vd_column < elf.elf_column_count);
+
+            cols[vd->vd_column].vc_name = vd->vd_name.get();
+            cols[vd->vd_column].vc_type = type_pair.first;
+            cols[vd->vd_column].vc_subtype = type_pair.second;
+            cols[vd->vd_column].vc_collator = vd->vd_collate;
+            cols[vd->vd_column].vc_comment = vd->vd_description;
         }
     };
 
@@ -1850,7 +1887,7 @@ public:
 
         for (const auto &elf_value_def : this->elt_format.elf_value_defs) {
             if (elf_value_def.second->vd_foreign_key) {
-                keys_inout.push_back(elf_value_def.first.to_string());
+                keys_inout.emplace_back(elf_value_def.first.to_string());
             }
         }
     };
@@ -1864,7 +1901,7 @@ public:
             return true;
         }
 
-        content_line_t    cl(lss.at(lc.lc_curr_line));
+        content_line_t cl(lss.at(lc.lc_curr_line));
         shared_ptr<logfile> lf = lss.find(cl);
         auto lf_iter = lf->begin() + cl;
         uint8_t mod_id = lf_iter->get_module_id();
@@ -1879,38 +1916,38 @@ public:
         if (format->get_name() == this->lfvi_format.get_name()) {
             return true;
         } else if (mod_id && mod_id == this->lfvi_format.lf_mod_index) {
-            std::vector<logline_value> values;
-            shared_buffer_ref body_ref;
-            struct line_range mod_name_range;
-            intern_string_t mod_name;
-            shared_buffer_ref line;
+            return lf->read_line(lf_iter).map([this, format, cl](auto line) {
+                std::vector<logline_value> values;
+                shared_buffer_ref body_ref;
+                struct line_range mod_name_range;
+                intern_string_t mod_name;
 
-            lf->read_line(lf_iter, line);
-            this->vi_attrs.clear();
-            format->annotate(cl, line, this->vi_attrs, values, false);
-            this->elt_container_body = find_string_attr_range(this->vi_attrs, &textview_curses::SA_BODY);
-            if (!this->elt_container_body.is_valid()) {
-                return false;
-            }
-            this->elt_container_body.ltrim(line.get_data());
-            body_ref.subset(line,
-                            this->elt_container_body.lr_start,
-                            this->elt_container_body.length());
-            mod_name_range = find_string_attr_range(this->vi_attrs,
-                                                    &logline::L_MODULE);
-            if (!mod_name_range.is_valid()) {
-                return false;
-            }
-            mod_name = intern_string::lookup(
+                this->vi_attrs.clear();
+                format->annotate(cl, line, this->vi_attrs, values, false);
+                this->elt_container_body = find_string_attr_range(this->vi_attrs, &textview_curses::SA_BODY);
+                if (!this->elt_container_body.is_valid()) {
+                    return false;
+                }
+                this->elt_container_body.ltrim(line.get_data());
+                body_ref.subset(line,
+                                this->elt_container_body.lr_start,
+                                this->elt_container_body.length());
+                mod_name_range = find_string_attr_range(this->vi_attrs,
+                                                        &logline::L_MODULE);
+                if (!mod_name_range.is_valid()) {
+                    return false;
+                }
+                mod_name = intern_string::lookup(
                     &line.get_data()[mod_name_range.lr_start],
                     mod_name_range.length());
-            this->vi_attrs.clear();
-            this->elt_module_format = external_log_format::MODULE_FORMATS[mod_name];
-            if (!this->elt_module_format.mf_mod_format) {
-                return false;
-            }
-            return this->elt_module_format.mf_mod_format->get_name() ==
-                    this->lfvi_format.get_name();
+                this->vi_attrs.clear();
+                this->elt_module_format = external_log_format::MODULE_FORMATS[mod_name];
+                if (!this->elt_module_format.mf_mod_format) {
+                    return false;
+                }
+                return this->elt_module_format.mf_mod_format->get_name() ==
+                       this->lfvi_format.get_name();
+            }).unwrapOr(false);
         }
 
         return false;
