@@ -33,7 +33,7 @@
 
 #include <stdio.h>
 
-#include "pcrepp.hh"
+#include "pcrepp/pcrepp.hh"
 #include "sql_util.hh"
 #include "log_format.hh"
 #include "log_vtab_impl.hh"
@@ -80,7 +80,7 @@ static string scrub_rdns(const string &str)
 }
 
 class generic_log_format : public log_format {
-    static pcrepp &scrub_pattern(void)
+    static pcrepp &scrub_pattern()
     {
         static pcrepp SCRUB_PATTERN(
             "\\d+-(\\d+-\\d+ \\d+:\\d+:\\d+(?:,\\d+)?:)\\w+:(.*)");
@@ -116,7 +116,7 @@ class generic_log_format : public log_format {
         return get_pcre_log_formats()[pat_index].name;
     }
 
-    intern_string_t get_name() const {
+    const intern_string_t get_name() const {
         return intern_string::lookup("generic_log");
     };
 
@@ -137,9 +137,9 @@ class generic_log_format : public log_format {
         }
     };
 
-    scan_result_t scan(nonstd::optional<logfile *> lf,
+    scan_result_t scan(logfile &lf,
                        vector<logline> &dst,
-                       off_t offset,
+                       const line_info &li,
                        shared_buffer_ref &sbr)
     {
         struct exttm log_time;
@@ -148,6 +148,7 @@ class generic_log_format : public log_format {
         const char *last_pos;
 
         if ((last_pos = this->log_scanf(
+                dst.size(),
                 sbr.get_data(),
                 sbr.length(),
                 get_pcre_log_formats(),
@@ -166,7 +167,7 @@ class generic_log_format : public log_format {
                 this->check_for_new_year(dst, log_time, log_tv);
             }
 
-            dst.emplace_back(offset, log_tv, level_val);
+            dst.emplace_back(li.li_file_range.fr_offset, log_tv, level_val);
             return SCAN_MATCH;
         }
 
@@ -354,20 +355,19 @@ public:
         const intern_string_t fd_name;
         logline_value::kind_t fd_kind;
         bool fd_identifier;
-        const char *fd_collator;
+        std::string fd_collator;
         int fd_numeric_index;
 
         field_def(const intern_string_t name)
             : fd_name(name),
               fd_kind(logline_value::VALUE_TEXT),
               fd_identifier(false),
-              fd_collator(nullptr),
               fd_numeric_index(-1) {
         };
 
         field_def &with_kind(logline_value::kind_t kind,
                              bool identifier = false,
-                             const char *collator = nullptr) {
+                             const std::string &collator = "") {
             this->fd_kind = kind;
             this->fd_identifier = identifier;
             this->fd_collator = collator;
@@ -385,20 +385,20 @@ public:
         this->lf_time_ordered = false;
     };
 
-    intern_string_t get_name(void) const {
-        static intern_string_t name = intern_string::lookup("bro");
+    const intern_string_t get_name() const {
+        static const intern_string_t name(intern_string::lookup("bro"));
 
         return this->blf_format_name.empty() ? name : this->blf_format_name;
     };
 
-    virtual void clear(void) {
+    virtual void clear() {
         this->log_format::clear();
         this->blf_format_name.clear();
         this->blf_field_defs.clear();
     };
 
     scan_result_t scan_int(std::vector<logline> &dst,
-                           off_t offset,
+                           const line_info &li,
                            shared_buffer_ref &sbr) {
         static const intern_string_t STATUS_CODE = intern_string::lookup("bro_status_code");
         static const intern_string_t TS = intern_string::lookup("bro_ts");
@@ -463,31 +463,37 @@ public:
         }
 
         if (found_ts) {
-            dst.emplace_back(offset, tv, level, 0, opid);
+            dst.emplace_back(li.li_file_range.fr_offset, tv, level, 0, opid);
             return SCAN_MATCH;
         } else {
             return SCAN_NO_MATCH;
         }
     }
 
-    scan_result_t scan(nonstd::optional<logfile *> lf,
+    scan_result_t scan(logfile &lf,
                        std::vector<logline> &dst,
-                       off_t offset,
+                       const line_info &li,
                        shared_buffer_ref &sbr) {
         static pcrepp SEP_RE(R"(^#separator\s+(.+))");
 
         if (!this->blf_format_name.empty()) {
-            return this->scan_int(dst, offset, sbr);
+            return this->scan_int(dst, li, sbr);
         }
 
-        if (dst.empty() || dst.size() > 20 || sbr.empty() || sbr.get_data()[0] == '#' || !lf) {
+        if (dst.empty() || dst.size() > 20 || sbr.empty() || sbr.get_data()[0] == '#') {
             return SCAN_NO_MATCH;
         }
 
         pcre_context_static<20> pc;
         auto line_iter = dst.begin();
-        string line = lf.value()->read_line(line_iter);
-        pcre_input pi(line);
+        auto read_result = lf.read_line(line_iter);
+
+        if (read_result.isErr()) {
+            return SCAN_NO_MATCH;
+        }
+
+        auto line = read_result.unwrap();
+        pcre_input pi(line.get_data(), 0, line.length());
 
         if (!SEP_RE.match(pc, pi)) {
             return SCAN_NO_MATCH;
@@ -499,8 +505,14 @@ public:
         this->blf_separator = intern_string::lookup(sep);
 
         for (++line_iter; line_iter != dst.end(); ++line_iter) {
-            string line = lf.value()->read_line(line_iter);
-            separated_string ss(line.c_str(), line.length());
+            auto next_read_result = lf.read_line(line_iter);
+
+            if (next_read_result.isErr()) {
+                return SCAN_NO_MATCH;
+            }
+
+            line = next_read_result.unwrap();
+            separated_string ss(line.get_data(), line.length());
 
             ss.with_separator(this->blf_separator.get());
             auto iter = ss.begin();
@@ -593,7 +605,7 @@ public:
             !this->blf_field_defs.empty()) {
             this->blf_header_size = dst.size() - 1;
             dst.clear();
-            return this->scan_int(dst, offset, sbr);
+            return this->scan_int(dst, li, sbr);
         }
 
         this->blf_format_name.clear();
@@ -620,8 +632,6 @@ public:
             string_fragment sf = *iter;
             logline_value::kind_t kind = fd.fd_kind;
 
-            struct line_range lr(sf.sf_begin, sf.sf_end);
-
             if (sf == this->blf_empty_field) {
                 sf.clear();
             } else if (sf == this->blf_unset_field) {
@@ -629,18 +639,28 @@ public:
                 kind = logline_value::VALUE_NULL;
             }
 
+            auto lr = line_range(sf.sf_begin, sf.sf_end);
+
             if (fd.fd_name == TS) {
                 sa.emplace_back(lr, &logline::L_TIMESTAMP);
             } else if (fd.fd_name == UID) {
                 sa.emplace_back(lr, &logline::L_OPID);
             }
 
-            shared_buffer_ref value_ref;
-            value_ref.subset(sbr, sf.sf_begin, sf.length());
-            values.emplace_back(fd.fd_name, kind, value_ref,
-                                fd.fd_identifier, nullptr, iter.index(),
-                                lr.lr_start, lr.lr_end, false,
-                                this);
+            if (lr.is_valid()) {
+                values.emplace_back(fd.fd_name,
+                                    kind,
+                                    sbr,
+                                    fd.fd_identifier,
+                                    nullptr,
+                                    iter.index(),
+                                    lr.lr_start,
+                                    lr.lr_end,
+                                    false,
+                                    this);
+            } else {
+                values.emplace_back(fd.fd_name, this);
+            }
         }
     };
 
@@ -700,7 +720,7 @@ public:
         return retval;
     };
 
-    log_vtab_impl *get_vtab_impl(void) const {
+    log_vtab_impl *get_vtab_impl() const {
         if (this->blf_format_name.empty()) {
             return nullptr;
         }
