@@ -29,10 +29,10 @@
 
 #include "config.h"
 
-#include "lnav_log.hh"
+#include "base/lnav_log.hh"
 #include "sql_util.hh"
 #include "log_vtab_impl.hh"
-#include "yajlpp_def.hh"
+#include "yajlpp/yajlpp_def.hh"
 #include "vtab_module.hh"
 
 #include "logfile_sub_source.hh"
@@ -114,9 +114,8 @@ std::string log_vtab_impl::get_table_statement()
                                   colname.in(),
                                   type_to_string(iter->vc_type),
                                   iter->vc_hidden ? "hidden" : "",
-                                  (iter->vc_collator == NULL ||
-                                   iter->vc_collator[0] == '\0') ?
-                                  "BINARY" : iter->vc_collator,
+                                  iter->vc_collator.empty() ?
+                                  "BINARY" : iter->vc_collator.c_str(),
                                   comment.c_str());
         oss << coldecl;
     }
@@ -483,31 +482,35 @@ static int vt_column(sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int col)
         }
 
         case VT_COL_FILTERS: {
-            auto &filters = vt->lss->get_filters();
-            auto &filter_state = ld->ld_filter_state;
-            yajlpp_gen gen;
+            auto &filter_mask = ld->ld_filter_state.lfo_filter_state.tfs_mask;
 
-            yajl_gen_config(gen, yajl_gen_beautify, false);
+            if (!filter_mask[line_number]) {
+                sqlite3_result_null(ctx);
+            } else {
+                auto &filters = vt->lss->get_filters();
+                yajlpp_gen gen;
 
-            {
-                yajlpp_array arr(gen);
+                yajl_gen_config(gen, yajl_gen_beautify, false);
 
-                for (auto &filter : filters) {
-                    if (filter->lf_deleted) {
-                        continue;
-                    }
+                {
+                    yajlpp_array arr(gen);
 
-                    uint32_t mask = (1UL << filter->get_index());
+                    for (auto &filter : filters) {
+                        if (filter->lf_deleted) {
+                            continue;
+                        }
 
-                    if (filter_state.lfo_filter_state.tfs_mask[line_number] &
-                        mask) {
-                        arr.gen(filter->get_index());
+                        uint32_t mask = (1UL << filter->get_index());
+
+                        if (filter_mask[line_number] & mask) {
+                            arr.gen(filter->get_index());
+                        }
                     }
                 }
-            }
 
-            to_sqlite(ctx, gen.to_string_fragment());
-            sqlite3_result_subtype(ctx, 'J');
+                to_sqlite(ctx, gen.to_string_fragment());
+                sqlite3_result_subtype(ctx, 'J');
+            }
             break;
         }
 
@@ -805,14 +808,12 @@ static int vt_best_index(sqlite3_vtab *tab, sqlite3_index_info *p_info)
     return SQLITE_OK;
 }
 
-static struct json_path_handler tags_handler[] = {
+static struct json_path_container tags_handler = {
     json_path_handler("#")
         .with_synopsis("<tag>")
         .with_description("A tag for the log line")
         .with_pattern(R"(^#[^\s]+$)")
-        .FOR_FIELD(bookmark_metadata, bm_tags),
-
-    json_path_handler()
+        .FOR_FIELD(bookmark_metadata, bm_tags)
 };
 
 static int vt_update(sqlite3_vtab *tab,
@@ -837,7 +838,7 @@ static int vt_update(sqlite3_vtab *tab,
 
         if (log_tags) {
             vector<string> errors;
-            yajlpp_parse_context ypc(log_vtab_data.lvd_source, tags_handler);
+            yajlpp_parse_context ypc(log_vtab_data.lvd_source, &tags_handler);
             auto_mem<yajl_handle_t> handle(yajl_free);
 
             handle = yajl_alloc(&ypc.ypc_callbacks, nullptr, &ypc);
