@@ -45,6 +45,7 @@
 
 #include <time.h>
 
+#include "base/string_util.hh"
 #include "logfile.hh"
 #include "lnav_util.hh"
 
@@ -56,7 +57,7 @@ static const size_t INDEX_RESERVE_INCREMENT = 1024;
 logfile::logfile(const string &filename, logfile_open_options &loo)
     : lf_filename(filename)
 {
-    require(filename.size() > 0);
+    require(!filename.empty());
 
     memset(&this->lf_stat, 0, sizeof(this->lf_stat));
     if (loo.loo_fd == -1) {
@@ -107,7 +108,7 @@ logfile::~logfile()
 {
 }
 
-bool logfile::exists(void) const
+bool logfile::exists() const
 {
     struct stat st;
 
@@ -134,19 +135,19 @@ void logfile::set_format_base_time(log_format *lf)
     lf->lf_date_time.set_base_time(file_time);
 }
 
-bool logfile::process_prefix(off_t offset, shared_buffer_ref &sbr)
+bool logfile::process_prefix(shared_buffer_ref &sbr, const line_info &li)
 {
     log_format::scan_result_t found = log_format::SCAN_NO_MATCH;
     size_t prescan_size = this->lf_index.size();
     time_t prescan_time = 0;
     bool retval = false;
 
-    if (this->lf_format.get() != NULL) {
+    if (this->lf_format.get() != nullptr) {
         if (!this->lf_index.empty()) {
-            prescan_time = this->lf_index[0].get_time();
+            prescan_time = this->lf_index[prescan_size - 1].get_time();
         }
         /* We've locked onto a format, just use that scanner. */
-        found = this->lf_format->scan(this, this->lf_index, offset, sbr);
+        found = this->lf_format->scan(*this, this->lf_index, li, sbr);
     }
     else if (this->lf_options.loo_detect_format &&
              this->lf_index.size() < MAX_UNRECOGNIZED_LINES) {
@@ -167,7 +168,7 @@ bool logfile::process_prefix(off_t offset, shared_buffer_ref &sbr)
 
             (*iter)->clear();
             this->set_format_base_time(*iter);
-            found = (*iter)->scan(this, this->lf_index, offset, sbr);
+            found = (*iter)->scan(*this, this->lf_index, li, sbr);
             if (found == log_format::SCAN_MATCH) {
 #if 0
                 require(this->lf_index.size() == 1 ||
@@ -201,7 +202,12 @@ bool logfile::process_prefix(off_t offset, shared_buffer_ref &sbr)
 
     switch (found) {
         case log_format::SCAN_MATCH:
-            if (!this->lf_index.empty() && prescan_time != this->lf_index[0].get_time()) {
+            if (!this->lf_index.empty()) {
+                this->lf_index.back().set_valid_utf(li.li_valid_utf);
+            }
+            if (prescan_size > 0 &&
+                this->lf_index.size() >= prescan_size &&
+                prescan_time != this->lf_index[prescan_size - 1].get_time()) {
                 retval = true;
             }
             if (prescan_size > 0 && prescan_size < this->lf_index.size()) {
@@ -210,14 +216,7 @@ bool logfile::process_prefix(off_t offset, shared_buffer_ref &sbr)
 
                 if (latest < second_to_last) {
                     if (this->lf_format->lf_time_ordered) {
-                        log_debug(
-                            "%s:%d: out-of-time-order line detected %d.%03d < %d.%03d",
-                            this->lf_filename.c_str(),
-                            prescan_size,
-                            latest.get_time(),
-                            latest.get_millis(),
-                            second_to_last.get_time(),
-                            second_to_last.get_millis());
+                        this->lf_out_of_time_order_count += 1;
                         for (size_t lpc = prescan_size;
                              lpc < this->lf_index.size(); lpc++) {
                             logline &line_to_update = this->lf_index[lpc];
@@ -255,12 +254,13 @@ bool logfile::process_prefix(off_t offset, shared_buffer_ref &sbr)
                 last_mod = ll.get_module_id();
                 last_opid = ll.get_opid();
             }
-            this->lf_index.push_back(logline(offset,
-                                             last_time,
-                                             last_millis,
-                                             last_level,
-                                             last_mod,
-                                             last_opid));
+            this->lf_index.emplace_back(li.li_file_range.fr_offset,
+                                        last_time,
+                                        last_millis,
+                                        last_level,
+                                        last_mod,
+                                        last_opid);
+            this->lf_index.back().set_valid_utf(li.li_valid_utf);
             break;
         }
         case log_format::SCAN_INCOMPLETE:
@@ -282,8 +282,10 @@ logfile::rebuild_result_t logfile::rebuild_index()
     }
 
     // Check the previous stat against the last to see if things are wonky.
-    if (this->lf_stat.st_size > st.st_size) {
-        log_info("truncated file detected, closing -- %s",
+    if (st.st_size < this->lf_stat.st_size ||
+        (this->lf_stat.st_size == st.st_size &&
+         this->lf_stat.st_mtime != st.st_mtime)) {
+        log_info("overwritten file detected, closing -- %s",
                  this->lf_filename.c_str());
         this->close();
         return RR_NO_NEW_LINES;
@@ -293,13 +295,13 @@ logfile::rebuild_result_t logfile::rebuild_index()
 
         // We haven't reached the end of the file.  Note that we use the
         // line buffer's notion of the file size since it may be compressed.
-        bool has_format = this->lf_format.get() != NULL;
+        bool has_format = this->lf_format.get() != nullptr;
         struct rusage begin_rusage;
-        shared_buffer_ref sbr;
-        off_t  last_off, off;
-        line_value lv;
+        off_t off;
         size_t begin_size = this->lf_index.size();
         bool record_rusage = this->lf_index.size() == 1;
+        off_t begin_index_size = this->lf_index_size;
+        size_t rollback_size = 0;
 
         if (record_rusage) {
             getrusage(RUSAGE_SELF, &begin_rusage);
@@ -314,77 +316,107 @@ logfile::rebuild_result_t logfile::rebuild_index()
              */
             while (this->lf_index.back().get_sub_offset() != 0) {
                 this->lf_index.pop_back();
+                rollback_size += 1;
             }
             this->lf_index.pop_back();
+            rollback_size += 1;
 
             this->lf_line_buffer.clear();
             if (!this->lf_index.empty()) {
                 off_t check_line_off = this->lf_index.back().get_offset();
 
-                if (!this->lf_line_buffer.read_line(check_line_off, sbr, &lv) ||
-                        off != check_line_off) {
-                    log_info("overwritten file detected, closing -- %s",
-                             this->lf_filename.c_str());
+                auto read_result = this->lf_line_buffer.read_range({
+                    check_line_off, this->lf_index_size - check_line_off
+                });
+
+                if (read_result.isErr()) {
+                    log_info("overwritten file detected, closing -- %s (%s)",
+                             this->lf_filename.c_str(),
+                             read_result.unwrapErr().c_str());
                     this->close();
-                    return RR_NO_NEW_LINES;
+                    return RR_INVALID;
                 }
             }
         }
         else {
             off = 0;
         }
-        last_off = off;
         if (this->lf_logline_observer != NULL) {
-            this->lf_logline_observer->logline_restart(*this);
+            this->lf_logline_observer->logline_restart(*this, rollback_size);
         }
 
         bool sort_needed = this->lf_sort_needed;
         this->lf_sort_needed = false;
 
-        while (this->lf_line_buffer.read_line(off, sbr, &lv)) {
-            size_t old_size = this->lf_index.size();
+        auto prev_range = file_range{off};
+        while (true) {
+            auto load_result = this->lf_line_buffer.load_next_line(prev_range);
 
-            if (old_size == 0) {
-                shared_buffer_ref avail_sbr;
-
-                this->lf_line_buffer.read_available(avail_sbr);
-                this->lf_text_format = detect_text_format(
-                    avail_sbr.get_data(), avail_sbr.length());
+            if (load_result.isErr()) {
+                this->close();
+                return RR_INVALID;
             }
 
+            auto li = load_result.unwrap();
+
+            if (li.li_file_range.empty()) {
+                break;
+            }
+            prev_range = li.li_file_range;
+
+            size_t old_size = this->lf_index.size();
+
+            // Update this early so that line_length() works
+            this->lf_index_size = li.li_file_range.next_offset();
+            if (old_size == 0) {
+                file_range fr = this->lf_line_buffer.get_available();
+                auto avail_data = this->lf_line_buffer.read_range(fr);
+
+                this->lf_text_format = avail_data.map(
+                    [](const shared_buffer_ref &avail_sbr) -> text_format_t {
+                        return detect_text_format(
+                            avail_sbr.get_data(), avail_sbr.length());
+                    })
+                    .unwrapOr(text_format_t::TF_UNKNOWN);
+            }
+
+            auto read_result = this->lf_line_buffer.read_range(li.li_file_range);
+            if (read_result.isErr()) {
+                this->close();
+                return RR_INVALID;
+            }
+
+            auto sbr = read_result.unwrap().rtrim(is_line_ending);
             this->lf_longest_line = std::max(this->lf_longest_line, sbr.length());
-            this->lf_partial_line = lv.lv_partial;
-            sort_needed = this->process_prefix(last_off, sbr) || sort_needed;
-            last_off = off;
+            this->lf_partial_line = li.li_partial;
+            sort_needed = this->process_prefix(sbr, li) || sort_needed;
 
             if (old_size > this->lf_index.size()) {
                 old_size = 0;
             }
 
-            for (auto iter = this->begin() + old_size;
-                    iter != this->end(); ++iter) {
-                if (this->lf_logline_observer != NULL) {
-                    this->lf_logline_observer->logline_new_line(*this, iter, sbr);
-                }
+            if (this->lf_logline_observer != nullptr) {
+                this->lf_logline_observer->logline_new_lines(
+                    *this, this->begin() + old_size, this->end(), sbr);
             }
 
-            if (this->lf_logfile_observer != NULL) {
+            if (this->lf_logfile_observer != nullptr) {
                 this->lf_logfile_observer->logfile_indexing(
                     *this,
-                    this->lf_line_buffer.get_read_offset(off),
+                    this->lf_line_buffer.get_read_offset(li.li_file_range.next_offset()),
                     st.st_size);
             }
 
-            if (!has_format && this->lf_format != NULL) {
+            if (!has_format && this->lf_format != nullptr) {
                 break;
             }
         }
 
-        if (this->lf_logline_observer != NULL) {
+        if (this->lf_logline_observer != nullptr) {
             this->lf_logline_observer->logline_eof(*this);
         }
 
-        if (record_rusage && (off - this->lf_index_size) > (500 * 1024)) {
+        if (record_rusage && (prev_range.fr_offset - begin_index_size) > (500 * 1024)) {
             struct rusage end_rusage;
 
             getrusage(RUSAGE_SELF, &end_rusage);
@@ -393,7 +425,7 @@ logfile::rebuild_result_t logfile::rebuild_index()
                      this->lf_filename.c_str(),
                      begin_size,
                      this->lf_index.size());
-            log_rusage(LOG_LEVEL_INFO, this->lf_activity.la_initial_index_rusage);
+            log_rusage(lnav_log_level_t::INFO, this->lf_activity.la_initial_index_rusage);
         }
 
         /*
@@ -401,7 +433,7 @@ logfile::rebuild_result_t logfile::rebuild_index()
          * doing the scanning, so use the line buffer's notion of the file
          * size.
          */
-        this->lf_index_size = off;
+        this->lf_index_size = prev_range.next_offset();
         this->lf_stat = st;
 
         if (sort_needed) {
@@ -416,83 +448,38 @@ logfile::rebuild_result_t logfile::rebuild_index()
         this->lf_index_time = st.st_mtime;
     }
 
+    if (this->lf_out_of_time_order_count) {
+        log_info("Detected %d out-of-time-order lines in file: %s",
+                 this->lf_out_of_time_order_count,
+                 this->lf_filename.c_str());
+        this->lf_out_of_time_order_count = 0;
+    }
+
     return retval;
 }
 
-void logfile::read_line(logfile::iterator ll, string &line_out)
+Result<shared_buffer_ref, std::string> logfile::read_line(logfile::iterator ll)
 {
     try {
-        off_t       off = ll->get_offset();
-        shared_buffer_ref sbr;
+        return this->lf_line_buffer.read_range(this->get_file_range(ll, false))
+            .map([&ll, this](auto sbr) {
+                sbr.rtrim(is_line_ending);
+                if (!ll->is_valid_utf()) {
+                    scrub_to_utf8(sbr.get_writable_data(), sbr.length());
+                }
 
-        line_out.clear();
-        if (this->lf_line_buffer.read_line(off, sbr)) {
-            if (this->lf_format != NULL) {
-                this->lf_format->get_subline(*ll, sbr);
-            }
-            line_out.append(sbr.get_data(), sbr.length());
-        }
-        else {
-            /* XXX */
-        }
+                if (this->lf_format != nullptr) {
+                    this->lf_format->get_subline(*ll, sbr);
+                }
+
+                return sbr;
+            });
     }
     catch (line_buffer::error & e) {
-        /* ... */
+        return Err(string(strerror(e.e_err)));
     }
 }
 
-bool logfile::read_line(logfile::iterator ll, shared_buffer_ref &sbr)
-{
-    try {
-        off_t       off = ll->get_offset();
-
-        if (this->lf_line_buffer.read_line(off, sbr)) {
-            if (this->lf_format != NULL) {
-                this->lf_format->get_subline(*ll, sbr);
-            }
-            return true;
-        }
-    }
-    catch (line_buffer::error & e) {
-    }
-    return false;
-}
-
-void logfile::read_full_message(logfile::iterator ll,
-                                string &msg_out,
-                                int max_lines)
-{
-    ostringstream stream;
-
-    do {
-        try {
-            off_t       off = ll->get_offset();
-            shared_buffer_ref sbr;
-
-            if (stream.tellp() > 0) {
-                stream.write("\n", 1);
-            }
-            if (this->lf_line_buffer.read_line(off, sbr)) {
-                this->lf_format->get_subline(*ll, sbr);
-                stream.write(sbr.get_data(), sbr.length());
-            }
-            else {
-                /* XXX */
-            }
-        }
-        catch (line_buffer::error & e) {
-            /* ... */
-        }
-        ++ll;
-        if (max_lines != -1) {
-            max_lines -= 1;
-        }
-    } while (ll != this->end() && ll->is_continued() &&
-             (max_lines == -1 || max_lines > 0));
-
-    msg_out = stream.str();
-}
-
 void logfile::read_full_message(logfile::iterator ll,
                                 shared_buffer_ref &msg_out,
                                 int max_lines)
@@ -502,15 +489,17 @@ void logfile::read_full_message(logfile::iterator ll,
     size_t line_len = this->line_length(ll);
 
     try {
-        off_t       off = ll->get_offset();
+        off_t off = ll->get_offset();
 
-        if (this->lf_line_buffer.read_range(off, line_len, msg_out)) {
-            if (this->lf_format.get() != NULL) {
-                this->lf_format->get_subline(*ll, msg_out, true);
-            }
+        auto read_result = this->lf_line_buffer.read_range({
+            off, static_cast<ssize_t>(line_len)});
+
+        if (read_result.isErr()) {
+            return;
         }
-        else {
-                /* XXX */
+        msg_out = read_result.unwrap();
+        if (this->lf_format.get() != nullptr) {
+            this->lf_format->get_subline(*ll, msg_out, true);
         }
     }
     catch (line_buffer::error & e) {
@@ -521,31 +510,38 @@ void logfile::read_full_message(logfile::iterator ll,
 void logfile::set_logline_observer(logline_observer *llo)
 {
     this->lf_logline_observer = llo;
-    if (llo != NULL) {
+    if (llo != nullptr) {
         this->reobserve_from(this->begin());
     }
 }
 
 void logfile::reobserve_from(iterator iter)
 {
-    if (this->lf_logline_observer != NULL) {
-        for (; iter != this->end(); ++iter) {
-            off_t offset = std::distance(this->begin(), iter);
-            shared_buffer_ref sbr;
+    for (; iter != this->end(); ++iter) {
+        off_t offset = std::distance(this->begin(), iter);
 
-            if (this->lf_logfile_observer != NULL) {
-                this->lf_logfile_observer->logfile_indexing(
-                        *this, offset, this->size());
-            }
-
-            this->read_line(iter, sbr);
-            this->lf_logline_observer->logline_new_line(*this, iter, sbr);
+        if (iter->get_sub_offset() > 0) {
+            continue;
         }
-        if (this->lf_logfile_observer != NULL) {
+
+        if (this->lf_logfile_observer != nullptr) {
             this->lf_logfile_observer->logfile_indexing(
-                    *this, this->size(), this->size());
+                *this, offset, this->size());
         }
 
+        this->read_line(iter).then([this, iter](auto sbr) {
+            auto iter_end = iter + 1;
+
+            while (iter_end != this->end() && iter_end->get_sub_offset() != 0) {
+                ++iter_end;
+            }
+            this->lf_logline_observer->logline_new_lines(
+                *this, iter, iter_end, sbr);
+        });
+    }
+    if (this->lf_logfile_observer != nullptr) {
+        this->lf_logfile_observer->logfile_indexing(
+            *this, this->size(), this->size());
         this->lf_logline_observer->logline_eof(*this);
     }
 }
@@ -554,3 +550,37 @@ filesystem::path logfile::get_path() const
 {
     return this->lf_filename;
 }
+
+size_t logfile::line_length(logfile::iterator ll, bool include_continues)
+{
+    iterator next_line = ll;
+    size_t retval;
+
+    if (!include_continues && this->lf_next_line_cache) {
+        if (ll->get_offset() == (*this->lf_next_line_cache).first) {
+            return this->lf_next_line_cache->second;
+        }
+    }
+
+    do {
+        ++next_line;
+    } while ((next_line != this->end()) &&
+             ((ll->get_offset() == next_line->get_offset()) ||
+              (include_continues && next_line->is_continued())));
+
+    if (next_line == this->end()) {
+        retval = this->lf_index_size - ll->get_offset();
+        if (retval > 0 && !this->lf_partial_line) {
+            retval -= 1;
+        }
+    }
+    else {
+        retval = next_line->get_offset() - ll->get_offset() - 1;
+        if (!include_continues) {
+            this->lf_next_line_cache = nonstd::make_optional(
+                std::make_pair(ll->get_offset(), retval));
+        }
+    }
+
+    return retval;
+}
