@@ -40,6 +40,7 @@
 #include "logfile_sub_source.hh"
 #include "command_executor.hh"
 #include "ansi_scrubber.hh"
+#include "lnav_config.hh"
 
 using namespace std;
 
@@ -104,8 +105,10 @@ logfile_sub_source::logfile_sub_source()
       lss_marked_only(false),
       lss_index_delegate(NULL),
       lss_longest_line(0),
-      lss_meta_grepper(*this)
+      lss_meta_grepper(*this),
+      lss_location_history(*this)
 {
+    this->tss_supports_filtering = true;
     this->clear_line_size_cache();
     this->clear_min_max_log_times();
 }
@@ -168,7 +171,9 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
 
     if (flags & RF_RAW) {
         shared_ptr<logfile> lf = this->find(line);
-        lf->read_line(lf->begin() + line, value_out);
+        value_out = lf->read_line(lf->begin() + line)
+            .map([](auto sbr) { return to_string(sbr); })
+            .unwrapOr({});
         return;
     }
 
@@ -180,11 +185,15 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
     this->lss_token_values.clear();
     this->lss_share_manager.invalidate_refs();
     if (flags & text_sub_source::RF_FULL) {
-        this->lss_token_file->read_full_message(this->lss_token_line,
-                                                this->lss_token_value);
+        shared_buffer_ref sbr;
+
+        this->lss_token_file->read_full_message(this->lss_token_line, sbr);
+        this->lss_token_value = to_string(sbr);
     } else {
         this->lss_token_value =
-            this->lss_token_file->read_line(this->lss_token_line);
+            this->lss_token_file->read_line(this->lss_token_line).map([](auto sbr) {
+                return to_string(sbr);
+            }).unwrapOr({});
     }
     this->lss_token_shift_start = 0;
     this->lss_token_shift_size = 0;
@@ -196,73 +205,78 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
         format->scrub(value_out);
     }
 
-    if (!this->lss_token_line->is_continued() ||
-        this->lss_token_line->get_sub_offset() != 0) {
-        shared_buffer_ref sbr;
+    shared_buffer_ref sbr;
 
-        sbr.share(this->lss_share_manager,
-                  (char *)this->lss_token_value.c_str(), this->lss_token_value.size());
-        format->annotate(sbr, this->lss_token_attrs, this->lss_token_values);
-        if (this->lss_token_line->get_sub_offset() != 0) {
-            this->lss_token_attrs.clear();
-        }
-        if (flags & RF_REWRITE) {
-            exec_context ec(&this->lss_token_values, pretty_sql_callback, pretty_pipe_callback);
-            string rewritten_line;
+    sbr.share(this->lss_share_manager,
+              (char *)this->lss_token_value.c_str(), this->lss_token_value.size());
+    if (this->lss_token_line->is_continued()) {
+        this->lss_token_attrs.emplace_back(
+            line_range{0, (int) this->lss_token_value.length()},
+            &textview_curses::SA_BODY);
+    } else {
+        format->annotate(line, sbr, this->lss_token_attrs,
+                         this->lss_token_values,
+                         false);
+    }
+    if (this->lss_token_line->get_sub_offset() != 0) {
+        this->lss_token_attrs.clear();
+    }
+    if (flags & RF_REWRITE) {
+        exec_context ec(&this->lss_token_values, pretty_sql_callback, pretty_pipe_callback);
+        string rewritten_line;
 
-            ec.ec_top_line = vis_line_t(row);
-            add_ansi_vars(ec.ec_global_vars);
-            add_global_vars(ec);
-            format->rewrite(ec, sbr, this->lss_token_attrs, rewritten_line);
-            this->lss_token_value.assign(rewritten_line);
-            value_out = this->lss_token_value;
-        }
+        ec.ec_top_line = vis_line_t(row);
+        add_ansi_vars(ec.ec_global_vars);
+        add_global_vars(ec);
+        format->rewrite(ec, sbr, this->lss_token_attrs, rewritten_line);
+        this->lss_token_value.assign(rewritten_line);
+        value_out = this->lss_token_value;
+    }
 
-        if ((this->lss_token_file->is_time_adjusted() ||
-             format->lf_timestamp_flags & ETF_MACHINE_ORIENTED) &&
-            format->lf_date_time.dts_fmt_lock != -1) {
-            auto time_attr = find_string_attr(
-                this->lss_token_attrs, &logline::L_TIMESTAMP);
-            if (time_attr != this->lss_token_attrs.end()) {
-                const struct line_range time_range = time_attr->sa_range;
-                struct timeval adjusted_time;
-                struct exttm adjusted_tm;
-                char buffer[128];
-                const char *fmt;
-                ssize_t len;
+    if ((this->lss_token_file->is_time_adjusted() ||
+         format->lf_timestamp_flags & ETF_MACHINE_ORIENTED) &&
+        format->lf_date_time.dts_fmt_lock != -1) {
+        auto time_attr = find_string_attr(
+            this->lss_token_attrs, &logline::L_TIMESTAMP);
+        if (time_attr != this->lss_token_attrs.end()) {
+            const struct line_range time_range = time_attr->sa_range;
+            struct timeval adjusted_time;
+            struct exttm adjusted_tm;
+            char buffer[128];
+            const char *fmt;
+            ssize_t len;
 
-                if (format->lf_timestamp_flags & ETF_MACHINE_ORIENTED) {
-                    format->lf_date_time.convert_to_timeval(
-                        &this->lss_token_value.c_str()[time_range.lr_start],
-                        time_range.length(),
-                        format->get_timestamp_formats(),
-                        adjusted_time);
-                    fmt = "%Y-%m-%d %H:%M:%S.%f";
-                    gmtime_r(&adjusted_time.tv_sec, &adjusted_tm.et_tm);
-                    adjusted_tm.et_nsec = adjusted_time.tv_usec * 1000;
-                    len = ftime_fmt(buffer, sizeof(buffer), fmt, adjusted_tm);
-                }
-                else {
-                    adjusted_time = this->lss_token_line->get_timeval();
-                    gmtime_r(&adjusted_time.tv_sec, &adjusted_tm.et_tm);
-                    adjusted_tm.et_nsec = adjusted_time.tv_usec * 1000;
-                    len = format->lf_date_time.ftime(buffer, sizeof(buffer), adjusted_tm);
-                }
-
-                if (len > time_range.length()) {
-                    ssize_t padding = len - time_range.length();
-
-                    value_out.insert(time_range.lr_start,
-                                     padding,
-                                     ' ');
-                }
-                value_out.replace(time_range.lr_start,
-                                  len,
-                                  buffer,
-                                  len);
-                this->lss_token_shift_start = time_range.lr_start;
-                this->lss_token_shift_size = len - time_range.length();
+            if (format->lf_timestamp_flags & ETF_MACHINE_ORIENTED) {
+                format->lf_date_time.convert_to_timeval(
+                    &this->lss_token_value.c_str()[time_range.lr_start],
+                    time_range.length(),
+                    format->get_timestamp_formats(),
+                    adjusted_time);
+                fmt = "%Y-%m-%d %H:%M:%S.%f";
+                gmtime_r(&adjusted_time.tv_sec, &adjusted_tm.et_tm);
+                adjusted_tm.et_nsec = adjusted_time.tv_usec * 1000;
+                len = ftime_fmt(buffer, sizeof(buffer), fmt, adjusted_tm);
+            } else {
+                adjusted_time = this->lss_token_line->get_timeval();
+                gmtime_r(&adjusted_time.tv_sec, &adjusted_tm.et_tm);
+                adjusted_tm.et_nsec = adjusted_time.tv_usec * 1000;
+                len = format->lf_date_time.ftime(buffer, sizeof(buffer),
+                                                 adjusted_tm);
             }
+
+            if (len > time_range.length()) {
+                ssize_t padding = len - time_range.length();
+
+                value_out.insert(time_range.lr_start,
+                                 padding,
+                                 ' ');
+            }
+            value_out.replace(time_range.lr_start,
+                              len,
+                              buffer,
+                              len);
+            this->lss_token_shift_start = time_range.lr_start;
+            this->lss_token_shift_size = len - time_range.length();
         }
     }
 
@@ -292,22 +306,31 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
     }
 
     if (this->lss_flags & F_TIME_OFFSET) {
-        int64_t start_millis, curr_millis;
+        int64_t curr_millis, diff;
+
+        curr_millis = this->lss_token_line->get_time_in_millis();
 
         vis_line_t prev_mark =
             tc.get_bookmarks()[&textview_curses::BM_USER].prev(vis_line_t(row));
-        if (prev_mark == -1) {
-            prev_mark = vis_line_t(0);
-        }
+        vis_line_t next_mark =
+            tc.get_bookmarks()[&textview_curses::BM_USER].next(vis_line_t(row));
+        if (prev_mark == -1 && next_mark != -1) {
+            auto next_line = this->find_line(this->at(next_mark));
 
-        logline *first_line = this->find_line(this->at(prev_mark));
-        start_millis = first_line->get_time_in_millis();
-        curr_millis = this->lss_token_line->get_time_in_millis();
-        int64_t diff = curr_millis - start_millis;
+            diff = curr_millis - next_line->get_time_in_millis();
+        } else {
+            if (prev_mark == -1) {
+                prev_mark = 0_vl;
+            }
+
+            auto first_line = this->find_line(this->at(prev_mark));
+            auto start_millis = first_line->get_time_in_millis();
+            diff = curr_millis - start_millis;
+        }
 
         value_out = "|" + value_out;
         string relstr;
-        size_t rel_length = str2reltime(diff, relstr);
+        size_t rel_length = duration2str(diff, relstr);
         value_out.insert(0, relstr);
         if (rel_length < 12) {
             value_out.insert(0, 12 - rel_length, ' ');
@@ -326,21 +349,8 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
     int attrs           = 0;
 
     value_out = this->lss_token_attrs;
-    switch (this->lss_token_line->get_msg_level()) {
-    case LEVEL_FATAL:
-    case LEVEL_CRITICAL:
-    case LEVEL_ERROR:
-        attrs = vc.attrs_for_role(view_colors::VCR_ERROR);
-        break;
 
-    case LEVEL_WARNING:
-        attrs = vc.attrs_for_role(view_colors::VCR_WARNING);
-        break;
-
-    default:
-        attrs = vc.attrs_for_role(view_colors::VCR_TEXT);
-        break;
-    }
+    attrs = vc.vc_level_attrs[this->lss_token_line->get_msg_level()].first;
 
     if ((row + 1) < (int)this->lss_filtered_index.size()) {
         next_line = this->find_line(this->at(vis_line_t(row + 1)));
@@ -363,30 +373,29 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
 
     value_out.emplace_back(lr, &view_curses::VC_STYLE, attrs);
 
-    for (auto lv_iter = line_values.cbegin();
-         lv_iter != line_values.cend();
-         ++lv_iter) {
+    for (const auto &line_value : line_values) {
         if ((!(this->lss_token_flags & RF_FULL) &&
-             lv_iter->lv_sub_offset != this->lss_token_line->get_sub_offset()) ||
-            !lv_iter->lv_origin.is_valid()) {
+            line_value.lv_sub_offset != this->lss_token_line->get_sub_offset()) ||
+            !line_value.lv_origin.is_valid()) {
             continue;
         }
 
-        if (lv_iter->lv_hidden) {
+        if (line_value.lv_hidden) {
             value_out.emplace_back(
-                lv_iter->lv_origin, &textview_curses::SA_HIDDEN);
+                line_value.lv_origin, &textview_curses::SA_HIDDEN);
         }
 
-        if (!lv_iter->lv_identifier || !lv_iter->lv_origin.is_valid()) {
+        if (!line_value.lv_identifier || !line_value.lv_origin.is_valid()) {
             continue;
         }
 
-        int id_attrs = vc.attrs_for_ident(lv_iter->text_value(),
-                                          lv_iter->text_length());
+        auto start = this->lss_token_value.c_str();
+        int id_attrs = vc.attrs_for_ident(&start[line_value.lv_origin.lr_start],
+                                          line_value.lv_origin.sublen(this->lss_token_value));
 
-        line_range ident_range = lv_iter->lv_origin;
+        line_range ident_range = line_value.lv_origin;
         if (this->lss_token_flags & RF_FULL) {
-            ident_range = lv_iter->origin_in_full_msg(
+            ident_range = line_value.origin_in_full_msg(
                 this->lss_token_value.c_str(), this->lss_token_value.length());
         }
 
@@ -504,6 +513,14 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
                 value_out.emplace_back(lr, &logline::L_PARTITION, &bm_iter->second);
             }
         }
+
+        auto bm_iter = this->lss_user_mark_metadata.find(this->at(vis_line_t(row)));
+
+        if (bm_iter != this->lss_user_mark_metadata.end()) {
+            lr.lr_start = 0;
+            lr.lr_end = -1;
+            value_out.emplace_back(lr, &logline::L_META, &bm_iter->second);
+        }
     }
 
     if (this->lss_token_file->is_time_adjusted()) {
@@ -537,16 +554,19 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
     }
 }
 
-bool logfile_sub_source::rebuild_index(bool force)
+logfile_sub_source::rebuild_result logfile_sub_source::rebuild_index()
 {
     iterator iter;
     size_t total_lines = 0;
-    bool retval, full_sort = false;
+    bool full_sort = false;
     int file_count = 0;
+    bool force = this->lss_force_rebuild;
+    rebuild_result retval = rebuild_result::rr_no_change;
 
-    force = force || this->lss_force_rebuild;
     this->lss_force_rebuild = false;
-    retval = force;
+    if (force) {
+        retval = rebuild_result::rr_full_rebuild;
+    }
 
     for (iter = this->lss_files.begin();
          iter != this->lss_files.end();
@@ -556,35 +576,42 @@ bool logfile_sub_source::rebuild_index(bool force)
         if (ld.get_file() == NULL) {
             if (ld.ld_lines_indexed > 0) {
                 force  = true;
-                retval = true;
+                retval = rebuild_result::rr_full_rebuild;
             }
         }
         else {
             logfile &lf = *ld.get_file();
 
-            switch (lf.rebuild_index()) {
-                case logfile::RR_NO_NEW_LINES:
-                    // No changes
-                    break;
-                case logfile::RR_NEW_LINES:
-                    retval = true;
-                    if (!this->lss_index.empty()) {
-                        logline &new_file_line = lf[ld.ld_lines_indexed];
-                        content_line_t cl = this->lss_index.back();
-                        logline *last_indexed_line = this->find_line(cl);
-
-                        // If there are new lines that are older than what we
-                        // have in the index, we need to resort.
-                        if (last_indexed_line == nullptr ||
-                            new_file_line < last_indexed_line->get_timeval()) {
-                            force = true;
+            if (!this->tss_view->is_paused()) {
+                switch (lf.rebuild_index()) {
+                    case logfile::RR_NO_NEW_LINES:
+                        // No changes
+                        break;
+                    case logfile::RR_NEW_LINES:
+                        if (retval == rebuild_result::rr_no_change) {
+                            retval = rebuild_result::rr_appended_lines;
                         }
-                    }
-                    break;
-                case logfile::RR_NEW_ORDER:
-                    retval = true;
-                    force = true;
-                    break;
+                        if (!this->lss_index.empty()) {
+                            logline &new_file_line = lf[ld.ld_lines_indexed - 1];
+                            content_line_t cl = this->lss_index.back();
+                            logline *last_indexed_line = this->find_line(cl);
+
+                            // If there are new lines that are older than what we
+                            // have in the index, we need to resort.
+                            if (last_indexed_line == nullptr ||
+                                new_file_line <
+                                last_indexed_line->get_timeval()) {
+                                force = true;
+                                retval = rebuild_result::rr_full_rebuild;
+                            }
+                        }
+                        break;
+                    case logfile::RR_INVALID:
+                    case logfile::RR_NEW_ORDER:
+                        retval = rebuild_result::rr_full_rebuild;
+                        force = true;
+                        break;
+                }
             }
             file_count += 1;
             total_lines += (*iter)->get_file()->size();
@@ -610,7 +637,7 @@ bool logfile_sub_source::rebuild_index(bool force)
         this->lss_filename_width = 0;
     }
 
-    if (retval || force) {
+    if (retval != rebuild_result::rr_no_change || force) {
         size_t index_size = 0, start_size = this->lss_index.size();
         logline_cmp line_cmper(*this);
 
@@ -644,6 +671,8 @@ bool logfile_sub_source::rebuild_index(bool force)
                 }
             }
 
+            // XXX get rid of this full sort on the initial run, it's not
+            // needed unless the file is not in time-order
             sort(this->lss_index.begin(), this->lss_index.end(), line_cmper);
         } else {
             kmerge_tree_c<logline, logfile_data, logfile::iterator> merge(
@@ -727,6 +756,17 @@ bool logfile_sub_source::rebuild_index(bool force)
         }
     }
 
+    switch (retval) {
+        case rebuild_result::rr_no_change:
+            break;
+        case rebuild_result::rr_full_rebuild:
+            this->tss_view->redo_search();
+            break;
+        case rebuild_result::rr_appended_lines:
+            this->tss_view->search_new_data();
+            break;
+    }
+
     return retval;
 }
 
@@ -757,7 +797,7 @@ void logfile_sub_source::text_update_marks(vis_bookmarks &bm)
                 bm[lss_user_mark.first].insert_once(vl);
 
                 if (lss_user_mark.first == &textview_curses::BM_USER) {
-                    logfile::iterator ll = lf->begin() + cl;
+                    auto ll = lf->begin() + cl;
 
                     ll->set_mark(true);
                 }
@@ -815,11 +855,10 @@ log_accel::direction_t logfile_sub_source::get_line_accel_direction(
 
 void logfile_sub_source::text_filters_changed()
 {
-    for (iterator iter = this->begin(); iter != this->end(); ++iter) {
-        logfile_data *ld = *iter;
+    for (auto ld : *this) {
         shared_ptr<logfile> lf = ld->get_file();
 
-        if (lf != NULL) {
+        if (lf != nullptr) {
             ld->ld_filter_state.clear_deleted_filter_state();
             lf->reobserve_from(lf->begin() + ld->ld_filter_state.get_min_count(lf->size()));
         }
@@ -829,7 +868,7 @@ void logfile_sub_source::text_filters_changed()
 
     this->get_filters().get_enabled_mask(filtered_in_mask, filtered_out_mask);
 
-    if (this->lss_index_delegate != NULL) {
+    if (this->lss_index_delegate != nullptr) {
         this->lss_index_delegate->index_start(*this);
     }
 
@@ -838,12 +877,12 @@ void logfile_sub_source::text_filters_changed()
         content_line_t cl = (content_line_t) this->lss_index[index_index];
         uint64_t line_number;
         logfile_data *ld = this->find_data(cl, line_number);
-        logfile::iterator line_iter = ld->get_file()->begin() + line_number;
+        auto line_iter = ld->get_file()->begin() + line_number;
 
         if (!ld->ld_filter_state.excluded(filtered_in_mask, filtered_out_mask,
                 line_number) && this->check_extra_filters(*line_iter)) {
             this->lss_filtered_index.push_back(index_index);
-            if (this->lss_index_delegate != NULL) {
+            if (this->lss_index_delegate != nullptr) {
                 shared_ptr<logfile> lf = ld->get_file();
                 this->lss_index_delegate->index_line(
                         *this, lf.get(), lf->begin() + line_number);
@@ -851,9 +890,14 @@ void logfile_sub_source::text_filters_changed()
         }
     }
 
-    if (this->lss_index_delegate != NULL) {
+    if (this->lss_index_delegate != nullptr) {
         this->lss_index_delegate->index_complete(*this);
     }
+
+    if (this->tss_view != nullptr) {
+        this->tss_view->reload_data();
+        this->tss_view->redo_search();
+    }
 }
 
 bool logfile_sub_source::list_input_handle_key(listview_curses &lv, int ch)
@@ -881,3 +925,76 @@ bool logfile_sub_source::list_input_handle_key(listview_curses &lv, int ch)
     }
     return false;
 }
+
+nonstd::optional<pair<grep_proc_source<vis_line_t> *, grep_proc_sink<vis_line_t> *>>
+logfile_sub_source::get_grepper()
+{
+    return make_pair(
+        (grep_proc_source<vis_line_t> *) &this->lss_meta_grepper,
+        (grep_proc_sink<vis_line_t> *) &this->lss_meta_grepper
+    );
+}
+
+void log_location_history::loc_history_append(vis_line_t top)
+{
+    if (top >= vis_line_t(this->llh_log_source.text_line_count())) {
+        return;
+    }
+
+    content_line_t cl = this->llh_log_source.at(top);
+
+    auto iter = this->llh_history.begin();
+    iter += this->llh_history.size() - this->lh_history_position;
+    this->llh_history.erase_from(iter);
+    this->lh_history_position = 0;
+    this->llh_history.push_back(cl);
+}
+
+nonstd::optional<vis_line_t> log_location_history::loc_history_back(vis_line_t current_top)
+{
+    while (this->lh_history_position < this->llh_history.size()) {
+        auto iter = this->llh_history.rbegin();
+
+        auto vis_for_pos = this->llh_log_source.find_from_content(*iter);
+
+        if (this->lh_history_position == 0 && vis_for_pos != current_top) {
+            return vis_for_pos;
+        }
+
+        if ((this->lh_history_position + 1) >= this->llh_history.size()) {
+            break;
+        }
+
+        this->lh_history_position += 1;
+
+        iter += this->lh_history_position;
+
+        vis_for_pos = this->llh_log_source.find_from_content(*iter);
+
+        if (vis_for_pos) {
+            return vis_for_pos;
+        }
+    }
+
+    return nonstd::nullopt;
+}
+
+nonstd::optional<vis_line_t>
+log_location_history::loc_history_forward(vis_line_t current_top)
+{
+    while (this->lh_history_position > 0) {
+        this->lh_history_position -= 1;
+
+        auto iter = this->llh_history.rbegin();
+
+        iter += this->lh_history_position;
+
+        auto vis_for_pos = this->llh_log_source.find_from_content(*iter);
+
+        if (vis_for_pos) {
+            return vis_for_pos;
+        }
+    }
+
+    return nonstd::nullopt;
+}
