@@ -144,8 +144,8 @@ void rl_change(void *dummy, readline_curses *rc)
 {
     textview_curses *tc = *lnav_data.ld_view_stack.top();
 
-    tc->get_highlights().erase("$preview");
-    tc->get_highlights().erase("$bodypreview");
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "preview"});
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "bodypreview"});
     lnav_data.ld_preview_source.clear();
     lnav_data.ld_preview_status_source.get_description().clear();
 
@@ -181,19 +181,19 @@ void rl_change(void *dummy, readline_curses *rc)
                 lnav_data.ld_bottom_source.grep_error("");
             }
             else if (args[0] == "config" && args.size() > 1) {
-                yajlpp_parse_context ypc("input", lnav_config_handlers);
+                yajlpp_parse_context ypc("input", &lnav_config_handlers);
 
                 ypc.set_path(args[1])
                     .with_obj(lnav_config);
                 ypc.update_callbacks();
 
-                if (ypc.ypc_current_handler != NULL) {
+                if (ypc.ypc_current_handler != nullptr) {
                     const json_path_handler_base *jph = ypc.ypc_current_handler;
                     char help_text[1024];
 
                     snprintf(help_text, sizeof(help_text),
                              ANSI_BOLD("%s %s") " -- %s    " ABORT_MSG,
-                             jph->jph_path,
+                             jph->jph_property.c_str(),
                              jph->jph_synopsis,
                              jph->jph_description);
                     lnav_data.ld_bottom_source.set_prompt(help_text);
@@ -221,7 +221,7 @@ void rl_change(void *dummy, readline_curses *rc)
 
                     al.clear();
                     etc.get_dimensions(height, width);
-                    format_example_text_for_term(ht, width, al);
+                    format_example_text_for_term(ht, eval_example, width, al);
                     lnav_data.ld_example_source.replace_with(al);
                 }
 
@@ -275,8 +275,8 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
     string term_val;
     string name;
 
-    tc->get_highlights().erase("$preview");
-    tc->get_highlights().erase("$bodypreview");
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "preview"});
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "bodypreview"});
     tc->reload_data();
 
     switch (lnav_data.ld_mode) {
@@ -294,17 +294,21 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
 
         lnav_data.ld_preview_status_source.get_description().clear();
         lnav_data.ld_preview_source.clear();
-        string result = execute_command(lnav_data.ld_exec_context, rc->get_value());
+        auto result = execute_command(lnav_data.ld_exec_context, rc->get_value());
 
-        if (result.empty()) {
-            lnav_data.ld_bottom_source.set_prompt(LNAV_CMD_PROMPT);
-            lnav_data.ld_bottom_source.grep_error("");
-        } else if (startswith(result, "error:")) {
-            lnav_data.ld_bottom_source.set_prompt("");
-            lnav_data.ld_bottom_source.grep_error(result);
+        if (result.isOk()) {
+            auto msg = result.unwrap();
+
+            if (msg.empty()) {
+                lnav_data.ld_bottom_source.set_prompt(LNAV_CMD_PROMPT);
+                lnav_data.ld_bottom_source.grep_error("");
+            } else {
+                lnav_data.ld_bottom_source.set_prompt(msg);
+                lnav_data.ld_bottom_source.grep_error("");
+            }
         } else {
-            lnav_data.ld_bottom_source.set_prompt(result);
-            lnav_data.ld_bottom_source.grep_error("");
+            lnav_data.ld_bottom_source.set_prompt("");
+            lnav_data.ld_bottom_source.grep_error(result.unwrapErr());
         }
 
         lnav_data.ld_preview_view.reload_data();
@@ -329,7 +333,7 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
                                          rc->get_value().c_str(),
                                          -1,
                                          stmt.out(),
-                                         NULL);
+                                         nullptr);
             if (retcode != SQLITE_OK) {
                 const char *errmsg = sqlite3_errmsg(lnav_data.ld_db);
 
@@ -347,53 +351,10 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
 
         annotate_sql_statement(al);
 
-        if (x > 0 && (int)x >= al.length()) {
-            x -= 1;
-        }
+        auto avail_help = find_sql_help_for_line(al, x);
 
-        while (x > 0 && isspace(al.get_string()[x])) {
-            x -= 1;
-        }
-
-        vector<string> kw;
-        auto iter = rfind_string_attr_if(sa, x, [&al, &name, &kw, x](auto sa) {
-            if (sa.sa_type != &SQL_FUNCTION_ATTR &&
-                sa.sa_type != &SQL_KEYWORD_ATTR) {
-                return false;
-            }
-
-            const string &str = al.get_string();
-            const line_range &lr = sa.sa_range;
-            int lpc;
-
-            if (sa.sa_type == &SQL_FUNCTION_ATTR) {
-                if (!sa.sa_range.contains(x)) {
-                    return false;
-                }
-            }
-
-            for (lpc = lr.lr_start; lpc < lr.lr_end; lpc++) {
-                if (!isalnum(str[lpc]) && str[lpc] != '_') {
-                    break;
-                }
-            }
-
-            string tmp_name = str.substr(lr.lr_start, lpc - lr.lr_start);
-            if (sa.sa_type == &SQL_KEYWORD_ATTR) {
-                tmp_name = toupper(tmp_name);
-            }
-            bool retval = sqlite_function_help.count(tmp_name) > 0;
-
-            if (retval) {
-                kw.push_back(tmp_name);
-                name = tmp_name;
-            }
-            return retval;
-        });
-
-        if (iter != sa.end()) {
-            auto func_pair = sqlite_function_help.equal_range(name);
-            size_t help_count = distance(func_pair.first, func_pair.second);
+        if (!avail_help.empty()) {
+            size_t help_count = avail_help.size();
             textview_curses &dtc = lnav_data.ld_doc_view;
             textview_curses &etc = lnav_data.ld_example_view;
             unsigned long doc_width, ex_width;
@@ -402,23 +363,12 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
 
             dtc.get_dimensions(doc_height, doc_width);
             etc.get_dimensions(ex_height, ex_width);
-            if (help_count > 1 && name != func_pair.first->second->ht_name) {
-                while (find(kw.begin(), kw.end(),
-                            func_pair.first->second->ht_name) == kw.end()) {
-                    ++func_pair.first;
-                }
-                func_pair.second = next(func_pair.first);
-                help_count = 1;
-            }
-            for (auto func_iter = func_pair.first;
-                 func_iter != func_pair.second;
-                 ++func_iter) {
-                const help_text &ht = *func_iter->second;
 
-                format_help_text_for_term(ht, min(70UL, doc_width), doc_al,
+            for (const auto& ht : avail_help) {
+                format_help_text_for_term(*ht, min(70UL, doc_width), doc_al,
                                           help_count > 1);
                 if (help_count == 1) {
-                    format_example_text_for_term(ht, ex_width, ex_al);
+                    format_example_text_for_term(*ht, eval_example, ex_width, ex_al);
                 }
             }
 
@@ -435,7 +385,7 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
             }
         }
 
-        auto ident_iter = find_string_attr_containing(sa, &SQL_IDENTIFIER_ATTR, x);
+        auto ident_iter = find_string_attr_containing(sa, &SQL_IDENTIFIER_ATTR, al.nearest_text(x));
         if (ident_iter != sa.end()) {
             string ident = al.get_substring(ident_iter->sa_range);
             intern_string_t intern_ident = intern_string::lookup(ident);
@@ -458,7 +408,7 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
 
             if (!ddl.empty()) {
                 lnav_data.ld_preview_source.replace_with(ddl)
-                    .set_text_format(TF_SQL)
+                    .set_text_format(text_format_t::TF_SQL)
                     .truncate_to(30);
                 lnav_data.ld_preview_status_source.get_description()
                          .set_value("Definition for table -- %s",
@@ -504,8 +454,12 @@ void rl_abort(void *dummy, readline_curses *rc)
     lnav_data.ld_doc_source.clear();
     lnav_data.ld_preview_status_source.get_description().clear();
     lnav_data.ld_preview_source.clear();
-    tc->get_highlights().erase("$preview");
-    tc->get_highlights().erase("$bodypreview");
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "preview"});
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "bodypreview"});
+
+    vector<string> errors;
+    lnav_config = rollback_lnav_config;
+    reload_config(errors);
 
     lnav_data.ld_bottom_source.grep_error("");
     switch (lnav_data.ld_mode) {
@@ -534,8 +488,8 @@ void rl_callback(void *dummy, readline_curses *rc)
     lnav_data.ld_example_source.clear();
     lnav_data.ld_preview_status_source.get_description().clear();
     lnav_data.ld_preview_source.clear();
-    tc->get_highlights().erase("$preview");
-    tc->get_highlights().erase("$bodypreview");
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "preview"});
+    tc->get_highlights().erase({highlight_source_t::PREVIEW, "bodypreview"});
     switch (lnav_data.ld_mode) {
     case LNM_PAGING:
     case LNM_FILTER:
@@ -544,7 +498,9 @@ void rl_callback(void *dummy, readline_curses *rc)
 
     case LNM_COMMAND:
         rc->set_alt_value("");
-        rc->set_value(execute_command(ec, rc->get_value()));
+        rc->set_value(execute_command(ec, rc->get_value())
+                          .map(ok_prefix)
+                          .orElse(err_to_ok).unwrap());
         break;
 
     case LNM_SEARCH:
@@ -573,17 +529,24 @@ void rl_callback(void *dummy, readline_curses *rc)
         break;
 
     case LNM_SQL: {
-        string result = execute_sql(ec, rc->get_value(), alt_msg);
+        auto result = execute_sql(ec, rc->get_value(), alt_msg);
         db_label_source &dls = lnav_data.ld_db_row_source;
+        string prompt;
 
-        if (!result.empty()) {
-            result = "SQL Result: " + result;
-            if (dls.dls_rows.size() > 1) {
-                ensure_view(&lnav_data.ld_views[LNV_DB]);
+        if (result.isOk()) {
+            auto msg = result.unwrap();
+
+            if (!msg.empty()) {
+                prompt = ok_prefix("SQL Result: " + msg);
+                if (dls.dls_rows.size() > 1) {
+                    ensure_view(&lnav_data.ld_views[LNV_DB]);
+                }
             }
+        } else {
+            prompt = result.orElse(err_to_ok).unwrap();
         }
 
-        rc->set_value(result);
+        rc->set_value(prompt);
         rc->set_alt_value(alt_msg);
         break;
     }
@@ -604,7 +567,9 @@ void rl_callback(void *dummy, readline_curses *rc)
             string path_and_args = rc->get_value();
 
             ec.ec_output_stack.back() = tmpout.in();
-            string result = execute_file(ec, path_and_args);
+            string result = execute_file(ec, path_and_args)
+                .map(ok_prefix)
+                .orElse(err_to_ok).unwrap();
             string::size_type lf_index = result.find('\n');
             if (lf_index != string::npos) {
                 result = result.substr(0, lf_index);
@@ -624,6 +589,7 @@ void rl_callback(void *dummy, readline_curses *rc)
                          timestamp);
                 lnav_data.ld_file_names[desc]
                     .with_fd(fd_copy)
+                    .with_include_in_session(false)
                     .with_detect_format(false);
                 lnav_data.ld_files_to_front.emplace_back(desc, 0);
 
