@@ -55,9 +55,11 @@
 
 #include <string>
 
-#include "pcrepp.hh"
+#include "lnav_config.hh"
+#include "pcrepp/pcrepp.hh"
+#include "shlex.hh"
 #include "auto_mem.hh"
-#include "lnav_log.hh"
+#include "base/lnav_log.hh"
 #include "lnav_util.hh"
 #include "ansi_scrubber.hh"
 #include "readline_curses.hh"
@@ -220,6 +222,7 @@ static ssize_t recvstring(int sock, char *buf, size_t len)
 char *readline_context::completion_generator(const char *text, int state)
 {
     static vector<string> matches;
+    vector<string> long_matches;
 
     char *retval = nullptr;
 
@@ -238,26 +241,52 @@ char *readline_context::completion_generator(const char *text, int state)
                     ((strchr(loaded_context->rc_quote_chars, poss_str[0]) !=
                         nullptr) &&
                      cmpfunc(text, &poss_str[1], len) == 0)) {
-                    matches.push_back(poss);
+                    auto poss_slash_count = std::count(poss.begin(), poss.end(), '/');
+
+                    if (endswith(poss.c_str(), "/")) {
+                        poss_slash_count -= 1;
+                    }
+                    if (std::count(&text[0], &text[len], '/') == poss_slash_count) {
+                        matches.emplace_back(poss);
+                    } else {
+                        long_matches.emplace_back(poss);
+                    }
                 }
             }
 
+            if (matches.empty()) {
+                matches = std::move(long_matches);
+            }
+
             if (matches.empty()) {
                 vector<pair<int, string>> fuzzy_matches;
+                vector<pair<int, string>> fuzzy_long_matches;
 
                 for (const auto &poss : (*arg_possibilities)) {
                     string poss_str = tolower(poss);
                     int score;
 
                     if (fts::fuzzy_match(text, poss_str.c_str(), score) && score > 0) {
-                        log_debug("match score %d %s %s", score, text, poss.c_str());
                         if (score <= 0) {
                             continue;
                         }
-                        fuzzy_matches.emplace_back(score, poss);
+
+                        auto poss_slash_count = std::count(poss_str.begin(), poss_str.end(), '/');
+                        if (endswith(poss.c_str(), "/")) {
+                            poss_slash_count -= 1;
+                        }
+                        if (std::count(&text[0], &text[len], '/') == poss_slash_count) {
+                            fuzzy_matches.emplace_back(score, poss);
+                        } else {
+                            fuzzy_long_matches.emplace_back(score, poss);
+                        }
                     }
                 }
 
+                if (fuzzy_matches.empty()) {
+                    fuzzy_matches = std::move(fuzzy_long_matches);
+                }
+
                 if (!fuzzy_matches.empty()) {
                     stable_sort(begin(fuzzy_matches), end(fuzzy_matches),
                         [](auto l, auto r) { return r.first < l.first; });
@@ -312,31 +341,50 @@ char **readline_context::attempted_completion(const char *text,
     else {
         char * space;
         string cmd;
-
-        rl_completion_append_character = 0;
-        space = strchr(rl_line_buffer, ' ');
-        if (space == nullptr) {
-            space = rl_line_buffer + strlen(rl_line_buffer);
+        vector<string> prefix;
+        int point = rl_point;
+        while (point > 0 && rl_line_buffer[point] != ' ') {
+            point -= 1;
         }
-        cmd = string(rl_line_buffer, space - rl_line_buffer);
+        shlex lexer(rl_line_buffer, point);
+        map<string, string> scope;
 
-        auto iter = loaded_context->rc_prototypes.find(cmd);
+        arg_possibilities = nullptr;
+        rl_completion_append_character = 0;
+        if (lexer.split(prefix, scope)) {
+            string prefix2 = join(prefix.begin(), prefix.end(), "\x1f");
+            auto prefix_iter = loaded_context->rc_prefixes.find(prefix2);
 
-        if (iter == loaded_context->rc_prototypes.end()) {
-            if (loaded_context->rc_possibilities.find("*") !=
-                loaded_context->rc_possibilities.end()) {
-                arg_possibilities = &loaded_context->rc_possibilities["*"];
-                rl_completion_append_character = loaded_context->rc_append_character;
+            if (prefix_iter != loaded_context->rc_prefixes.end()) {
+                arg_possibilities = &(loaded_context->rc_possibilities[prefix_iter->second]);
             }
-        } else {
-            vector<string> &proto = loaded_context->rc_prototypes[cmd];
+        }
 
-            if (proto.empty()) {
-                arg_possibilities = NULL;
-            } else if (proto[0] == "filename") {
-                return NULL; /* XXX */
+        if (arg_possibilities == nullptr) {
+            space = strchr(rl_line_buffer, ' ');
+            if (space == nullptr) {
+                space = rl_line_buffer + strlen(rl_line_buffer);
+            }
+            cmd = string(rl_line_buffer, space - rl_line_buffer);
+
+            auto iter = loaded_context->rc_prototypes.find(cmd);
+
+            if (iter == loaded_context->rc_prototypes.end()) {
+                if (loaded_context->rc_possibilities.find("*") !=
+                    loaded_context->rc_possibilities.end()) {
+                    arg_possibilities = &loaded_context->rc_possibilities["*"];
+                    rl_completion_append_character = loaded_context->rc_append_character;
+                }
             } else {
-                arg_possibilities = &(loaded_context->rc_possibilities[proto[0]]);
+                vector<string> &proto = loaded_context->rc_prototypes[cmd];
+
+                if (proto.empty()) {
+                    arg_possibilities = nullptr;
+                } else if (proto[0] == "filename") {
+                    return nullptr; /* XXX */
+                } else {
+                    arg_possibilities = &(loaded_context->rc_possibilities[proto[0]]);
+                }
             }
         }
     }
@@ -372,6 +420,36 @@ int readline_context::command_complete(int count, int key)
     return rl_insert(count, key);
 }
 
+readline_context::readline_context(const std::string &name,
+                                   readline_context::command_map_t *commands,
+                                   bool case_sensitive)
+    : rc_name(name),
+      rc_case_sensitive(case_sensitive),
+      rc_quote_chars("\"'"),
+      rc_highlighter(nullptr)
+{
+    if (commands != nullptr) {
+        command_map_t::iterator iter;
+
+        for (iter = commands->begin(); iter != commands->end(); ++iter) {
+            std::string cmd = iter->first;
+
+            this->rc_possibilities["__command"].insert(cmd);
+            iter->second->c_func(INIT_EXEC_CONTEXT, cmd, this->rc_prototypes[cmd]);
+        }
+    }
+
+    memset(&this->rc_history, 0, sizeof(this->rc_history));
+    history_set_history_state(&this->rc_history);
+
+    auto config_dir = dotlnav_path();
+    auto hpath = (config_dir / this->rc_name) + ".history";
+    read_history(hpath.str().c_str());
+    this->save();
+
+    this->rc_append_character = ' ';
+}
+
 readline_curses::readline_curses()
     : rc_active_context(-1),
       rc_child(-1),
@@ -596,7 +674,7 @@ void readline_curses::start()
                 }
                 else {
                     int  context, prompt_start = 0;
-                    char type[32];
+                    char type[1024];
 
                     msg[rc] = '\0';
                     if (sscanf(msg, "i:%d:%n", &rl_point, &prompt_start) == 1) {
@@ -640,6 +718,16 @@ void readline_curses::start()
                             _exit(1);
                         }
                     }
+                    else if (sscanf(msg,
+                                    "apre:%d:%1023[^\x1d]\x1d%n",
+                                    &context,
+                                    type,
+                                    &prompt_start) == 2) {
+                        require(this->rc_contexts[context] != NULL);
+
+                        this->rc_contexts[context]->rc_prefixes[string(type)] =
+                            string(&msg[prompt_start]);
+                    }
                     else if (sscanf(msg,
                                     "ap:%d:%31[^:]:%n",
                                     &context,
@@ -662,6 +750,9 @@ void readline_curses::start()
                                                       rem_possibility(string(type),
                                                                       string(&msg[prompt_start]));
                     }
+                    else if (sscanf(msg, "cpre:%d", &context) == 1) {
+                        this->rc_contexts[context]->rc_prefixes.clear();
+                    }
                     else if (sscanf(msg, "cp:%d:%s", &context, type)) {
                         this->rc_contexts[context]->clear_possibilities(type);
                     }
@@ -706,22 +797,15 @@ void readline_curses::start()
         }
     }
 
+    auto config_dir = dotlnav_path();
     std::map<int, readline_context *>::iterator citer;
     for (citer = this->rc_contexts.begin();
          citer != this->rc_contexts.end();
          ++citer) {
-        char *home;
-
         citer->second->load();
-        home = getenv("HOME");
-        if (home) {
-            char hpath[2048];
 
-            snprintf(hpath, sizeof(hpath),
-                     "%s/.lnav/%s.history",
-                     home, citer->second->get_name().c_str());
-            write_history(hpath);
-        }
+        auto hpath = (config_dir / citer->second->get_name()) + ".history";
+        write_history(hpath.str().c_str());
         citer->second->save();
     }
 
@@ -955,6 +1039,37 @@ void readline_curses::abort()
     }
 }
 
+void readline_curses::add_prefix(int context,
+                                 const vector<string> &prefix,
+                                 const string &value)
+{
+    char buffer[1024];
+    string prefix_wire = join(prefix.begin(), prefix.end(), "\x1f");
+
+    snprintf(buffer, sizeof(buffer),
+             "apre:%d:%s\x1d%s",
+             context,
+             prefix_wire.c_str(),
+             value.c_str());
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer) + 1) == -1) {
+        perror("add_prefix: write failed");
+    }
+}
+
+void readline_curses::clear_prefixes(int context)
+{
+    char buffer[1024];
+
+    snprintf(buffer, sizeof(buffer), "cpre:%d", context);
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer) + 1) == -1) {
+        perror("add_possibility: write failed");
+    }
+}
+
 void readline_curses::add_possibility(int context,
                                       const string &type,
                                       const string &value)
@@ -1015,9 +1130,11 @@ void readline_curses::do_update()
         int alt_start        = -1;
         struct line_range lr(0, 0);
         attr_line_t       al, alt_al;
+        view_colors &vc = view_colors::singleton();
 
         wmove(this->vc_window, this->get_actual_y(), this->vc_left);
-        wclrtoeol(this->vc_window);
+        wattron(this->vc_window, vc.attrs_for_role(view_colors::VCR_TEXT));
+        whline(this->vc_window, ' ', this->vc_width);
 
         if (time(nullptr) > this->rc_value_expiration) {
             this->rc_value.clear();
@@ -1070,7 +1187,7 @@ void readline_curses::do_update()
 
 std::string readline_curses::get_match_string() const
 {
-    auto len = this->vc_x - this->rc_match_start;
+    auto len = ::min((size_t) this->vc_x, this->rc_line_buffer.size()) - this->rc_match_start;
     auto context = this->get_active_context();
 
     if (context->get_append_character() != 0 &&
