@@ -31,7 +31,7 @@
 
 #include "config.h"
 
-#include "pcrepp.hh"
+#include "pcrepp/pcrepp.hh"
 #include "sql_util.hh"
 #include "shlex.hh"
 #include "lnav_util.hh"
@@ -79,7 +79,7 @@ static void find_matching_bracket(attr_line_t &al, int x, char left, char right)
     const string &line = al.get_string();
     int depth = 0;
 
-    if (x < 0 || x > line.length()) {
+    if (x < 0 || x > (int) line.length()) {
         return;
     }
 
@@ -359,6 +359,8 @@ void readline_command_highlighter(attr_line_t &al, int x)
         R"(^:(filter-in|filter-out|delete-filter|enable-filter|disable-filter|highlight|clear-highlight|create-search-table\s+[^\s]+\s+))");
     static const pcrepp SH_PREFIXES("^:(eval|open|append-to|write-to|write-csv-to|write-json-to)");
     static const pcrepp IDENT_PREFIXES("^:(tag|untag|delete-tags)");
+    static const pcrepp COLOR_PREFIXES("^:(config)");
+    static const pcrepp COLOR_RE("(#(?:[a-fA-F0-9]{3}|[a-fA-F0-9]{6}))");
 
     view_colors &vc = view_colors::singleton();
     int keyword_attrs = (
@@ -370,7 +372,6 @@ void readline_command_highlighter(attr_line_t &al, int x)
     size_t ws_index;
 
 
-
     ws_index = line.find(' ');
     string command = line.substr(0, ws_index);
     if (ws_index != string::npos) {
@@ -387,6 +388,28 @@ void readline_command_highlighter(attr_line_t &al, int x)
         readline_shlex_highlighter(al, x);
     }
     pi.reset(line);
+    if (COLOR_PREFIXES.match(pc, pi)) {
+        pi.reset(line);
+        if (COLOR_RE.match(pc, pi)) {
+            pcre_context::capture_t *cap = pc[0];
+            string hash_color = pi.get_substr(cap);
+            string errmsg;
+            rgb_color rgb_fg, rgb_bg;
+            attr_t color_hint_attrs = vc.attrs_for_role(view_colors::VCR_COLOR_HINT);
+            int pnum = PAIR_NUMBER(color_hint_attrs);
+
+            if (rgb_color::from_str(hash_color, rgb_bg, errmsg)) {
+                pnum -= 1;
+                vc.ensure_color_pair(pnum, rgb_fg, rgb_bg);
+
+                al.get_attrs().emplace_back(
+                    line_range{cap->c_begin, cap->c_begin + 1},
+                    &view_curses::VC_ROLE,
+                    view_colors::VCR_COLOR_HINT);
+            }
+        }
+    }
+    pi.reset(line);
     if (IDENT_PREFIXES.match(pc, pi) && ws_index != string::npos) {
         size_t start = ws_index, last;
 
@@ -520,34 +543,34 @@ void readline_shlex_highlighter(attr_line_t &al, int x)
 
     while (lexer.tokenize(cap, token)) {
         switch (token) {
-            case ST_ERROR:
+            case shlex_token_t::ST_ERROR:
                 al.with_attr(string_attr(
                         line_range(cap.c_begin, cap.c_end),
                         &view_curses::VC_STYLE,
                         error_attrs));
                 break;
-            case ST_TILDE:
-            case ST_ESCAPE:
+            case shlex_token_t::ST_TILDE:
+            case shlex_token_t::ST_ESCAPE:
                 al.with_attr(string_attr(
                         line_range(cap.c_begin, cap.c_end),
                         &view_curses::VC_STYLE,
                         special_char));
                 break;
-            case ST_DOUBLE_QUOTE_START:
-            case ST_SINGLE_QUOTE_START:
+            case shlex_token_t::ST_DOUBLE_QUOTE_START:
+            case shlex_token_t::ST_SINGLE_QUOTE_START:
                 quote_start = cap.c_begin;
                 break;
-            case ST_DOUBLE_QUOTE_END:
-            case ST_SINGLE_QUOTE_END:
+            case shlex_token_t::ST_DOUBLE_QUOTE_END:
+            case shlex_token_t::ST_SINGLE_QUOTE_END:
                 al.with_attr(string_attr(
                         line_range(quote_start, cap.c_end),
                         &view_curses::VC_STYLE,
                         string_attrs));
                 quote_start = -1;
                 break;
-            case ST_VARIABLE_REF:
-            case ST_QUOTED_VARIABLE_REF: {
-                int extra = token == ST_VARIABLE_REF ? 0 : 1;
+            case shlex_token_t::ST_VARIABLE_REF:
+            case shlex_token_t::ST_QUOTED_VARIABLE_REF: {
+                int extra = token == shlex_token_t::ST_VARIABLE_REF ? 0 : 1;
                 string ident = str.substr(cap.c_begin + 1 + extra, cap.length() - 1 - extra * 2);
                 int attrs = vc.attrs_for_ident(ident.c_str(), ident.size());
 
@@ -567,7 +590,7 @@ void readline_shlex_highlighter(attr_line_t &al, int x)
                 }
                 break;
             }
-            case ST_WHITESPACE:
+            case shlex_token_t::ST_WHITESPACE:
                 break;
         }
     }
