@@ -43,8 +43,8 @@
 #include <utility>
 #include <yajl/api/yajl_tree.h>
 
-#include "yajlpp.hh"
-#include "yajlpp_def.hh"
+#include "yajlpp/yajlpp.hh"
+#include "yajlpp/yajlpp_def.hh"
 #include "lnav.hh"
 #include "logfile.hh"
 #include "sql_util.hh"
@@ -133,7 +133,7 @@ template <typename... Args, std::size_t... Idx>
 int bind_values_helper(sqlite3_stmt *stmt, std::index_sequence<Idx...> idxs, Args ... args) {
     int rcs[] = { bind_to_sqlite(stmt, Idx + 1, args)... };
 
-    for (int lpc = 0; lpc < idxs.size(); lpc++) {
+    for (size_t lpc = 0; lpc < idxs.size(); lpc++) {
         if (rcs[lpc] != SQLITE_OK) {
             log_error("Failed to bind column %d in statement: %s", lpc, sqlite3_sql(stmt));
             return rcs[lpc];
@@ -165,19 +165,23 @@ static bool bind_line(sqlite3 *db,
     sqlite3_clear_bindings(stmt);
 
     auto line_iter = lf->begin() + cl;
-    shared_buffer_ref sbr;
-    lf->read_line(line_iter, sbr);
-    std::string line_hash = hash_bytes(sbr.get_data(), sbr.length(),
-                                       &cl, sizeof(cl),
-                                       NULL);
+    auto read_result = lf->read_line(line_iter);
 
-    int rc = bind_values(stmt,
-                         lf->original_line_time(line_iter),
-                         lf->get_format()->get_name(),
-                         line_hash,
-                         session_time);
+    if (read_result.isErr()) {
+        return false;
+    }
 
-    return rc == SQLITE_OK;
+    auto line_hash = read_result.map([cl](auto sbr) {
+        return hash_bytes(sbr.get_data(), sbr.length(),
+                          &cl, sizeof(cl),
+                          nullptr);
+    }).unwrap();
+
+    return bind_values(stmt,
+                       lf->original_line_time(line_iter),
+                       lf->get_format()->get_name(),
+                       line_hash,
+                       session_time) == SQLITE_OK;
 }
 
 struct session_file_info {
@@ -210,11 +214,9 @@ static void cleanup_session_data()
     static_root_mem<glob_t, globfree>   session_file_list;
     std::list<struct session_file_info> session_info_list;
     map<string, int> session_count;
-    string           session_file_pattern;
+    auto session_file_pattern = dotlnav_path() / "*-*.ts*.json";
 
-    session_file_pattern = dotlnav_path("*-*.ts*.json");
-
-    if (glob(session_file_pattern.c_str(),
+    if (glob(session_file_pattern.str().c_str(),
              0,
              nullptr,
              session_file_list.inout()) == 0) {
@@ -287,58 +289,59 @@ static void cleanup_session_data()
 }
 
 void init_session()
+{
+    lnav_data.ld_session_time = time(nullptr);
+    lnav_data.ld_session_id.clear();
+}
+
+static nonstd::optional<std::string> compute_session_id()
 {
     byte_array<2, uint64> hash;
     SpookyHash context;
-
-    lnav_data.ld_session_time = time(nullptr);
+    bool has_files = false;
 
     context.Init(0, 0);
     hash_updater updater(&context);
     for (auto &ld_file_name : lnav_data.ld_file_names) {
+        if (!ld_file_name.second.loo_include_in_session) {
+            continue;
+        }
+        has_files = true;
         updater(ld_file_name.first);
     }
+    if (!has_files) {
+        return nonstd::nullopt;
+    }
     context.Final(hash.out(0), hash.out(1));
 
-    lnav_data.ld_session_id = hash.to_string();
-
-    log_info("init_session: time=%d; id=%s", lnav_data.ld_session_time,
-        lnav_data.ld_session_id.c_str());
+    return hash.to_string();
 }
 
-void scan_sessions()
+nonstd::optional<session_pair_t> scan_sessions()
 {
-    std::list<session_pair_t> &session_file_names =
-        lnav_data.ld_session_file_names;
-
-    static_root_mem<glob_t, globfree>   view_info_list;
-    std::list<session_pair_t>::iterator iter;
-    char   view_info_pattern_base[128];
-    string view_info_pattern;
-    string old_session_name;
-    int    index;
+    static_root_mem<glob_t, globfree> view_info_list;
+    char view_info_pattern_base[128];
 
     cleanup_session_data();
 
-    if (lnav_data.ld_session_file_index >= 0 &&
-        lnav_data.ld_session_file_index < (int)session_file_names.size()) {
-        iter = session_file_names.begin();
-
-        advance(iter, lnav_data.ld_session_file_index);
-        old_session_name = iter->second;
+    const auto session_id = compute_session_id();
+    if (!session_id) {
+        return nonstd::nullopt;
     }
+    std::list<session_pair_t> &session_file_names =
+        lnav_data.ld_session_id[session_id.value()];
 
     session_file_names.clear();
 
     snprintf(view_info_pattern_base, sizeof(view_info_pattern_base),
              "view-info-%s.*.json",
-             lnav_data.ld_session_id.c_str());
-    view_info_pattern = dotlnav_path(view_info_pattern_base);
-    if (glob(view_info_pattern.c_str(), 0, nullptr,
+             session_id.value().c_str());
+    auto view_info_pattern = dotlnav_path() / view_info_pattern_base;
+    if (glob(view_info_pattern.str().c_str(), 0, nullptr,
              view_info_list.inout()) == 0) {
         for (size_t lpc = 0; lpc < view_info_list->gl_pathc; lpc++) {
             const char *path = view_info_list->gl_pathv[lpc];
-            int         timestamp, ppid, rc;
+            int timestamp, ppid, rc;
             const char *base;
 
             base = strrchr(path, '/');
@@ -373,16 +376,11 @@ void scan_sessions()
         session_file_names.pop_front();
     }
 
-    lnav_data.ld_session_file_index = ((int)session_file_names.size()) - 1;
-
-    for (index = 0, iter = session_file_names.begin();
-         iter != session_file_names.end();
-         index++, ++iter) {
-        if (iter->second == old_session_name) {
-            lnav_data.ld_session_file_index = index;
-            break;
-        }
+    if (session_file_names.empty()) {
+        return nonstd::nullopt;
     }
+
+    return nonstd::make_optional(session_file_names.back());
 }
 
 static void load_time_bookmarks()
@@ -390,15 +388,15 @@ static void load_time_bookmarks()
     logfile_sub_source &lss = lnav_data.ld_log_source;
     std::map<content_line_t, bookmark_metadata> &bm_meta = lss.get_user_bookmark_metadata();
     auto_mem<sqlite3, sqlite_close_wrapper> db;
-    string db_path = dotlnav_path(LOG_METADATA_NAME);
+    auto db_path = dotlnav_path() / LOG_METADATA_NAME;
     auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
     logfile_sub_source::iterator file_iter;
     bool reload_needed = false;
     auto_mem<char, sqlite3_free> errmsg;
 
-    log_info("loading bookmark db: %s", db_path.c_str());
+    log_info("loading bookmark db: %s", db_path.str().c_str());
 
-    if (sqlite3_open(db_path.c_str(), db.out()) != SQLITE_OK) {
+    if (sqlite3_open(db_path.str().c_str(), db.out()) != SQLITE_OK) {
         return;
     }
 
@@ -496,13 +494,18 @@ static void load_time_bookmarks()
                         break;
                     }
 
-                    shared_buffer_ref sbr;
                     content_line_t cl = content_line_t(std::distance(lf->begin(), line_iter));
-                    lf->read_line(line_iter, sbr);
+                    auto read_result = lf->read_line(line_iter);
+
+                    if(read_result.isErr()) {
+                        break;
+                    }
+
+                    auto sbr = read_result.unwrap();
 
                     string line_hash = hash_bytes(sbr.get_data(), sbr.length(),
                                                   &cl, sizeof(cl),
-                                                  NULL);
+                                                  nullptr);
 
                     if (line_hash == log_hash) {
                         content_line_t line_cl = content_line_t(
@@ -530,7 +533,7 @@ static void load_time_bookmarks()
                             } else {
                                 lss.set_user_mark(&textview_curses::BM_META,
                                                   line_cl);
-                                for (int lpc = 0; lpc < tag_list.in()->u.array.len; lpc++) {
+                                for (size_t lpc = 0; lpc < tag_list.in()->u.array.len; lpc++) {
                                     yajl_val elem = tag_list.in()->u.array.values[lpc];
 
                                     if (!YAJL_IS_STRING(elem)) {
@@ -659,9 +662,15 @@ static void load_time_bookmarks()
                         break;
                     }
 
-                    lf->read_line(line_iter, line);
+                    auto read_result = lf->read_line(line_iter);
 
-                    string line_hash = hash_string(line);
+                    if (read_result.isErr()) {
+                        break;
+                    }
+
+                    auto sbr = read_result.unwrap();
+
+                    string line_hash = hash_bytes(sbr.get_data(), sbr.length(), nullptr);
                     if (line_hash == log_hash) {
                         int file_line = std::distance(lf->begin(), line_iter);
                         content_line_t line_cl = content_line_t(
@@ -800,60 +809,57 @@ static int read_commands(yajlpp_parse_context *ypc, const unsigned char *str, si
     return 1;
 }
 
-static struct json_path_handler view_handlers[] = {
-        json_path_handler("top_line"),
-
-        json_path_handler()
+static struct json_path_container view_def_handlers = {
+    json_path_handler("top_line",  read_top_line),
+    json_path_handler("search",    read_last_search),
+    json_path_handler("word_wrap", read_word_wrap),
+    json_path_handler("commands#", read_commands)
 };
 
-static struct json_path_handler view_info_handlers[] = {
-    json_path_handler("/save-time",               read_save_time),
-    json_path_handler("/time-offset",             read_time_offset),
-    json_path_handler("/files#",                  read_files),
-    json_path_handler("/views/([^/]+)/top_line",  read_top_line),
-    json_path_handler("/views/([^/]+)/search",    read_last_search),
-    json_path_handler("/views/([^/]+)/word_wrap", read_word_wrap),
-    json_path_handler("/views/([^/]+)/commands#", read_commands),
+static struct json_path_container view_handlers = {
+    json_path_handler(pcrepp("([^/]+)"))
+        .with_children(view_def_handlers)
+};
 
-    json_path_handler()
+static struct json_path_container view_info_handlers = {
+    json_path_handler("save-time",               read_save_time),
+    json_path_handler("time-offset",             read_time_offset),
+    json_path_handler("files#",                  read_files),
+    json_path_handler("views")
+        .with_children(view_handlers)
 };
 
 void load_session()
 {
-    std::list<session_pair_t>::iterator sess_iter;
-    yajl_handle          handle;
-    auto_fd fd;
-
-    if (lnav_data.ld_session_file_names.empty()) {
-        load_time_bookmarks();
-        return;
-    }
-
-    sess_iter = lnav_data.ld_session_file_names.begin();
-    advance(sess_iter, lnav_data.ld_session_file_index);
-    lnav_data.ld_session_load_time = sess_iter->first.second;
-    lnav_data.ld_session_save_time = sess_iter->first.second;
-    string &view_info_name = sess_iter->second;
-
-    yajlpp_parse_context ypc(view_info_name, view_info_handlers);
-    handle    = yajl_alloc(&ypc.ypc_callbacks, nullptr, &ypc);
-
     load_time_bookmarks();
+    scan_sessions() | [](const auto pair) {
+        yajl_handle handle;
+        auto_fd fd;
 
-    if ((fd = open(view_info_name.c_str(), O_RDONLY)) < 0) {
-        perror("cannot open session file");
-    }
-    else {
-        unsigned char buffer[1024];
-        ssize_t        rc;
+        lnav_data.ld_session_load_time = pair.first.second;
+        lnav_data.ld_session_save_time = pair.first.second;
+        const string &view_info_name = pair.second;
 
-        log_info("loading session file: %s", view_info_name.c_str());
-        while ((rc = read(fd, buffer, sizeof(buffer))) > 0) {
-            yajl_parse(handle, buffer, rc);
+        yajlpp_parse_context ypc(view_info_name, &view_info_handlers);
+        handle = yajl_alloc(&ypc.ypc_callbacks, nullptr, &ypc);
+
+        load_time_bookmarks();
+
+        if ((fd = open(view_info_name.c_str(), O_RDONLY)) < 0) {
+            perror("cannot open session file");
         }
-        yajl_complete_parse(handle);
-    }
-    yajl_free(handle);
+        else {
+            unsigned char buffer[1024];
+            ssize_t        rc;
+
+            log_info("loading session file: %s", view_info_name.c_str());
+            while ((rc = read(fd, buffer, sizeof(buffer))) > 0) {
+                yajl_parse(handle, buffer, rc);
+            }
+            yajl_complete_parse(handle);
+        }
+        yajl_free(handle);
+    };
 }
 
 static void yajl_writer(void *context, const char *str, size_t len)
@@ -957,12 +963,12 @@ static void save_user_bookmarks(
 static void save_time_bookmarks()
 {
     auto_mem<sqlite3, sqlite_close_wrapper> db;
-    string db_path = dotlnav_path(LOG_METADATA_NAME);
+    auto db_path = dotlnav_path() / LOG_METADATA_NAME;
     auto_mem<char, sqlite3_free> errmsg;
     auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
 
-    if (sqlite3_open(db_path.c_str(), db.out()) != SQLITE_OK) {
-        log_error("unable to open bookmark DB -- %s\n", db_path.c_str());
+    if (sqlite3_open(db_path.str().c_str(), db.out()) != SQLITE_OK) {
+        log_error("unable to open bookmark DB -- %s\n", db_path.str().c_str());
         return;
     }
 
@@ -1166,10 +1172,18 @@ static void save_time_bookmarks()
         line_iter = lf->begin() + lf->get_time_offset_line();
         struct timeval offset = lf->get_time_offset();
 
+        auto read_result = lf->read_line(line_iter);
+
+        if (read_result.isErr()) {
+            return;
+        }
+
         bind_values(stmt.in(),
                     lf->original_line_time(line_iter),
                     lf->get_format()->get_name(),
-                    hash_string(lf->read_line(line_iter)),
+                    read_result.map([](auto sbr) {
+                        return hash_bytes(sbr.get_data(), sbr.length(), nullptr);
+                    }).unwrap(),
                     lnav_data.ld_session_time,
                     offset.tv_sec,
                     offset.tv_usec);
@@ -1195,28 +1209,26 @@ static void save_time_bookmarks()
     }
 }
 
-void save_session()
+static void save_session_with_id(const std::string session_id)
 {
-    string view_file_name, view_file_tmp_name;
-
     auto_mem<FILE> file(fclose);
-    char           view_base_name[256];
-    yajl_gen       handle = nullptr;
-
-    save_time_bookmarks();
+    yajl_gen handle = nullptr;
 
     /* TODO: save the last search query */
 
+    log_info("saving session with id: %s", session_id.c_str());
+
+    char view_base_name[256];
     snprintf(view_base_name, sizeof(view_base_name),
              "view-info-%s.ts%ld.ppid%d.json",
-             lnav_data.ld_session_id.c_str(),
+             session_id.c_str(),
              lnav_data.ld_session_time,
              getppid());
 
-    view_file_name     = dotlnav_path(view_base_name);
-    view_file_tmp_name = view_file_name + ".tmp";
+    auto view_file_name = dotlnav_path() / view_base_name;
+    auto view_file_tmp_name = view_file_name + ".tmp";
 
-    if ((file = fopen(view_file_tmp_name.c_str(), "w")) == nullptr) {
+    if ((file = fopen(view_file_tmp_name.str().c_str(), "w")) == nullptr) {
         perror("Unable to open session file");
     }
     else if (nullptr == (handle = yajl_gen_alloc(nullptr))) {
@@ -1280,13 +1292,12 @@ void save_session()
                         continue;
                     }
 
-                    filter_stack::iterator filter_iter;
                     filter_stack &fs = tss->get_filters();
 
                     view_map.gen("commands");
                     yajlpp_array cmd_array(handle);
 
-                    for (const auto filter : fs) {
+                    for (const auto& filter : fs) {
                         string cmd = filter->to_command();
 
                         if (cmd.empty()) {
@@ -1307,10 +1318,10 @@ void save_session()
                     for (hl_iter = hmap.begin();
                          hl_iter != hmap.end();
                          ++hl_iter) {
-                        if (hl_iter->first[0] == '$') {
+                        if (hl_iter->first.first != highlight_source_t::INTERACTIVE) {
                             continue;
                         }
-                        cmd_array.gen("highlight " + hl_iter->first);
+                        cmd_array.gen("highlight " + hl_iter->first.second);
                     }
 
                     if (lpc == LNV_LOG) {
@@ -1321,7 +1332,7 @@ void save_session()
                                 continue;
                             }
 
-                            for (auto vd : elf->elf_value_defs) {
+                            for (const auto& vd : elf->elf_value_defs) {
                                 if (!vd.second->vd_user_hidden) {
                                     continue;
                                 }
@@ -1360,31 +1371,47 @@ void save_session()
 
         fclose(file.release());
 
-        log_perror(rename(view_file_tmp_name.c_str(), view_file_name.c_str()));
+        log_perror(rename(view_file_tmp_name.str().c_str(),
+            view_file_name.str().c_str()));
 
-        log_info("Saved session: %s", view_file_name.c_str());
+        log_info("Saved session: %s", view_file_name.str().c_str());
+    }
+}
+
+void save_session()
+{
+    save_time_bookmarks();
+
+    const auto opt_session_id = compute_session_id();
+    opt_session_id | [](auto &session_id) {
+        save_session_with_id(session_id);
+    };
+    for (const auto pair : lnav_data.ld_session_id) {
+        if (opt_session_id && pair.first == opt_session_id.value()) {
+            continue;
+        }
+        save_session_with_id(pair.first);
     }
 }
 
 void reset_session()
 {
-    textview_curses::highlight_map_t &hmap =
-        lnav_data.ld_views[LNV_LOG].get_highlights();
-    auto hl_iter = hmap.begin();
-
     log_info("reset session: time=%d", lnav_data.ld_session_time);
 
     save_session();
-    scan_sessions();
 
     lnav_data.ld_session_time = time(nullptr);
 
-    while (hl_iter != hmap.end()) {
-        if (hl_iter->first[0] == '$') {
-            ++hl_iter;
-        }
-        else {
-            hmap.erase(hl_iter++);
+    for (auto &tc : lnav_data.ld_views) {
+        auto &hmap = tc.get_highlights();
+        auto hl_iter = hmap.begin();
+
+        while (hl_iter != hmap.end()) {
+            if (hl_iter->first.first != highlight_source_t::INTERACTIVE) {
+                ++hl_iter;
+            } else {
+                hmap.erase(hl_iter++);
+            }
         }
     }
 
