@@ -40,9 +40,10 @@
 
 #include "auto_mem.hh"
 #include "sql_util.hh"
-#include "lnav_log.hh"
+#include "base/lnav_log.hh"
 #include "lnav_util.hh"
-#include "pcrepp.hh"
+#include "pcrepp/pcrepp.hh"
+#include "sqlite-extension-func.hh"
 
 using namespace std;
 
@@ -56,6 +57,7 @@ const char *sql_keywords[] = {
     "AFTER",
     "ALL",
     "ALTER",
+    "ALWAYS",
     "ANALYZE",
     "AND",
     "AS",
@@ -77,6 +79,7 @@ const char *sql_keywords[] = {
     "CONSTRAINT",
     "CREATE",
     "CROSS",
+    "CURRENT",
     "CURRENT_DATE",
     "CURRENT_TIME",
     "CURRENT_TIMESTAMP",
@@ -88,22 +91,29 @@ const char *sql_keywords[] = {
     "DESC",
     "DETACH",
     "DISTINCT",
+    "DO",
     "DROP",
     "EACH",
     "ELSE",
     "END",
     "ESCAPE",
     "EXCEPT",
+    "EXCLUDE",
     "EXCLUSIVE",
     "EXISTS",
     "EXPLAIN",
     "FAIL",
+    "FILTER",
+    "FIRST",
+    "FOLLOWING",
     "FOR",
     "FOREIGN",
     "FROM",
     "FULL",
+    "GENERATED",
     "GLOB",
-    "GROUP BY",
+    "GROUP",
+    "GROUPS",
     "HAVING",
     "IF",
     "IGNORE",
@@ -121,6 +131,7 @@ const char *sql_keywords[] = {
     "ISNULL",
     "JOIN",
     "KEY",
+    "LAST",
     "LEFT",
     "LIKE",
     "LIMIT",
@@ -128,19 +139,27 @@ const char *sql_keywords[] = {
     "NATURAL",
     "NO",
     "NOT",
+    "NOTHING",
     "NOTNULL",
     "NULL",
+    "NULLS",
     "OF",
     "OFFSET",
     "ON",
     "OR",
-    "ORDER BY",
+    "ORDER",
+    "OTHERS",
     "OUTER",
+    "OVER",
+    "PARTITION",
     "PLAN",
     "PRAGMA",
+    "PRECEDING",
     "PRIMARY",
     "QUERY",
     "RAISE",
+    "RANGE",
+    "RECURSIVE",
     "REFERENCES",
     "REGEXP",
     "REINDEX",
@@ -151,6 +170,7 @@ const char *sql_keywords[] = {
     "RIGHT",
     "ROLLBACK",
     "ROW",
+    "ROWS",
     "SAVEPOINT",
     "SELECT",
     "SET",
@@ -158,9 +178,11 @@ const char *sql_keywords[] = {
     "TEMP",
     "TEMPORARY",
     "THEN",
+    "TIES",
     "TO",
     "TRANSACTION",
     "TRIGGER",
+    "UNBOUNDED",
     "UNION",
     "UNIQUE",
     "UPDATE",
@@ -171,7 +193,9 @@ const char *sql_keywords[] = {
     "VIRTUAL",
     "WHEN",
     "WHERE",
+    "WINDOW",
     "WITH",
+    "WITHOUT",
 };
 
 const char *sql_function_names[] = {
@@ -230,6 +254,8 @@ const char *sql_function_names[] = {
     NULL
 };
 
+multimap<std::string, help_text *> sqlite_function_help;
+
 static int handle_db_list(void *ptr,
                           int ncols,
                           char **colvalues,
@@ -521,20 +547,20 @@ static void sqlite_logger(void *dummy, int code, const char *msg)
 
     switch (code) {
     case SQLITE_OK:
-        level = LOG_LEVEL_DEBUG;
+        level = lnav_log_level_t::DEBUG;
         break;
 #ifdef SQLITE_NOTICE
     case SQLITE_NOTICE:
-        level = LOG_LEVEL_INFO;
+        level = lnav_log_level_t::INFO;
         break;
 #endif
 #ifdef SQLITE_WARNING
     case SQLITE_WARNING:
-        level = LOG_LEVEL_WARNING;
+        level = lnav_log_level_t::WARNING;
         break;
 #endif
     default:
-        level = LOG_LEVEL_ERROR;
+        level = lnav_log_level_t::ERROR;
         break;
     }
 
@@ -769,22 +795,22 @@ static struct {
     const char *collator;
     const char *sample;
 } TYPE_TEST_VALUE[] = {
-        { SQLITE3_TEXT, NULL, "foobar" },
-        { SQLITE_INTEGER, NULL, "123" },
-        { SQLITE_FLOAT, NULL, "123.0" },
+        { SQLITE3_TEXT, "", "foobar" },
+        { SQLITE_INTEGER, "", "123" },
+        { SQLITE_FLOAT, "", "123.0" },
         { SQLITE_TEXT, "ipaddress", "127.0.0.1" },
 
         { SQLITE_NULL }
 };
 
-int guess_type_from_pcre(const string &pattern, const char **collator)
+int guess_type_from_pcre(const string &pattern, std::string &collator)
 {
     try {
         pcrepp re(pattern.c_str());
         vector<int> matches;
         int retval = SQLITE3_TEXT;
 
-        *collator = NULL;
+        collator.clear();
         for (int lpc = 0; TYPE_TEST_VALUE[lpc].sqlite_type != SQLITE_NULL; lpc++) {
             pcre_context_static<30> pc;
             pcre_input pi(TYPE_TEST_VALUE[lpc].sample);
@@ -797,7 +823,7 @@ int guess_type_from_pcre(const string &pattern, const char **collator)
 
         if (matches.size() == 1) {
             retval = TYPE_TEST_VALUE[matches.front()].sqlite_type;
-            *collator = TYPE_TEST_VALUE[matches.front()].collator;
+            collator = TYPE_TEST_VALUE[matches.front()].collator;
         }
 
         return retval;
@@ -849,6 +875,7 @@ string_attr_type SQL_FUNCTION_ATTR("sql_func");
 string_attr_type SQL_STRING_ATTR("sql_string");
 string_attr_type SQL_OPERATOR_ATTR("sql_oper");
 string_attr_type SQL_PAREN_ATTR("sql_paren");
+string_attr_type SQL_COMMA_ATTR("sql_comma");
 string_attr_type SQL_GARBAGE_ATTR("sql_garbage");
 
 void annotate_sql_statement(attr_line_t &al)
@@ -860,12 +887,13 @@ void annotate_sql_statement(attr_line_t &al)
         pcrepp re;
         string_attr_type_t type;
     } PATTERNS[] = {
-        { {keyword_re_str.c_str(), PCRE_CASELESS}, &SQL_KEYWORD_ATTR },
-        { {R"(\A'[^']*('(?:'[^']*')*|$))"}, &SQL_STRING_ATTR },
-        { {R"(\A(\$?\b[a-z_]\w*)|\"([^\"]+)\"|\[([^\]]+)])", PCRE_CASELESS}, &SQL_IDENTIFIER_ATTR },
-        { {R"(\A(\*|<|>|=|!|\-|\+|\|\|))"}, &SQL_OPERATOR_ATTR },
-        { {R"(\A\(|\))"}, &SQL_PAREN_ATTR },
-        { {R"(\A.)"}, &SQL_GARBAGE_ATTR },
+        { pcrepp{R"(\A,)"}, &SQL_COMMA_ATTR },
+        { pcrepp{R"(\A\(|\A\))"}, &SQL_PAREN_ATTR },
+        { pcrepp{keyword_re_str.c_str(), PCRE_CASELESS}, &SQL_KEYWORD_ATTR },
+        { pcrepp{R"(\A'[^']*('(?:'[^']*')*|$))"}, &SQL_STRING_ATTR },
+        { pcrepp{R"(\A(\$?\b[a-z_]\w*)|\"([^\"]+)\"|\[([^\]]+)])", PCRE_CASELESS}, &SQL_IDENTIFIER_ATTR },
+        { pcrepp{R"(\A(\*|<|>|=|!|\-|\+|\|\|))"}, &SQL_OPERATOR_ATTR },
+        { pcrepp{R"(\A.)"}, &SQL_GARBAGE_ATTR },
     };
 
     static pcrepp ws_pattern(R"(\A\s+)");
@@ -934,4 +962,74 @@ void annotate_sql_statement(attr_line_t &al)
     }
 
     remove_string_attr(sa, &SQL_PAREN_ATTR);
+    stable_sort(sa.begin(), sa.end());
+}
+
+vector<const help_text *> find_sql_help_for_line(const attr_line_t &al, size_t x)
+{
+    vector<const help_text *> retval;
+    const auto& sa = al.get_attrs();
+    string name;
+
+    x = al.nearest_text(x);
+
+    vector<string> kw;
+    auto iter = rfind_string_attr_if(sa, x, [&al, &name, &kw, x](auto sa) {
+        if (sa.sa_type != &SQL_FUNCTION_ATTR &&
+            sa.sa_type != &SQL_KEYWORD_ATTR) {
+            return false;
+        }
+
+        const string &str = al.get_string();
+        const line_range &lr = sa.sa_range;
+        int lpc;
+
+        if (sa.sa_type == &SQL_FUNCTION_ATTR) {
+            if (!sa.sa_range.contains(x)) {
+                return false;
+            }
+        }
+
+        for (lpc = lr.lr_start; lpc < lr.lr_end; lpc++) {
+            if (!isalnum(str[lpc]) && str[lpc] != '_') {
+                break;
+            }
+        }
+
+        string tmp_name = str.substr(lr.lr_start, lpc - lr.lr_start);
+        if (sa.sa_type == &SQL_KEYWORD_ATTR) {
+            tmp_name = toupper(tmp_name);
+        }
+        bool retval = sqlite_function_help.count(tmp_name) > 0;
+
+        if (retval) {
+            kw.push_back(tmp_name);
+            name = tmp_name;
+        }
+        return retval;
+    });
+
+    if (iter != sa.end()) {
+        auto func_pair = sqlite_function_help.equal_range(name);
+        size_t help_count = distance(func_pair.first, func_pair.second);
+
+        if (help_count > 1 && name != func_pair.first->second->ht_name) {
+            while (func_pair.first != func_pair.second) {
+                if (find(kw.begin(), kw.end(),
+                         func_pair.first->second->ht_name) == kw.end()) {
+                    ++func_pair.first;
+                } else {
+                    func_pair.second = next(func_pair.first);
+                    break;
+                }
+            }
+        }
+        for (auto func_iter = func_pair.first;
+             func_iter != func_pair.second;
+             ++func_iter) {
+            retval.emplace_back(func_iter->second);
+        }
+    }
+
+    return retval;
 }
