@@ -40,7 +40,7 @@
 #include <utility>
 
 #include "lnav.hh"
-#include "lnav_log.hh"
+#include "base/lnav_log.hh"
 #include "sql_util.hh"
 #include "views_vtab.hh"
 #include "view_curses.hh"
@@ -123,13 +123,14 @@ struct lnav_views : public tvt_iterator_cursor<lnav_views> {
     static constexpr const char *CREATE_STMT = R"(
 -- Access lnav's views through this table.
 CREATE TABLE lnav_views (
-    name text PRIMARY KEY,  -- The name of the view.
-    top integer,            -- The number of the line at the top of the view, starting from zero.
-    left integer,           -- The left position of the viewport.
-    height integer,         -- The height of the viewport.
-    inner_height integer,   -- The number of lines in the view.
-    top_time datetime,      -- The time of the top line in the view, if the content is time-based.
-    search text             -- The text to search for in the view.
+    name TEXT PRIMARY KEY,  -- The name of the view.
+    top INTEGER,            -- The number of the line at the top of the view, starting from zero.
+    left INTEGER,           -- The left position of the viewport.
+    height INTEGER,         -- The height of the viewport.
+    inner_height INTEGER,   -- The number of lines in the view.
+    top_time DATETIME,      -- The time of the top line in the view, if the content is time-based.
+    paused INTEGER,         -- Indicates if the view is paused and will not load new data.
+    search TEXT             -- The text to search for in the view.
 );
 )";
 
@@ -169,7 +170,7 @@ CREATE TABLE lnav_views (
                 sqlite3_result_int(ctx, tc.get_inner_height());
                 break;
             case 5: {
-                text_time_translator *time_source = dynamic_cast<text_time_translator *>(tc.get_sub_source());
+                auto *time_source = dynamic_cast<text_time_translator *>(tc.get_sub_source());
 
                 if (time_source != nullptr && tc.get_inner_height() > 0) {
                     char timestamp[64];
@@ -182,7 +183,10 @@ CREATE TABLE lnav_views (
                 }
                 break;
             }
-            case 6: {
+            case 6:
+                sqlite3_result_int(ctx, tc.is_paused());
+                break;
+            case 7: {
                 const string &str = tc.get_last_search();
 
                 sqlite3_result_text(ctx, str.c_str(), str.length(), SQLITE_TRANSIENT);
@@ -213,6 +217,7 @@ CREATE TABLE lnav_views (
                    int64_t height,
                    int64_t inner_height,
                    const char *top_time,
+                   bool is_paused,
                    const char *search) {
         textview_curses &tc = lnav_data.ld_views[index];
         text_time_translator *time_source = dynamic_cast<text_time_translator *>(tc.get_sub_source());
@@ -237,6 +242,7 @@ CREATE TABLE lnav_views (
             }
         }
         tc.set_left(left);
+        tc.set_paused(is_paused);
         tc.execute_search(search);
 
         return SQLITE_OK;
@@ -249,7 +255,7 @@ struct lnav_view_stack : public tvt_iterator_cursor<lnav_view_stack> {
     static constexpr const char *CREATE_STMT = R"(
 -- Access lnav's view stack through this table.
 CREATE TABLE lnav_view_stack (
-    name text
+    name TEXT
 );
 )";
 
@@ -358,7 +364,7 @@ struct lnav_view_filter_base {
                 filter_stack &fs = tss->get_filters();
 
                 this->i_filter_index += 1;
-                if (this->i_filter_index >= fs.size()) {
+                if (this->i_filter_index >= (ssize_t) fs.size()) {
                     this->i_filter_index = -1;
                     this->i_view_index = lnav_view_t(this->i_view_index + 1);
                 } else {
@@ -409,11 +415,11 @@ struct lnav_view_filters : public tvt_iterator_cursor<lnav_view_filters>,
     static constexpr const char *CREATE_STMT = R"(
 -- Access lnav's filters through this table.
 CREATE TABLE lnav_view_filters (
-    view_name text,     -- The name of the view.
-    filter_id integer,  -- The filter identifier.
-    enabled integer,    -- Indicates if the filter is enabled/disabled.
-    type text,          -- The type of filter (i.e. in/out).
-    pattern text        -- The filter pattern.
+    view_name TEXT,                   -- The name of the view.
+    filter_id INTEGER DEFAULT 0,      -- The filter identifier.
+    enabled   INTEGER DEFAULT 1,      -- Indicates if the filter is enabled/disabled.
+    type      TEXT    DEFAULT 'out',  -- The type of filter (i.e. in/out).
+    pattern   TEXT                    -- The filter pattern.
 );
 )";
 
@@ -461,18 +467,22 @@ CREATE TABLE lnav_view_filters (
     int insert_row(sqlite3_vtab *tab,
                    sqlite3_int64 &rowid_out,
                    lnav_view_t view_index,
-                   int64_t _filter_id,
-                   bool enabled,
-                   text_filter::type_t type,
+                   nonstd::optional<int64_t> _filter_id,
+                   nonstd::optional<bool> enabled,
+                   nonstd::optional<text_filter::type_t> type,
                    pair<string, pcre *> pattern) {
         textview_curses &tc = lnav_data.ld_views[view_index];
         text_sub_source *tss = tc.get_sub_source();
         filter_stack &fs = tss->get_filters();
-        auto pf = make_shared<pcre_filter>(type,
-                                           pattern.first,
-                                           fs.next_index(),
-                                           pattern.second);
+        auto pf = make_shared<pcre_filter>(
+            type.value_or(text_filter::type_t::EXCLUDE),
+            pattern.first,
+            fs.next_index(),
+            pattern.second);
         fs.add_filter(pf);
+        if (!enabled.value_or(true)) {
+            pf->disable();
+        }
         tss->text_filters_changed();
         tc.set_needs_update();
 
@@ -480,8 +490,8 @@ CREATE TABLE lnav_view_filters (
     }
 
     int delete_row(sqlite3_vtab *tab, sqlite3_int64 rowid) {
-        lnav_view_t view_index = lnav_view_t(rowid >> 32);
-        int filter_index = rowid & 0xffffffffLL;
+        auto view_index = lnav_view_t(rowid >> 32);
+        size_t filter_index = rowid & 0xffffffffLL;
         textview_curses &tc = lnav_data.ld_views[view_index];
         text_sub_source *tss = tc.get_sub_source();
         filter_stack &fs = tss->get_filters();
