@@ -35,7 +35,7 @@
 #include <fstream>
 #include <iostream>
 
-#include "pcrepp.hh"
+#include "pcrepp/pcrepp.hh"
 #include "textview_curses.hh"
 #include "data_scanner.hh"
 #include "data_parser.hh"
@@ -66,7 +66,8 @@ int main(int argc, char *argv[])
     bool prompt = false, is_log = false, pretty_print = false;
 
     {
-        std::vector<std::string> paths, errors;
+        std::vector<filesystem::path> paths;
+        vector<string> errors;
 
         load_formats(paths, errors);
     }
@@ -153,8 +154,11 @@ int main(int argc, char *argv[])
                     for (iter = root_formats.begin();
                        iter != root_formats.end() && !found;
                        ++iter) {
+                        line_info li = { {13}};
+                        logfile *lf = nullptr; // XXX
+
                         (*iter)->clear();
-                        if ((*iter)->scan(nonstd::nullopt, index, 13, sbr) == log_format::SCAN_MATCH) {
+                        if ((*iter)->scan(*lf, index, li, sbr) == log_format::SCAN_MATCH) {
                             format = (*iter)->specialized();
                             found = true;
                         }
