@@ -60,13 +60,6 @@ void rebuild_indexes()
 
 readline_context::command_map_t lnav_commands;
 
-extern "C" {
-const char *help_txt = "";
-}
-
-extern "C" {
-const char *default_log_formats_json = "";
-}
 int main(int argc, char *argv[])
 {
     int retval = EXIT_SUCCESS;
