@@ -1,5 +1,41 @@
 #! /bin/bash
 
+run_test ${lnav_test} -n -d /tmp/lnav.err \
+    -c ":goto 0" \
+    -c ":next-mark error" \
+    -c ":prev-location" \
+    ${test_dir}/logfile_access_log.0
+
+check_output "prev-location is not working" <<EOF
+192.168.202.254 - - [20/Jul/2009:22:59:26 +0000] "GET /vmw/cgi/tramp HTTP/1.0" 200 134 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkboot.gz HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
+EOF
+
+run_test ${lnav_test} -n -d /tmp/lnav.err \
+    -c ":goto 0" \
+    -c ":next-mark error" \
+    -c ":prev-location" \
+    -c ":next-location" \
+    ${test_dir}/logfile_access_log.0
+
+check_output "next-location is not working" <<EOF
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkboot.gz HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
+EOF
+
+run_test ${lnav_test} -n -d /tmp/lnav.err \
+    -c ":filter-in vmk" \
+    -c ":rebuild" \
+    -c ":reset-session" \
+    -c ":rebuild" \
+    ${test_dir}/logfile_access_log.0
+
+check_output "filter-in vmk is not working" <<EOF
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkboot.gz HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
+EOF
+
 run_test ${lnav_test} -n \
     -c ":switch-to-view help" \
     ${test_dir}/logfile_access_log.0
@@ -12,7 +48,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "hide-fields with unknown" <<EOF
-error: unknown field(s) -- foobar
+command-option:1: error: unknown field(s) -- foobar
 EOF
 
 run_test ${lnav_test} -n \
@@ -20,9 +56,9 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_output "hide-fields with uri" <<EOF
-192.168.202.254 - - [20/Jul/2009:22:59:26 +0000] "GET ... HTTP/1.0" 200 134 "-" "gPXE/0.9.7"
-192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET ... HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
-192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET ... HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:22:59:26 +0000] "GET ⋮ HTTP/1.0" 200 134 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET ⋮ HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET ⋮ HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
 EOF
 
 run_test ${lnav_test} -n \
@@ -30,9 +66,9 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_output "hide-fields with IP and uri" <<EOF
-... - - [20/Jul/2009:22:59:26 +0000] "GET ... HTTP/1.0" 200 134 "-" "gPXE/0.9.7"
-... - - [20/Jul/2009:22:59:29 +0000] "GET ... HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
-... - - [20/Jul/2009:22:59:29 +0000] "GET ... HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
+⋮ - - [20/Jul/2009:22:59:26 +0000] "GET ⋮ HTTP/1.0" 200 134 "-" "gPXE/0.9.7"
+⋮ - - [20/Jul/2009:22:59:29 +0000] "GET ⋮ HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
+⋮ - - [20/Jul/2009:22:59:29 +0000] "GET ⋮ HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
 EOF
 
 run_test ${lnav_test} -f- -n < ${test_dir}/formats/scripts/multiline-echo.lnav
@@ -47,7 +83,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "config bad option" <<EOF
-error: unknown configuration option -- /bad/option
+command-option:1: error: unknown configuration option -- /bad/option
 EOF
 
 check_output "config bad option" <<EOF
@@ -90,7 +126,7 @@ EOF
 
 check_output "config clock-format" <<EOF
 info: changed config option -- /ui/clock-format
-info: reset option
+info: reset option -- /ui/clock-format
 info: /ui/clock-format = "%a %b %d %H:%M:%S %Z"
 EOF
 
@@ -113,7 +149,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "include nonexistent" <<EOF
-error:command-option:1:unknown script -- nonexistent.lnav -- file not found
+command-option:1: error: unknown script -- nonexistent.lnav -- file not found
 EOF
 
 
@@ -127,6 +163,16 @@ check_output "adjust-log-time is not working" <<EOF
 192.168.202.254 - - [01/Jan/2010:00:00:03 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
 EOF
 
+run_test ${lnav_test} -n \
+    -c ":adjust-log-time -1h" \
+    ${test_dir}/logfile_access_log.0
+
+check_output "adjust-log-time is not working" <<EOF
+192.168.202.254 - - [20/Jul/2009:21:59:26 +0000] "GET /vmw/cgi/tramp HTTP/1.0" 200 134 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:21:59:29 +0000] "GET /vmw/vSphere/default/vmkboot.gz HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
+192.168.202.254 - - [20/Jul/2009:21:59:29 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
+EOF
+
 
 run_test ${lnav_test} -n \
     -c ":goto 1" \
@@ -185,7 +231,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "goto invalid is working" <<EOF
-error: expecting line number/percentage, timestamp, or relative time
+command-option:1: error: expecting line number/percentage, timestamp, or relative time
 EOF
 
 check_output "goto invalid is not working" <<EOF
@@ -230,7 +276,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "goto invalid is not working" <<EOF
-error: unknown bookmark type
+command-option:2: error: unknown bookmark type
 EOF
 
 check_output "invalid mark-type is working" <<EOF
@@ -380,7 +426,7 @@ run_test eval ${lnav_test} -d /tmp/lnav.err -n \
     $TOO_MANY_FILTERS \
     ${test_dir}/logfile_filter.0
 check_error_output "able to create too many filters?" <<EOF
-error: filter limit reached, try combining filters with a pipe symbol (e.g. foo|bar)
+command-option:33: error: filter limit reached, try combining filters with a pipe symbol (e.g. foo|bar)
 EOF
 
 
@@ -398,7 +444,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "double close works" <<EOF
-error: no log files loaded
+command-option:2: error: no log files loaded
 EOF
 
 check_output "double close is working" <<EOF
@@ -423,7 +469,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "open non-existent is working" <<EOF
-error: cannot stat file: /non-existent -- No such file or directory
+command-option:2: error: cannot stat file: /non-existent -- No such file or directory
 EOF
 
 check_output "open non-existent is not working" <<EOF
@@ -443,10 +489,10 @@ EOF
 
 run_test ${lnav_test} -n \
     -c ":goto 1" \
-    -c ":write-raw-to -" \
+    -c ":write-screen-to -" \
     "${test_dir}/logfile_access_log.0"
 
-check_output "write-raw-to not working" <<EOF
+check_output "write-screen-to not working" <<EOF
 192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkboot.gz HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
 192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
 EOF
@@ -470,6 +516,7 @@ check_output "write-json-to is not working" <<EOF
         "log_mark": 0,
         "log_comment": null,
         "log_tags": null,
+        "log_filters": null,
         "c_ip": "192.168.202.254",
         "cs_method": "GET",
         "cs_referer": "-",
@@ -490,6 +537,7 @@ check_output "write-json-to is not working" <<EOF
         "log_mark": 0,
         "log_comment": null,
         "log_tags": null,
+        "log_filters": null,
         "c_ip": "192.168.202.254",
         "cs_method": "GET",
         "cs_referer": "-",
@@ -510,6 +558,7 @@ check_output "write-json-to is not working" <<EOF
         "log_mark": 0,
         "log_comment": null,
         "log_tags": null,
+        "log_filters": null,
         "c_ip": "192.168.202.254",
         "cs_method": "GET",
         "cs_referer": "-",
@@ -525,6 +574,17 @@ check_output "write-json-to is not working" <<EOF
 EOF
 
 
+run_test ${lnav_test} -n \
+    -c ";select * from access_log" \
+    -c ':write-jsonlines-to -' \
+    ${test_dir}/logfile_access_log.0
+
+check_output "write-jsonlines-to is not working" <<EOF
+{"log_line":0,"log_part":null,"log_time":"2009-07-20 22:59:26.000","log_idle_msecs":0,"log_level":"info","log_mark":0,"log_comment":null,"log_tags":null,"log_filters":null,"c_ip":"192.168.202.254","cs_method":"GET","cs_referer":"-","cs_uri_query":null,"cs_uri_stem":"/vmw/cgi/tramp","cs_user_agent":"gPXE/0.9.7","cs_username":"-","cs_version":"HTTP/1.0","sc_bytes":134,"sc_status":200}
+{"log_line":1,"log_part":null,"log_time":"2009-07-20 22:59:29.000","log_idle_msecs":3000,"log_level":"error","log_mark":0,"log_comment":null,"log_tags":null,"log_filters":null,"c_ip":"192.168.202.254","cs_method":"GET","cs_referer":"-","cs_uri_query":null,"cs_uri_stem":"/vmw/vSphere/default/vmkboot.gz","cs_user_agent":"gPXE/0.9.7","cs_username":"-","cs_version":"HTTP/1.0","sc_bytes":46210,"sc_status":404}
+{"log_line":2,"log_part":null,"log_time":"2009-07-20 22:59:29.000","log_idle_msecs":0,"log_level":"info","log_mark":0,"log_comment":null,"log_tags":null,"log_filters":null,"c_ip":"192.168.202.254","cs_method":"GET","cs_referer":"-","cs_uri_query":null,"cs_uri_stem":"/vmw/vSphere/default/vmkernel.gz","cs_user_agent":"gPXE/0.9.7","cs_username":"-","cs_version":"HTTP/1.0","sc_bytes":78929,"sc_status":200}
+EOF
+
 # By setting the LNAVSECURE mode before executing the command, we will disable
 # the access to the write-json-to command and the output would just be the
 # actual display of select query rather than json output.
@@ -535,7 +595,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "We managed to bypass LNAVSECURE mode" <<EOF
-error: write-json-to -- unavailable in secure mode
+command-option:2: error: write-json-to -- unavailable in secure mode
 EOF
 unset LNAVSECURE
 
@@ -550,10 +610,12 @@ check_output "pipe-to is not working" <<EOF
 EOF
 
 run_test ${lnav_test} -n \
+    -c ":echo Hello, World!" \
     -c ":goto 2" \
     -c ":pipe-line-to sed -e 's/World!/Bork!/g' -e 's/2009//g'" \
     ${test_dir}/logfile_multiline.0
 check_output "pipe-line-to is not working" <<EOF
+Hello, World!
 -07-20 22:59:30,221:ERROR:Goodbye, Bork!
 EOF
 
@@ -732,7 +794,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "clear-highlight did not report an error?" <<EOF
-error: highlight does not exist
+command-option:1: error: highlight does not exist -- foobar
 EOF
 
 run_test ${lnav_test} -n \
@@ -762,6 +824,7 @@ run_test ${lnav_test} -n \
     -c ":shexec echo 'Jan  3 09:23:38 veridian automount[16442]: attempting to mount entry /auto/opt' >> logfile_rollover.1.live" \
     -c ":rebuild" \
     -c ":switch-to-view histogram" \
+    -c ":goto 0" \
     logfile_rollover.1.live
 
 check_output "rollover is not working with histogram" <<EOF
@@ -826,7 +889,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "bad zoom level is not rejected?" <<EOF
-error: invalid zoom level -- bad
+command-option:1: error: invalid zoom level -- bad
 EOF
 
 
