@@ -1,12 +1,12 @@
 #! /bin/bash
 
 if locale -a | grep fr_FR; then
-    cp ${srcdir}/logfile_syslog_fr.0 logfile_syslog_fr.0
-    touch -t 200711030923 logfile_syslog_fr.0
+    cp ${srcdir}/logfile_syslog_fr.0 logfile_syslog_fr_test.0
+    touch -t 200711030923 logfile_syslog_fr_test.0
     run_test env LC_ALL=fr_FR.UTF-8 ${lnav_test} -n \
         -c ";SELECT log_time FROM syslog_log" \
         -c ":write-csv-to -" \
-        logfile_syslog_fr.0
+        logfile_syslog_fr_test.0
 
     check_output "french locale is not recognized" <<EOF
 log_time
@@ -19,8 +19,10 @@ chmod ugo-r unreadable.log
 
 run_test ${lnav_test} -n unreadable.log
 
-sed -i "" -e "s|/.*/unreadable.log|unreadable.log|g" `test_err_filename`
+sed -e "s|/.*/unreadable.log|unreadable.log|g" `test_err_filename` \
+    > test_logfile.unreadable.out
 
+mv test_logfile.unreadable.out `test_err_filename`
 check_error_output "able to read an unreadable log file?" <<EOF
 error: Permission denied -- 'unreadable.log'
 EOF
@@ -63,9 +65,9 @@ run_test ./drive_logfile ${srcdir}/logfile_empty.0
 
 on_error_fail_with "Didn't handle empty log?"
 
-cp ${srcdir}/logfile_syslog.0 logfile_syslog.0
-touch -t 200711030923 logfile_syslog.0
-run_test ./drive_logfile -t -f syslog_log logfile_syslog.0
+cp ${srcdir}/logfile_syslog.0 logfile_syslog_test.0
+touch -t 200711030923 logfile_syslog_test.0
+run_test ./drive_logfile -t -f syslog_log logfile_syslog_test.0
 
 check_output "Syslog timestamp interpreted incorrectly?" <<EOF
 Nov 03 09:23:38 2007 -- 000
@@ -378,6 +380,28 @@ count(*)
       17
 EOF
 
+run_test ${lnav_test} -n \
+    ${srcdir}/logfile_syslog_with_header.0
+
+check_output "multi-pattern logs don't work?" <<EOF
+Header1: abc
+Header2: def
+Nov  3 09:23:38 veridian automount[7998]: lookup(file): lookup for foobar failed
+Nov  3 09:23:38 veridian automount[16442]: attempting to mount entry /auto/opt
+Nov  3 09:23:38 veridian automount[7999]: lookup(file): lookup for opt failed
+Nov  3 09:47:02 veridian sudo: timstack : TTY=pts/6 ; PWD=/auto/wstimstack/rpms/lbuild/test ; USER=root ; COMMAND=/usr/bin/tail /var/log/messages
+EOF
+
+run_test ${lnav_test} -n \
+    ${srcdir}/logfile_generic_with_header.0
+
+check_output "multi-pattern logs don't work?" <<EOF
+Header1: abc
+Header2: def
+2012-07-02 10:22:40,672:DEBUG:foo bar baz
+2014-10-08 16:56:38,344:WARN:foo bar baz
+EOF
+
 # XXX get this working...
 # run_test ${lnav_test} -n -I ${test_dir} <(cat ${srcdir}/logfile_access_log.0)
 #
