@@ -4,7 +4,7 @@ lnav_test="${top_builddir}/src/lnav-test"
 
 export HOME="./meta-sessions"
 rm -rf "./meta-sessions"
-mkdir -p $HOME
+mkdir -p $HOME/.config
 
 run_test ${lnav_test} -n \
     -c ":comment Hello, World!" \
@@ -21,6 +21,16 @@ check_output ":tag did not work?" <<EOF
 192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
 EOF
 
+if test ! -d meta-sessions/.config/lnav; then
+    echo "error: configuration not stored in .config/lnav?"
+    exit 1
+fi
+
+if test -d meta-sessions/.lnav; then
+    echo "error: configuration stored in .lnav?"
+    exit 1
+fi
+
 run_test ${lnav_test} -n \
     -c ":load-session" \
     -c ";UPDATE access_log SET log_mark = 1" \
@@ -112,10 +122,10 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "updating log_tags is not working?" <<EOF
-error:command-option:1:command-option:line 1
+command-option:1: error: command-option:line 1
   unexpected JSON value
   accepted paths --
-    # <tag> -- A tag for the log line
+     <tag> -- A tag for the log line
 EOF
 
 run_test ${lnav_test} -n \
@@ -124,7 +134,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_error_output "updating log_tags is not working?" <<EOF
-error:command-option:1:Value does not match pattern: ^#[^\s]+$
+command-option:1: error: Value does not match pattern: ^#[^\s]+$
 EOF
 
 run_test ${lnav_test} -n \
