@@ -64,7 +64,8 @@ int main(int argc, char *argv[])
 
     setenv("HOME", "/", 1);
 
-    vector<string> paths, errors;
+    vector<string> errors;
+    vector<filesystem::path> paths;
 
     load_config(paths, errors);
 
